@@ -120,7 +120,7 @@ int main(int argc, char **argv)
 	int lsockets[2], nlsockets = 0;
 	unsigned int dboptions = 0;
 #ifdef C_LINUX
-	struct stat sb;
+	STATBUF sb;
 #endif
 
     if(check_flevel())
@@ -221,6 +221,8 @@ int main(int argc, char **argv)
     logok = optget(opts, "LogClean")->enabled;
     logg_size = optget(opts, "LogFileMaxSize")->numarg;
     logg_verbose = mprintf_verbose = optget(opts, "LogVerbose")->enabled;
+    if (logg_size)
+        logg_rotate = optget(opts, "LogRotate")->enabled;
     mprintf_send_timeout = optget(opts, "SendBufTimeout")->numarg;
 
     do { /* logger initialized */
@@ -273,7 +275,7 @@ int main(int argc, char **argv)
 
 #ifdef C_LINUX
     procdev = 0;
-    if(stat("/proc", &sb) != -1 && !sb.st_size)
+    if(STAT("/proc", &sb) != -1 && !sb.st_size)
 	procdev = sb.st_dev;
 #endif
 
@@ -318,7 +320,7 @@ int main(int argc, char **argv)
 
 
     if(logg_size)
-	logg("#Log file size limited to %d bytes.\n", logg_size);
+	logg("#Log file size limited to %u bytes.\n", logg_size);
     else
 	logg("#Log file size limit disabled.\n");
 
@@ -499,6 +501,9 @@ int main(int argc, char **argv)
 	break;
     }
 
+    if (optget(opts, "DisableCertCheck")->enabled)
+        engine->dconf->pe |= PE_CONF_DISABLECERT;
+
     logg("#Loaded %u signatures.\n", sigs);
     if((ret = cl_engine_compile(engine)) != 0) {
 	logg("!Database initialization error: %s\n", cl_strerror(ret));
@@ -565,7 +570,12 @@ int main(int argc, char **argv)
 #ifdef C_BSD	    
 	/* workaround for OpenBSD bug, see https://wwws.clamav.net/bugzilla/show_bug.cgi?id=885 */
 	for(ret=0;ret<nlsockets;ret++) {
-		fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK);
+		if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) | O_NONBLOCK) == -1) {
+            logg("!fcntl for lsockets[] failed\n");
+            close(lsockets[ret]);
+            ret = 1;
+            break;
+        }
 	}
 #endif
 	gengine = engine;
@@ -578,7 +588,12 @@ int main(int argc, char **argv)
 	gengine = NULL;
 #ifdef C_BSD
 	for(ret=0;ret<nlsockets;ret++) {
-		fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK);
+		if (fcntl(lsockets[ret], F_SETFL, fcntl(lsockets[ret], F_GETFL) & ~O_NONBLOCK) == -1) {
+            logg("!fcntl for lsockets[] failed\n");
+            close(lsockets[ret]);
+            ret = 1;
+            break;
+        }
 	}
 #endif
 	if(!debug_mode)
