@@ -103,10 +103,10 @@ void hash_callback(int fd, unsigned long long size, const unsigned char *md5, co
 }
 
 #define BUFFSIZE 1024
-int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
+int scan_callback(STATBUF *sb, char *filename, const char *msg, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data)
 {
     struct scan_cb_data *scandata = data->data;
-    const char *virname;
+    const char *virname = NULL;
     const char **virpp = &virname;
     int ret;
     int type = scandata->type;
@@ -225,8 +225,7 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
 	return CL_SUCCESS;
     }
 
-    thrmgr_setactivetask(filename,
-			 type == TYPE_MULTISCAN ? "MULTISCANFILE" : NULL);
+    thrmgr_setactivetask(filename, NULL);
     context.filename = filename;
     context.virsize = 0;
     ret = cl_scanfile_callback(filename, virpp, &scandata->scanned, scandata->engine, scandata->options, &context);
@@ -234,22 +233,27 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
 
     if (scandata->options & CL_SCAN_ALLMATCHES) {
 	virpp = (const char **)*virpp; /* temp hack for scanall mode until api augmentation */
-	virname = virpp[0];
+	if (virpp) virname = virpp[0];
     }
 
     if (thrmgr_group_need_terminate(scandata->conn->group)) {
 	free(filename);
-	if (ret == CL_VIRUS && scandata->options & CL_SCAN_ALLMATCHES)
+	if ((scandata->options & CL_SCAN_ALLMATCHES) && (virpp != &virname))
 	    free((void *)virpp);
 	logg("*Client disconnected while scanjob was active\n");
 	return ret == CL_ETIMEOUT ? ret : CL_BREAK;
     }
 
+    if ((ret == CL_VIRUS) && (virname == NULL)) {
+        logg("*%s: reported CL_VIRUS but no virname returned!\n", filename);
+        ret = CL_EMEM;
+    }
+
     if (ret == CL_VIRUS) {
 	scandata->infected++;
 	if (conn_reply_virus(scandata->conn, filename, virname) == -1) {
 	    free(filename);
-	    if (scandata->options & CL_SCAN_ALLMATCHES)
+	    if((scandata->options & CL_SCAN_ALLMATCHES) && (virpp != &virname))
 		free((void *)virpp);
 	    return CL_ETIMEOUT;
 	}
@@ -258,7 +262,8 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
 	    while (NULL != virpp[i])
 		if (conn_reply_virus(scandata->conn, filename, virpp[i++]) == -1) {
 		    free(filename);
-		    free((void *)virpp);
+		    if (virpp != &virname)
+			free((void *)virpp);
 		    return CL_ETIMEOUT;
 		}
 	}
@@ -277,6 +282,8 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     } else if (ret != CL_CLEAN) {
 	scandata->errors++;
 	if (conn_reply(scandata->conn, filename, cl_strerror(ret), "ERROR") == -1) {
+	    if((scandata->options & CL_SCAN_ALLMATCHES) && (virpp != &virname))
+		free((void *)virpp);
 	    free(filename);
 	    return CL_ETIMEOUT;
 	}
@@ -286,7 +293,7 @@ int scan_callback(struct stat *sb, char *filename, const char *msg, enum cli_ftw
     }
 
     free(filename);
-    if (ret == CL_VIRUS && scandata->options & CL_SCAN_ALLMATCHES)
+    if((scandata->options & CL_SCAN_ALLMATCHES) && (virpp != &virname))
 	free((void *)virpp);
     if(ret == CL_EMEM) /* stop scanning */
 	return ret;
@@ -304,7 +311,7 @@ int scan_pathchk(const char *path, struct cli_ftw_cbdata *data)
 {
 	struct scan_cb_data *scandata = data->data;
 	const struct optstruct *opt;
-	struct stat statbuf;
+	STATBUF statbuf;
 
     if((opt = optget(scandata->opts, "ExcludePath"))->enabled) {
 	while(opt) {
@@ -318,7 +325,7 @@ int scan_pathchk(const char *path, struct cli_ftw_cbdata *data)
     }
 
     if(!optget(scandata->opts, "CrossFilesystems")->enabled) {
-	if(stat(path, &statbuf) == 0) {
+	if(STAT(path, &statbuf) == 0) {
 	    if(statbuf.st_dev != scandata->dev) {
 		if(scandata->type != TYPE_MULTISCAN)
 		    conn_reply_single(scandata->conn, path, "Excluded (another filesystem)");
@@ -336,7 +343,7 @@ int scanfd(const client_conn_t *conn, unsigned long int *scanned,
 {
     int ret, fd = conn->scanfd;
 	const char *virname;
-	struct stat statbuf;
+	STATBUF statbuf;
 	struct cb_context context;
 	char fdstr[32];
 	const char*reply_fdstr;
@@ -353,7 +360,7 @@ int scanfd(const client_conn_t *conn, unsigned long int *scanned,
 	    snprintf(fdstr, sizeof(fdstr), "fd[%d]", fd);
 	    reply_fdstr = fdstr;
 	}
-	if(fstat(fd, &statbuf) == -1 || !S_ISREG(statbuf.st_mode)) {
+	if(FSTAT(fd, &statbuf) == -1 || !S_ISREG(statbuf.st_mode)) {
 		logg("%s: Not a regular file. ERROR\n", fdstr);
 		if (conn_reply(conn, reply_fdstr, "Not a regular file", "ERROR") == -1)
 		    return CL_ETIMEOUT;
@@ -428,7 +435,7 @@ int scanstream(int odesc, unsigned long int *scanned, const struct cl_engine *en
 	if((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
 	    continue;
 
-	if(bind(sockfd, (struct sockaddr *) &server, sizeof(struct sockaddr_in)) == -1)
+	if(bind(sockfd, (struct sockaddr *) &server, (socklen_t)sizeof(struct sockaddr_in)) == -1)
 	    closesocket(sockfd);
 	else {
 	    bound = 1;
@@ -440,13 +447,16 @@ int scanstream(int odesc, unsigned long int *scanned, const struct cl_engine *en
     timeout = optget(opts, "ReadTimeout")->numarg;
     firsttimeout = optget(opts, "CommandReadTimeout")->numarg;
 
-    if(!bound && !portscan) {
+    if(!bound) {
 	logg("!ScanStream: Can't find any free port.\n");
 	mdprintf(odesc, "Can't find any free port. ERROR%c", term);
-	closesocket(sockfd);
 	return -1;
     } else {
-	listen(sockfd, 1);
+	if (listen(sockfd, 1) == -1) {
+        logg("!ScanStream: listen() error on socket. Error returned is %s.\n", strerror(errno));
+        closesocket(sockfd);
+        return -1;
+    }
 	if(mdprintf(odesc, "PORT %u%c", port, term) <= 0) {
 	    logg("!ScanStream: error transmitting port.\n");
 	    closesocket(sockfd);
@@ -464,7 +474,7 @@ int scanstream(int odesc, unsigned long int *scanned, const struct cl_engine *en
     }
 
     addrlen = sizeof(peer);
-    if((acceptd = accept(sockfd, (struct sockaddr *) &peer, &addrlen)) == -1) {
+    if((acceptd = accept(sockfd, (struct sockaddr *) &peer, (socklen_t *)&addrlen)) == -1) {
 	closesocket(sockfd);
 	mdprintf(odesc, "accept() ERROR%c", term);
 	logg("!ScanStream %u: accept() failed.\n", port);
