@@ -192,7 +192,7 @@ int command(client_conn_t *conn, int *virus)
     struct cl_engine *engine = conn->engine;
     unsigned int options = conn->options;
     const struct optstruct *opts = conn->opts;
-    int type = -1; /* TODO: make this enum */
+    enum scan_type type = TYPE_INIT;
     int maxdirrec;
     int ret = 0;
     int flags = CLI_FTW_STD;
@@ -200,7 +200,7 @@ int command(client_conn_t *conn, int *virus)
     struct scan_cb_data scandata;
     struct cli_ftw_cbdata data;
     unsigned ok, error, total;
-    struct stat sb;
+    STATBUF sb;
     jobgroup_t *group = NULL;
 
     if (thrmgr_group_need_terminate(conn->group)) {
@@ -236,7 +236,7 @@ int command(client_conn_t *conn, int *virus)
 	    int multiscan, max, alive;
 
 	    /* use MULTISCAN only for directories (bb #1869) */
-	    if (stat(conn->filename, &sb) == 0 &&
+	    if (STAT(conn->filename, &sb) == 0 &&
 		!S_ISDIR(sb.st_mode)) {
 		thrmgr_setactivetask(NULL, "CONTSCAN");
 		type = TYPE_CONTSCAN;
@@ -266,8 +266,12 @@ int command(client_conn_t *conn, int *virus)
 	    thrmgr_setactivetask(NULL, "MULTISCAN");
 	    type = TYPE_MULTISCAN;
 	    scandata.group = group = thrmgr_group_new();
-	    if (!group)
-		return CL_EMEM;
+	    if (!group) {
+	      if(optget(opts, "ExitOnOOM")->enabled)
+		return -1;
+	      else
+		return 1;
+	    }
 	    break;
 	    }
 	case COMMAND_MULTISCANFILE:
@@ -281,152 +285,171 @@ int command(client_conn_t *conn, int *virus)
 	    *virus = scandata.infected;
 	    if (ret == CL_BREAK) {
 		thrmgr_group_terminate(conn->group);
-		return CL_BREAK;
+		return 1;
 	    }
 	    return scandata.errors > 0 ? scandata.errors : 0;
 	case COMMAND_FILDES:
 	    thrmgr_setactivetask(NULL, "FILDES");
 #ifdef HAVE_FD_PASSING
-	    if (conn->scanfd == -1)
+	    if (conn->scanfd == -1) {
 		conn_reply_error(conn, "FILDES: didn't receive file descriptor.");
+		return 1;
+	    }
 	    else {
 		ret = scanfd(conn, NULL, engine, options, opts, desc, 0);
 		if (ret == CL_VIRUS) {
 		    *virus = 1;
+		    ret = 0;
 		} else if (ret == CL_EMEM) {
 		    if(optget(opts, "ExitOnOOM")->enabled)
 			ret = -1;
+		    else
+		        ret = 1;
 		} else if (ret == CL_ETIMEOUT) {
 			thrmgr_group_terminate(conn->group);
 			ret = 1;
 		} else
 		    ret = 0;
+		logg("$Closed fd %d\n", conn->scanfd);
+		close(conn->scanfd);
 	    }
-	    logg("$Closed fd %d\n", conn->scanfd);
-	    close(conn->scanfd);
 	    return ret;
 #else
-	    conn_reply_error(conn, "FILDES support not compiled in.");
-	    close(conn->scanfd);
-	    return 0;
-#endif
-	case COMMAND_STATS:
-	    thrmgr_setactivetask(NULL, "STATS");
-	    if (conn->group)
-		mdprintf(desc, "%u: ", conn->id);
-	    thrmgr_printstats(desc, conn->term);
-	    return 0;
-	case COMMAND_STREAM:
-	    thrmgr_setactivetask(NULL, "STREAM");
-	    ret = scanstream(desc, NULL, engine, options, opts, conn->term);
-	    if (ret == CL_VIRUS)
-		*virus = 1;
-	    if (ret == CL_EMEM) {
-		if(optget(opts, "ExitOnOOM")->enabled)
-		    return -1;
+	     conn_reply_error(conn, "FILDES support not compiled in.");
+	     close(conn->scanfd);
+	     return 0;
+ #endif
+	 case COMMAND_STATS:
+	     thrmgr_setactivetask(NULL, "STATS");
+	     if (conn->group)
+		 mdprintf(desc, "%u: ", conn->id);
+	     thrmgr_printstats(desc, conn->term);
+	     return 0;
+	 case COMMAND_STREAM:
+	     thrmgr_setactivetask(NULL, "STREAM");
+	     ret = scanstream(desc, NULL, engine, options, opts, conn->term);
+	     if (ret == CL_VIRUS)
+		 *virus = 1;
+	     if (ret == CL_EMEM) {
+		 if(optget(opts, "ExitOnOOM")->enabled)
+		     return -1;
+		 else
+		     return 1;
+	     }
+	     return 0;
+	 case COMMAND_INSTREAMSCAN:
+	     thrmgr_setactivetask(NULL, "INSTREAM");
+	     ret = scanfd(conn, NULL, engine, options, opts, desc, 1);
+	     if (ret == CL_VIRUS) {
+		 *virus = 1;
+		 ret = 0;
+	     } else if (ret == CL_EMEM) {
+		 if(optget(opts, "ExitOnOOM")->enabled)
+		     ret = -1;
+		 else
+		     ret = 1;
+	     } else if (ret == CL_ETIMEOUT) {
+		 thrmgr_group_terminate(conn->group);
+		 ret = 1;
+	     } else
+		 ret = 0;
+	     if (ftruncate(conn->scanfd, 0) == -1) {
+		 /* not serious, we're going to close it and unlink it anyway */
+		 logg("*ftruncate failed: %d\n", errno);
+	     }
+	     close(conn->scanfd);
+	     conn->scanfd = -1;
+	     cli_unlink(conn->filename);
+	     return ret;
+	 case COMMAND_ALLMATCHSCAN:
+	     if (!optget(opts, "AllowAllMatchScan")->enabled) {
+		logg("$Rejecting ALLMATCHSCAN command.\n");
+		conn_reply(conn, conn->filename, "ALLMATCHSCAN command disabled by clamd configuration.", "ERROR");
+		return 1;
 	    }
-	    return 0;
-	case COMMAND_INSTREAMSCAN:
-	    thrmgr_setactivetask(NULL, "INSTREAM");
-	    ret = scanfd(conn, NULL, engine, options, opts, desc, 1);
-	    if (ret == CL_VIRUS) {
-		*virus = 1;
-	    } else if (ret == CL_EMEM) {
-		if(optget(opts, "ExitOnOOM")->enabled)
-		    ret = -1;
-	    } else if (ret == CL_ETIMEOUT) {
-		thrmgr_group_terminate(conn->group);
-		ret = 1;
-	    } else
-		ret = 0;
-	    if (ftruncate(conn->scanfd, 0) == -1) {
-		/* not serious, we're going to close it and unlink it anyway */
-		logg("*ftruncate failed: %d\n", errno);
-	    }
-	    close(conn->scanfd);
-	    conn->scanfd = -1;
-	    cli_unlink(conn->filename);
-	    return ret;
-	case COMMAND_ALLMATCHSCAN:
-	     thrmgr_setactivetask(NULL, "ALLMATCHSCAN");
-	     scandata.options |= CL_SCAN_ALLMATCHES;
-	     type = TYPE_SCAN;
-	     break;default:
-	    logg("!Invalid command distpached: %d\n", conn->cmdtype);
-	    return 1;
-    }
-
-    scandata.type = type;
-    maxdirrec = optget(opts, "MaxDirectoryRecursion")->numarg;
-    if (optget(opts, "FollowDirectorySymlinks")->enabled)
-	flags |= CLI_FTW_FOLLOW_DIR_SYMLINK;
-    if (optget(opts, "FollowFileSymlinks")->enabled)
-	flags |= CLI_FTW_FOLLOW_FILE_SYMLINK;
-
-    if(!optget(opts, "CrossFilesystems")->enabled)
-	if(stat(conn->filename, &sb) == 0)
-	    scandata.dev = sb.st_dev;
-
-    ret = cli_ftw(conn->filename, flags,  maxdirrec ? maxdirrec : INT_MAX, scan_callback, &data, scan_pathchk);
-    if (ret == CL_EMEM)
-	if(optget(opts, "ExitOnOOM")->enabled)
-	    return -1;
-    if (scandata.group && conn->cmdtype == COMMAND_MULTISCAN) {
-	thrmgr_group_waitforall(group, &ok, &error, &total);
-	pthread_mutex_lock(&conn->thrpool->pool_mutex);
-	conn->thrpool->thr_multiscan--;
-	pthread_mutex_unlock(&conn->thrpool->pool_mutex);
-    } else {
-	error = scandata.errors;
-	total = scandata.total;
-	ok = total - error - scandata.infected;
-    }
-
-    if (ok + error == total && (error != total)) {
-	if (conn_reply_single(conn, conn->filename, "OK") == -1)
-	    ret = CL_ETIMEOUT;
-    }
-    *virus = total - (ok + error);
-
-    if (ret == CL_ETIMEOUT)
-	thrmgr_group_terminate(conn->group);
-    return error;
-}
-
-static int dispatch_command(client_conn_t *conn, enum commands cmd, const char *argument)
-{
-    int ret = 0;
-    int bulk;
-    client_conn_t *dup_conn = (client_conn_t *) malloc(sizeof(struct client_conn_tag));
-
-    if(!dup_conn) {
-	logg("!Can't allocate memory for client_conn\n");
-	return -1;
-    }
-    memcpy(dup_conn, conn, sizeof(*conn));
-    dup_conn->cmdtype = cmd;
-    if(cl_engine_addref(dup_conn->engine)) {
-	logg("!cl_engine_addref() failed\n");
-	free(dup_conn);
-	return -1;
-    }
-    dup_conn->scanfd = -1;
-    bulk = 1;
-    switch (cmd) {
-	case COMMAND_FILDES:
-	    if (conn->scanfd == -1) {
-		conn_reply_error(dup_conn, "No file descriptor received.");
-		ret = 1;
-	    }
-	    dup_conn->scanfd = conn->scanfd;
-	    /* consume FD */
-	    conn->scanfd = -1;
+	    thrmgr_setactivetask(NULL, "ALLMATCHSCAN");
+	    scandata.options |= CL_SCAN_ALLMATCHES;
+	    type = TYPE_SCAN;
 	    break;
-	case COMMAND_SCAN:
-	case COMMAND_CONTSCAN:
-	case COMMAND_MULTISCAN:
-        case COMMAND_ALLMATCHSCAN:
-	    dup_conn->filename = strdup(argument);
+	 default:
+	    logg("!Invalid command dispatched: %d\n", conn->cmdtype);
+	    return 1;
+     }
+
+     scandata.type = type;
+     maxdirrec = optget(opts, "MaxDirectoryRecursion")->numarg;
+     if (optget(opts, "FollowDirectorySymlinks")->enabled)
+	 flags |= CLI_FTW_FOLLOW_DIR_SYMLINK;
+     if (optget(opts, "FollowFileSymlinks")->enabled)
+	 flags |= CLI_FTW_FOLLOW_FILE_SYMLINK;
+
+     if(!optget(opts, "CrossFilesystems")->enabled)
+	 if(STAT(conn->filename, &sb) == 0)
+	     scandata.dev = sb.st_dev;
+
+     ret = cli_ftw(conn->filename, flags,  maxdirrec ? maxdirrec : INT_MAX, scan_callback, &data, scan_pathchk);
+     if (ret == CL_EMEM) {
+	 if(optget(opts, "ExitOnOOM")->enabled)
+	     return -1;
+	 else
+	     return 1;
+     }
+     if (scandata.group && type == TYPE_MULTISCAN) {
+	 thrmgr_group_waitforall(group, &ok, &error, &total);
+	 pthread_mutex_lock(&conn->thrpool->pool_mutex);
+	 conn->thrpool->thr_multiscan--;
+	 pthread_mutex_unlock(&conn->thrpool->pool_mutex);
+     } else {
+	 error = scandata.errors;
+	 total = scandata.total;
+	 ok = total - error - scandata.infected;
+     }
+
+     if (ok + error == total && (error != total)) {
+	 if (conn_reply_single(conn, conn->filename, "OK") == -1)
+	     ret = CL_ETIMEOUT;
+     }
+     *virus = total - (ok + error);
+
+     if (ret == CL_ETIMEOUT)
+	 thrmgr_group_terminate(conn->group);
+     return error;
+ }
+
+ static int dispatch_command(client_conn_t *conn, enum commands cmd, const char *argument)
+ {
+     int ret = 0;
+     int bulk;
+     client_conn_t *dup_conn = (client_conn_t *) malloc(sizeof(struct client_conn_tag));
+
+     if(!dup_conn) {
+	 logg("!Can't allocate memory for client_conn\n");
+	 return -1;
+     }
+     memcpy(dup_conn, conn, sizeof(*conn));
+     dup_conn->cmdtype = cmd;
+     if(cl_engine_addref(dup_conn->engine)) {
+	 logg("!cl_engine_addref() failed\n");
+	 free(dup_conn);
+	 return -1;
+     }
+     dup_conn->scanfd = -1;
+     bulk = 1;
+     switch (cmd) {
+	 case COMMAND_FILDES:
+	     if (conn->scanfd == -1) {
+		 conn_reply_error(dup_conn, "No file descriptor received.");
+		 ret = 1;
+	     }
+	     dup_conn->scanfd = conn->scanfd;
+	     /* consume FD */
+	     conn->scanfd = -1;
+	     break;
+	 case COMMAND_SCAN:
+	 case COMMAND_CONTSCAN:
+	 case COMMAND_MULTISCAN:
+	 case COMMAND_ALLMATCHSCAN:
+	    dup_conn->filename = cli_strdup_to_utf8(argument);
 	    if (!dup_conn->filename) {
 		logg("!Failed to allocate memory for filename\n");
 		ret = -1;
