@@ -188,14 +188,15 @@ static int ftw_chkpath(const char *path, struct cli_ftw_cbdata *data)
 
 /* Sends a proper scan request to clamd and parses its replies
  * This is used only in non IDSESSION mode
- * Returns the number of infected files or -1 on error */
+ * Returns the number of infected files or -1 on error
+ * NOTE: filename may be NULL for STREAM scantype. */
 int dsresult(int sockd, int scantype, const char *filename, int *printok, int *errors) {
     int infected = 0, len = 0, beenthere = 0;
     char *bol, *eol;
     struct RCVLN rcv;
-    struct stat sb;
+    STATBUF sb;
 
-    if(chkpath(filename))
+    if(filename && chkpath(filename))
 	return 0;
     recvlninit(&rcv, sockd);
 
@@ -221,10 +222,12 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
     break;
 
     case STREAM:
+        /* NULL filename safe in send_stream() */
 	len = send_stream(sockd, filename);
 	break;
 #ifdef HAVE_FD_PASSING
     case FILDES:
+        /* NULL filename safe in send_fdpass() */
 	len = send_fdpass(sockd, filename);
 	break;
 #endif
@@ -289,9 +292,17 @@ int dsresult(int sockd, int scantype, const char *filename, int *printok, int *e
 	}
     }
     if(!beenthere) {
-	stat(filename, &sb);
+        if (!filename) {
+	    logg("STDIN: noreply from clamd\n.");
+	    return -1;
+	}
+        if(STAT(filename, &sb) == -1) {
+	    logg("~%s: stat() failed with %s, clamd may not be responding\n",
+		 filename, strerror(errno));
+	    return -1;
+	}
 	if(!S_ISDIR(sb.st_mode)) {
-	    logg("~%s: no reply from clamd\n", filename ? filename : "STDIN");
+	    logg("~%s: no reply from clamd\n", filename);
 	    return -1;
 	}
     }
@@ -309,7 +320,7 @@ struct client_serial_data {
 
 /* FTW callback for scanning in non IDSESSION mode
  * Returns SUCCESS or BREAK on success, CL_EXXX on error */
-static int serial_callback(struct stat *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data) {
+static int serial_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data) {
     struct client_serial_data *c = (struct client_serial_data *)data->data;
     int sockd, ret;
     const char *f = filename;
@@ -331,7 +342,7 @@ static int serial_callback(struct stat *sb, char *filename, const char *path, en
     case warning_skipped_link:
 	return CL_SUCCESS;
     case warning_skipped_special:
-	logg("!%s: Not supported file type\n", path);
+	logg("^%s: Not supported file type\n", path);
 	c->errors++;
 	return CL_SUCCESS;
     case visit_directory_toplev:
@@ -463,7 +474,7 @@ static int dspresult(struct client_parallel_data *c) {
 
 /* FTW callback for scanning in IDSESSION mode
  * Returns SUCCESS on success, CL_EXXX or BREAK on error */
-static int parallel_callback(struct stat *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data) {
+static int parallel_callback(STATBUF *sb, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data) {
     struct client_parallel_data *c = (struct client_parallel_data *)data->data;
     struct SCANID *cid;
     int res = CL_CLEAN;
@@ -484,7 +495,7 @@ static int parallel_callback(struct stat *sb, char *filename, const char *path,
 	logg("^Directory recursion limit reached\n");
 	return CL_SUCCESS;
     case warning_skipped_special:
-	logg("!%s: Not supported file type\n", path);
+	logg("^%s: Not supported file type\n", path);
 	c->errors++;
     case warning_skipped_link:
     case visit_directory_toplev:
