@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *  Copyright (C) 2007-2012 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -71,7 +71,7 @@ dev_t procdev;
 /* FIXME: If possible, handle users correctly */
 static int checkaccess(const char *path, const char *username, int mode)
 {
-    return !_access(path, mode);
+    return !access(path, mode);
 }
 #else
 static int checkaccess(const char *path, const char *username, int mode)
@@ -120,13 +120,106 @@ static int checkaccess(const char *path, const char *username, int mode)
 }
 #endif
 
+struct metachain {
+    char **chains;
+    unsigned lastadd;
+    unsigned lastvir;
+    unsigned level;
+    unsigned n;
+};
+
+static cl_error_t pre(int fd, const char *type, void *context)
+{
+    struct metachain *c = context;
+    if (c) {
+	c->level++;
+    }
+    return CL_CLEAN;
+}
+
+static int print_chain(struct metachain *c, char *str, unsigned len)
+{
+    unsigned i;
+    unsigned na = 0;
+    for (i=0;i<c->n-1;i++) {
+	unsigned int n = strlen(c->chains[i]);
+	if (na)
+	    str[na++] = '!';
+	if (n + na + 2 > len)
+	    break;
+	memcpy(str + na, c->chains[i], n);
+	na += n;
+    }
+    str[na] = '\0';
+    str[len-1] = '\0';
+    return i == c->n-1 ? 0 : 1;
+}
+
+static cl_error_t post(int fd, int result, const char *virname, void *context)
+{
+    struct metachain *c = context;
+    if (c && c->n) {
+	char str[128];
+	int toolong = print_chain(c, str, sizeof(str));
+	if (c->level == c->lastadd && !virname)
+	    free(c->chains[--c->n]);
+	if (virname && !c->lastvir)
+	    c->lastvir = c->level;
+    }
+    if (c)
+	c->level--;
+    return CL_CLEAN;
+}
+
+static cl_error_t meta(const char* container_type, unsigned long fsize_container, const char *filename,
+		       unsigned long fsize_real,  int is_encrypted, unsigned int filepos_container, void *context)
+{
+    int na = 0;
+    char prev[128];
+    struct metachain *c = context;
+    const char *type = !strncmp(container_type,"CL_TYPE_",8) ? container_type + 8 : container_type;
+    unsigned n = strlen(type) + 1 + strlen(filename) + 1;
+    char *chain;
+    char **chains;
+    int toolong;
+
+    if (!c)
+	return CL_CLEAN;
+    chain = malloc(n);
+    if (!chain)
+	return CL_CLEAN;
+    if (!strcmp(type, "ANY"))
+	snprintf(chain, n,"%s", filename);
+    else
+	snprintf(chain, n,"%s:%s", type, filename);
+    if (c->lastadd != c->level) {
+	n = c->n + 1;
+	chains = realloc(c->chains, n * sizeof(*chains));
+	if (!chains) {
+	    free(chain);
+	    return CL_CLEAN;
+	}
+	c->chains = chains;
+	c->n = n;
+	c->lastadd = c->level;
+    } else {
+	free(c->chains[c->n-1]);
+    }
+    c->chains[c->n-1] = chain;
+    toolong = print_chain(c, prev, sizeof(prev));
+    logg("*Scanning %s%s!%s\n", prev,toolong ? "..." : "", chain);
+    return CL_CLEAN;
+}
+
 static void scanfile(const char *filename, struct cl_engine *engine, const struct optstruct *opts, unsigned int options)
 {
-	int ret = 0, fd, included, printclean = 1;
+	int ret = 0, fd, included;
+	unsigned i;
 	const struct optstruct *opt;
 	const char *virname;
-        const char **virpp = &virname; 
-	struct stat sb;
+	const char **virpp = &virname;
+	STATBUF sb;
+	struct metachain chain;
 
     if((opt = optget(opts, "exclude"))->enabled) {
 	while(opt) {
@@ -156,7 +249,7 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
     }
 
     /* argh, don't scan /proc files */
-    if(stat(filename, &sb) != -1) {
+    if(STAT(filename, &sb) != -1) {
 #ifdef C_LINUX
 	if(procdev && sb.st_dev == procdev) {
 	    if(!printinfected)
@@ -182,6 +275,14 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
 	}
 #endif
 
+    memset(&chain, 0, sizeof(chain));
+    if(optget(opts, "archive-verbose")->enabled) {
+	chain.chains = malloc(sizeof(*chain.chains));
+	if (chain.chains) {
+	    chain.chains[0] = strdup(filename);
+	    chain.n = 1;
+	}
+    }
     logg("*Scanning %s\n", filename);
 
     if((fd = safe_open(filename, O_RDONLY|O_BINARY)) == -1) {
@@ -190,10 +291,19 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
 	return;
     }
 
-    if((ret = cl_scandesc(fd, &virname, &info.blocks, engine, options)) == CL_VIRUS) {
-        if (options & CL_SCAN_ALLMATCHES) {                                                                               
+
+    if((ret = cl_scandesc_callback(fd, virpp, &info.blocks, engine, options, &chain)) == CL_VIRUS) {
+	if(optget(opts, "archive-verbose")->enabled) {
+	    if (chain.n > 1) {
+		char str[128];
+		int toolong = print_chain(&chain, str, sizeof(str));
+		logg("~%s%s!(%d)%s: %s FOUND\n", str, toolong ? "..." : "", chain.lastvir-1, chain.chains[chain.n-1], virname);
+	    } else if (chain.lastvir)
+		logg("~%s!(%d): %s FOUND\n", filename, chain.lastvir-1, virname);
+	}
+	if (options & CL_SCAN_ALLMATCHES) {
 	    int i = 0;
-	    virpp = (const char **)*virpp; /* temp hack for allscan until API enhancement */
+	    virpp = (const char **)*virpp; /* horrible */
 	    virname = virpp[0];
 	    while (virpp[i])
 		logg("~%s: %s FOUND\n", filename, virpp[i++]);
@@ -217,6 +327,9 @@ static void scanfile(const char *filename, struct cl_engine *engine, const struc
 	info.errors++;
     }
 
+    for (i=0;i<chain.n;i++)
+	free(chain.chains[i]);
+    free(chain.chains);
     close(fd);
 
     if(ret == CL_VIRUS && action)
@@ -227,7 +340,7 @@ static void scandirs(const char *dirname, struct cl_engine *engine, const struct
 {
 	DIR *dd;
 	struct dirent *dent;
-	struct stat sb;
+	STATBUF sb;
 	char *fname;
 	int included;
 	const struct optstruct *opt;
@@ -271,18 +384,22 @@ static void scandirs(const char *dirname, struct cl_engine *engine, const struct
 	info.dirs++;
 	depth++;
 	while((dent = readdir(dd))) {
-	    if(dent->d_ino)
-	    {
+	    if(dent->d_ino) {
 		if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
 		    /* build the full name */
 		    fname = malloc(strlen(dirname) + strlen(dent->d_name) + 2);
+		    if (fname == NULL) { /* oops, malloc() failed, print warning and return */
+			logg("!scandirs: Memory allocation failed for fname\n");
+			break;
+		    }
+
 		    if(!strcmp(dirname, PATHSEP))
 			sprintf(fname, PATHSEP"%s", dent->d_name);
 		    else
 			sprintf(fname, "%s"PATHSEP"%s", dirname, dent->d_name);
 
 		    /* stat the file */
-		    if(lstat(fname, &sb) != -1) {
+		    if(LSTAT(fname, &sb) != -1) {
 			if(!optget(opts, "cross-fs")->enabled) {
 			    if(sb.st_dev != dev) {
 				if(!printinfected)
@@ -295,7 +412,7 @@ static void scandirs(const char *dirname, struct cl_engine *engine, const struct
 			    if(dirlnk != 2 && filelnk != 2) {
 				if(!printinfected)
 				    logg("%s: Symbolic link\n", fname);
-			    } else if(stat(fname, &sb) != -1) {
+			    } else if(STAT(fname, &sb) != -1) {
 				if(S_ISREG(sb.st_mode) && filelnk == 2) {
 				    scanfile(fname, engine, opts, options);
 				} else if(S_ISDIR(sb.st_mode) && dirlnk == 2) {
@@ -346,7 +463,10 @@ static int scanstdin(const struct cl_engine *engine, const struct optstruct *opt
 	return 2;
     }
 
-    file = cli_gentemp(tmpdir);
+    if(!(file = cli_gentemp(tmpdir))) {
+	logg("!Can't generate tempfile name\n");
+	return 2;
+    }
 
     if(!(fs = fopen(file, "wb"))) {
 	logg("!Can't open %s for writing\n", file);
@@ -405,7 +525,7 @@ int scanmanager(const struct optstruct *opts)
 	int ret = 0, i;
 	unsigned int options = 0, dboptions = 0, dirlnk = 1, filelnk = 1;
 	struct cl_engine *engine;
-	struct stat sb;
+	STATBUF sb;
 	char *file, cwd[1024], *pua_cats = NULL;
 	const char *filename;
 	const struct optstruct *opt;
@@ -479,6 +599,7 @@ int scanmanager(const struct optstruct *opts)
 	    while(opt) {
 		if(!(pua_cats = realloc(pua_cats, i + strlen(opt->strarg) + 3))) {
 		    logg("!Can't allocate memory for pua_cats\n");
+		    cl_engine_free(engine);
 		    return 2;
 		}
 		sprintf(pua_cats + i, ".%s", opt->strarg);
@@ -513,6 +634,9 @@ int scanmanager(const struct optstruct *opts)
     if(optget(opts, "bytecode-unsigned")->enabled)
 	dboptions |= CL_DB_BYTECODE_UNSIGNED;
 
+    if(optget(opts, "bytecode-statistics")->enabled)
+	dboptions |= CL_DB_BYTECODE_STATS;
+
     if((opt = optget(opts,"bytecode-timeout"))->enabled)
 	cl_engine_set_num(engine, CL_ENGINE_BYTECODE_TIMEOUT, opt->numarg);
     if((opt = optget(opts,"bytecode-mode"))->enabled) {
@@ -563,6 +687,18 @@ int scanmanager(const struct optstruct *opts)
 	return 2;
     }
 
+    if(optget(opts, "archive-verbose")->enabled) {
+	cl_engine_set_clcb_meta(engine, meta);
+	cl_engine_set_clcb_pre_cache(engine, pre);
+	cl_engine_set_clcb_post_scan(engine, post);
+    }
+
+    if (optget(opts, "nocerts")->enabled)
+        engine->dconf->pe |= PE_CONF_DISABLECERT;
+
+    if (optget(opts, "dumpcerts")->enabled)
+        engine->dconf->pe |= PE_CONF_DUMPCERT;
+
     /* set limits */
 
     if((opt = optget(opts, "max-scansize"))->active) {
@@ -608,6 +744,48 @@ int scanmanager(const struct optstruct *opts)
 	}
     }
 
+    /* Engine max sizes */
+
+    if((opt = optget(opts, "max-embeddedpe"))->active) {
+	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_EMBEDDEDPE, opt->numarg))) {
+	    logg("!cli_engine_set_num(CL_ENGINE_MAX_EMBEDDEDPE) failed: %s\n", cl_strerror(ret));
+	    cl_engine_free(engine);
+	    return 2;
+	}
+    }
+
+    if((opt = optget(opts, "max-htmlnormalize"))->active) {
+	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_HTMLNORMALIZE, opt->numarg))) {
+	    logg("!cli_engine_set_num(CL_ENGINE_MAX_HTMLNORMALIZE) failed: %s\n", cl_strerror(ret));
+	    cl_engine_free(engine);
+	    return 2;
+	}
+    }
+
+    if((opt = optget(opts, "max-htmlnotags"))->active) {
+	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_HTMLNOTAGS, opt->numarg))) {
+	    logg("!cli_engine_set_num(CL_ENGINE_MAX_HTMLNOTAGS) failed: %s\n", cl_strerror(ret));
+	    cl_engine_free(engine);
+	    return 2;
+	}
+    }
+
+    if((opt = optget(opts, "max-scriptnormalize"))->active) {
+	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_SCRIPTNORMALIZE, opt->numarg))) {
+	    logg("!cli_engine_set_num(CL_ENGINE_MAX_SCRIPTNORMALIZE) failed: %s\n", cl_strerror(ret));
+	    cl_engine_free(engine);
+	    return 2;
+	}
+    }
+
+    if((opt = optget(opts, "max-ziptypercg"))->active) {
+	if((ret = cl_engine_set_num(engine, CL_ENGINE_MAX_ZIPTYPERCG, opt->numarg))) {
+	    logg("!cli_engine_set_num(CL_ENGINE_MAX_ZIPTYPERCG) failed: %s\n", cl_strerror(ret));
+	    cl_engine_free(engine);
+	    return 2;
+	}
+    }
+
     /* set scan options */
     if(optget(opts, "allmatch")->enabled)
 	options |= CL_SCAN_ALLMATCHES;
@@ -642,6 +820,9 @@ int scanmanager(const struct optstruct *opts)
     if(optget(opts, "scan-pdf")->enabled)
 	options |= CL_SCAN_PDF;
 
+    if(optget(opts, "scan-swf")->enabled)
+	options |= CL_SCAN_SWF;
+
     if(optget(opts, "scan-html")->enabled)
 	options |= CL_SCAN_HTML;
 
@@ -656,6 +837,9 @@ int scanmanager(const struct optstruct *opts)
 	options |= CL_SCAN_INTERNAL_COLLECT_SHA;
 #endif
 
+    if(optget(opts, "dev-performance")->enabled)
+	options |= CL_SCAN_PERFORMANCE_INFO;
+
     if(optget(opts, "detect-structured")->enabled) {
 	options |= CL_SCAN_STRUCTURED;
 
@@ -700,7 +884,7 @@ int scanmanager(const struct optstruct *opts)
 
 #ifdef C_LINUX
     procdev = (dev_t) 0;
-    if(stat("/proc", &sb) != -1 && !sb.st_size)
+    if(STAT("/proc", &sb) != -1 && !sb.st_size)
 	procdev = sb.st_dev;
 #endif
 
@@ -711,7 +895,7 @@ int scanmanager(const struct optstruct *opts)
 	    logg("!Can't get absolute pathname of current working directory\n");
 	    ret = 2;
 	} else {
-	    stat(cwd, &sb);
+	    STAT(cwd, &sb);
 	    scandirs(cwd, engine, opts, options, 1, sb.st_dev);
 	}
 
@@ -723,7 +907,7 @@ int scanmanager(const struct optstruct *opts)
 	    logg("^Only scanning files from --file-list (files passed at cmdline are ignored)\n");
 
 	while((filename = filelist(opts, &ret)) && (file = strdup(filename))) {
-	    if(lstat(file, &sb) == -1) {
+	    if(LSTAT(file, &sb) == -1) {
 		logg("^%s: Can't access file\n", file);
 		perror(file);
 		ret = 2;
@@ -739,7 +923,7 @@ int scanmanager(const struct optstruct *opts)
 		    if(dirlnk == 0 && filelnk == 0) {
 			if(!printinfected)
 			    logg("%s: Symbolic link\n", file);
-		    } else if(stat(file, &sb) != -1) {
+		    } else if(STAT(file, &sb) != -1) {
 			if(S_ISREG(sb.st_mode) && filelnk) {
 			    scanfile(file, engine, opts, options);
 			} else if(S_ISDIR(sb.st_mode) && dirlnk) {
@@ -762,6 +946,11 @@ int scanmanager(const struct optstruct *opts)
 	}
     }
 
+    if(optget(opts, "bytecode-statistics")->enabled) {
+	cli_sigperf_print();
+	cli_sigperf_events_destroy();
+    }
+
     /* free the engine */
     cl_engine_free(engine);
 
