@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -21,6 +21,31 @@
 #ifndef __CLAMAV_H
 #define __CLAMAV_H
 
+/* Certain OSs already use 64bit variables in their stat struct */
+#define STAT64_BLACKLIST !defined(__FreeBSD__) && !defined(__APPLE__)
+
+#if defined(HAVE_STAT64) && STAT64_BLACKLIST
+
+#include <unistd.h>
+
+#define STATBUF struct stat64
+#define STAT stat64
+#define LSTAT lstat64
+#define FSTAT fstat64
+#define safe_open(a, b) open(a, b|O_LARGEFILE)
+#else
+
+#define STATBUF struct stat
+#define STAT stat
+#define LSTAT lstat
+#define FSTAT fstat
+/* Nothing is safe in windows, not even open, safe_open defined under /win32 */
+#ifndef _WIN32
+#define safe_open open
+#endif
+
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -66,6 +91,7 @@ typedef enum {
     CL_EMAXSIZE,
     CL_EMAXFILES,
     CL_EFORMAT,
+    CL_EPARSE,
     CL_EBYTECODE,/* may be reported in testmode */
     CL_EBYTECODE_TESTFAIL, /* may be reported in testmode */
 
@@ -93,6 +119,9 @@ typedef enum {
 #define CL_DB_BYTECODE      0x2000
 #define CL_DB_SIGNED	    0x4000  /* internal */
 #define CL_DB_BYTECODE_UNSIGNED	0x8000
+#define CL_DB_UNSIGNED	    0x10000 /* internal */
+#define CL_DB_BYTECODE_STATS 0x20000
+#define CL_DB_ENHANCED      0x40000
 
 /* recommended db settings */
 #define CL_DB_STDOPT	    (CL_DB_PHISHING | CL_DB_PHISHING_URLS | CL_DB_BYTECODE)
@@ -120,11 +149,13 @@ typedef enum {
 #define CL_SCAN_HEURISTIC_PRECEDENCE    0x80000
 #define CL_SCAN_BLOCKMACROS		0x100000
 #define CL_SCAN_ALLMATCHES		0x200000
+#define CL_SCAN_SWF			0x400000
 
+#define CL_SCAN_PERFORMANCE_INFO        0x40000000 /* collect performance timings */
 #define CL_SCAN_INTERNAL_COLLECT_SHA    0x80000000 /* Enables hash output in sha-collect builds - for internal use only */
 
 /* recommended scan settings */
-#define CL_SCAN_STDOPT		(CL_SCAN_ARCHIVE | CL_SCAN_MAIL | CL_SCAN_OLE2 | CL_SCAN_PDF | CL_SCAN_HTML | CL_SCAN_PE | CL_SCAN_ALGORITHMIC | CL_SCAN_ELF)
+#define CL_SCAN_STDOPT		(CL_SCAN_ARCHIVE | CL_SCAN_MAIL | CL_SCAN_OLE2 | CL_SCAN_PDF | CL_SCAN_HTML | CL_SCAN_PE | CL_SCAN_ALGORITHMIC | CL_SCAN_ELF | CL_SCAN_SWF)
 
 /* cl_countsigs options */
 #define CL_COUNTSIGS_OFFICIAL	    0x1
@@ -157,7 +188,12 @@ enum cl_engine_field {
     CL_ENGINE_KEEPTMP,		    /* uint32_t */
     CL_ENGINE_BYTECODE_SECURITY,    /* uint32_t */
     CL_ENGINE_BYTECODE_TIMEOUT,     /* uint32_t */
-    CL_ENGINE_BYTECODE_MODE         /* uint32_t */
+    CL_ENGINE_BYTECODE_MODE,        /* uint32_t */
+    CL_ENGINE_MAX_EMBEDDEDPE,       /* uint64_t */
+    CL_ENGINE_MAX_HTMLNORMALIZE,    /* uint64_t */
+    CL_ENGINE_MAX_HTMLNOTAGS,       /* uint64_t */
+    CL_ENGINE_MAX_SCRIPTNORMALIZE,  /* uint64_t */
+    CL_ENGINE_MAX_ZIPTYPERCG        /* uint64_t */
 };
 
 enum bytecode_security {
@@ -196,13 +232,42 @@ extern int cl_engine_addref(struct cl_engine *engine);
 extern int cl_engine_free(struct cl_engine *engine);
 
 
-/* CALLBACKS - WARNING: unstable API - WIP */
+/* CALLBACKS */
 
+/* I certainly wish I could declare the callback protoes stable and
+   move on to better things. But real life crossed my way enough times
+   already and what looked perfect had to evolve somehow.
+   So all I can say is I'll try my best not to break these things in the long run.
+   But I just can't guarantee that won't happen (again). */
+
+typedef cl_error_t (*clcb_pre_cache)(int fd, const char *type, void *context);
+/* PRE-CACHE
+   Called for each processed file (both the entry level - AKA 'outer' - file and
+   inner files - those generated when processing archive and container files), before
+   the actual scanning takes place.
 
-typedef cl_error_t (*clcb_pre_scan)(int fd, void *context);
-/* PRE-SCAN
 Input:
 fd      = File descriptor which is about to be scanned
+type    = File type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE")
+context = Opaque application provided data
+
+Output:
+CL_CLEAN = File is scanned
+CL_BREAK = Whitelisted by callback - file is skipped and marked as clean
+CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected
+*/
+extern void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cache callback);
+
+typedef cl_error_t (*clcb_pre_scan)(int fd, const char *type, void *context);
+/* PRE-SCAN
+   Called for each NEW file (inner and outer) before the scanning takes place. This is
+   roughly the the same as clcb_before_cache, but it is affected by clean file caching.
+   This means that it won't be called if a clean cached file (inner or outer) is
+   scanned a second time.
+
+Input:
+fd      = File descriptor which is about to be scanned
+type    = File type detected via magic - i.e. NOT on the fly - (e.g. "CL_TYPE_MSEXE")
 context = Opaque application provided data
 
 Output:
@@ -212,9 +277,10 @@ CL_VIRUS = Blacklisted by callback - file is skipped and marked as infected
 */
 extern void cl_engine_set_clcb_pre_scan(struct cl_engine *engine, clcb_pre_scan callback);
 
-
 typedef cl_error_t (*clcb_post_scan)(int fd, int result, const char *virname, void *context);
 /* POST-SCAN
+   Called for each processed file (inner and outer), after the scanning is complete.
+
 Input:
 fd      = File descriptor which is was scanned
 result  = The scan result for the file
@@ -229,11 +295,12 @@ CL_VIRUS = Blacklisted by callback - scan result is set to CL_VIRUS
 extern void cl_engine_set_clcb_post_scan(struct cl_engine *engine, clcb_post_scan callback);
 
 
-typedef int (*clcb_sigload)(const char *type, const char *name, void *context);
+typedef int (*clcb_sigload)(const char *type, const char *name, unsigned int custom, void *context);
 /* SIGNATURE LOAD
 Input:
 type = The signature type (e.g. "db", "ndb", "mdb", etc.)
 name = The virus name
+custom = The signature is official (custom == 0) or custom (custom != 0)
 context = Opaque application provided data
 
 Output:
@@ -270,9 +337,15 @@ extern void cl_set_clcb_msg(clcb_msg callback);
 typedef void (*clcb_hash)(int fd, unsigned long long size, const unsigned char *md5, const char *virname, void *context);
 extern void cl_engine_set_clcb_hash(struct cl_engine *engine, clcb_hash callback);
 
+/* Archive member metadata callback. Return CL_VIRUS to blacklist, CL_CLEAN to
+ * continue scanning */
+typedef cl_error_t (*clcb_meta)(const char* container_type, unsigned long fsize_container, const char *filename,
+			  unsigned long fsize_real,  int is_encrypted, unsigned int filepos_container, void *context);
+extern void cl_engine_set_clcb_meta(struct cl_engine *engine, clcb_meta callback);
+
 struct cl_stat {
     char *dir;
-    struct stat *stattab;
+    STATBUF *stattab;
     char **statdname;
     unsigned int entries;
 };
@@ -326,6 +399,42 @@ extern const char *cl_retver(void);
 /* others */
 extern const char *cl_strerror(int clerror);
 
+/* custom data scanning */
+struct cl_fmap;
+typedef struct cl_fmap cl_fmap_t;
+
+/* handle - the handle passed to cl_fmap_open_handle, its meaning is up to the
+ *    callback's implementation
+ * buf, count, offset - read 'count' bytes starting at 'offset' into the buffer 'buf'
+ * Thread safety: it is guaranteed that only one callback is executing for a specific handle at
+ * any time, but there might be multiple callbacks executing for different
+ * handle at the same time.
+ */
+typedef off_t (*clcb_pread)(void* handle, void *buf, size_t count, off_t offset);
+
+/* Open a map for scanning custom data accessed by a handle and pread (lseek +
+ * read)-like interface. For example a WIN32 HANDLE.
+ * By default fmap will use aging to discard old data, unless you tell it not
+ * to.
+ * The handle will be passed to the callback each time.
+ */
+extern cl_fmap_t *cl_fmap_open_handle(void* handle, size_t offset, size_t len,
+				      clcb_pread, int use_aging);
+
+/* Open a map for scanning custom data, where the data is already in memory,
+ * either in the form of a buffer, a memory mapped file, etc.
+ * Note that the memory [start, start+len) must be the _entire_ file,
+ * you can't give it parts of a file and expect detection to work.
+ */
+extern cl_fmap_t *cl_fmap_open_memory(const void *start, size_t len);
+
+/* Releases resources associated with the map, you should release any resources
+ * you hold only after (handles, maps) calling this function */
+extern void cl_fmap_close(cl_fmap_t*);
+
+/* Scan custom data */
+extern int cl_scanmap_callback(cl_fmap_t *map, const char **virname, unsigned long int *scanned, const struct cl_engine *engine, unsigned int scanoptions, void *context);
+
 #ifdef __cplusplus
 }
 #endif
