@@ -70,6 +70,9 @@ static struct dconf_module modules[] = {
     { "PE",	    "MEW",	    PE_CONF_MEW,	    1 },
     { "PE",	    "UPACK",	    PE_CONF_UPACK,	    1 },
     { "PE",	    "ASPACK",	    PE_CONF_ASPACK,	    1 },
+    { "PE",	    "CATALOG",	    PE_CONF_CATALOG,	    1 },
+    { "PE",     "DISABLECERT",  PE_CONF_DISABLECERT,    0 },
+    { "PE",     "DUMPCERT",     PE_CONF_DUMPCERT,       0 },
 
     { "ELF",	    NULL,	    0x1,		    1 },
 
@@ -92,6 +95,7 @@ static struct dconf_module modules[] = {
     { "ARCHIVE",    "AUTOIT",	    ARCH_CONF_AUTOIT,	    1 },
     { "ARCHIVE",    "ISHIELD",	    ARCH_CONF_ISHIELD,	    1 },
     { "ARCHIVE",    "7zip",	    ARCH_CONF_7Z,	    1 },
+    { "ARCHIVE",    "ISO9660",	    ARCH_CONF_ISO9660,	    1 },
 
     { "DOCUMENT",   "HTML",	    DOC_CONF_HTML,	    1 },
     { "DOCUMENT",   "RTF",	    DOC_CONF_RTF,	    1 },
@@ -99,6 +103,7 @@ static struct dconf_module modules[] = {
     { "DOCUMENT",   "SCRIPT",	    DOC_CONF_SCRIPT,	    1 },
     { "DOCUMENT",   "HTMLSKIPRAW",  DOC_CONF_HTML_SKIPRAW,  1 },
     { "DOCUMENT",   "JSNORM",       DOC_CONF_JSNORM,        1 },
+    { "DOCUMENT",   "SWF",	    DOC_CONF_SWF,	    1 },
 
     { "MAIL",	    "MBOX",	    MAIL_CONF_MBOX,	    1 },
     { "MAIL",	    "TNEF",	    MAIL_CONF_TNEF,	    1 },
