@@ -59,6 +59,9 @@ struct cli_dconf {
 #define PE_CONF_MEW	    0x2000
 #define PE_CONF_UPACK	    0x4000
 #define PE_CONF_ASPACK	    0x8000
+#define PE_CONF_CATALOG	    0x10000
+#define PE_CONF_DISABLECERT 0x20000
+#define PE_CONF_DUMPCERT    0x40000
 
 /* Archive flags */
 #define ARCH_CONF_RAR	    0x1
@@ -78,6 +81,7 @@ struct cli_dconf {
 #define ARCH_CONF_CPIO	    0x4000
 #define ARCH_CONF_ISHIELD   0x8000
 #define ARCH_CONF_7Z        0x10000
+#define ARCH_CONF_ISO9660   0x20000
 
 /* Document flags */
 #define DOC_CONF_HTML		0x1
@@ -86,6 +90,7 @@ struct cli_dconf {
 #define DOC_CONF_SCRIPT 	0x8
 #define DOC_CONF_HTML_SKIPRAW	0x10
 #define DOC_CONF_JSNORM         0x20
+#define DOC_CONF_SWF		0x40
 
 /* Mail flags */
 #define MAIL_CONF_MBOX	    0x1
