@@ -38,6 +38,7 @@
 #include "str.h"
 #include "textdet.h"
 #include "default.h"
+#include "iowrap.h"
 
 #include "htmlnorm.h"
 #include "entconv.h"
@@ -96,6 +97,10 @@ static const struct ftmap_s {
     { "CL_TYPE_AUTOIT",		CL_TYPE_AUTOIT		},
     { "CL_TYPE_ISHIELD_MSI",	CL_TYPE_ISHIELD_MSI	},
     { "CL_TYPE_7Z",		CL_TYPE_7Z		},
+    { "CL_TYPE_7ZSFX",		CL_TYPE_7ZSFX		},
+    { "CL_TYPE_SWF",		CL_TYPE_SWF		},
+    { "CL_TYPE_ISO9660",	CL_TYPE_ISO9660		},
+    { "CL_TYPE_JAVA",		CL_TYPE_JAVA		},
     { NULL,			CL_TYPE_IGNORED		}
 };
 
@@ -110,6 +115,17 @@ cli_file_t cli_ftcode(const char *name)
     return CL_TYPE_ERROR;
 }
 
+const char *cli_ftname(cli_file_t code)
+{
+	unsigned int i;
+
+    for(i = 0; ftmap[i].name; i++)
+	if(ftmap[i].code == code)
+	    return ftmap[i].name;
+
+    return NULL;
+}
+
 void cli_ftfree(const struct cl_engine *engine)
 {
 	struct cli_ftype *ftypes=engine->ftypes, *pt;
@@ -141,11 +157,13 @@ cli_file_t cli_filetype(const unsigned char *buf, size_t buflen, const struct cl
     return cli_texttype(buf, buflen);
 }
 
-int is_tar(unsigned char *buf, unsigned int nbytes);
+int is_tar(const unsigned char *buf, unsigned int nbytes);
 
 cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine)
 {
-	unsigned char *buff, *decoded;
+	unsigned char buffer[MAGIC_BUFFER_SIZE];
+	const unsigned char *buff;
+	unsigned char *decoded;
 	int bread = MIN(map->len, MAGIC_BUFFER_SIZE), sret;
 	cli_file_t ret = CL_TYPE_BINARY_DATA;
 	struct cli_matcher *root;
@@ -158,9 +176,16 @@ cli_file_t cli_filetype2(fmap_t *map, const struct cl_engine *engine)
     }
 
     buff = fmap_need_off_once(map, 0, bread);
-    if(!buff)
-	return CL_TYPE_ERROR;
-
+    if(buff) {
+        sret = cli_memcpy(buffer, buff, bread);
+        if(sret) {
+            cli_errmsg("cli_filetype2: fileread error!\n");
+            return CL_TYPE_ERROR;
+        }
+        sret = 0;
+    } else {
+        return CL_TYPE_ERROR;
+    }
     ret = cli_filetype(buff, bread, engine);
 
     if(ret == CL_TYPE_BINARY_DATA) {
