@@ -26,26 +26,24 @@
 #include <stdlib.h>
 
 
-int hm_addhash(struct cli_matcher *root, const char *hash, uint32_t size, const char *virusname) {
-    const struct cli_htu32_element *item;
-    struct cli_sz_hash *szh;
-    struct cli_htu32 *ht;
+int hm_addhash_str(struct cli_matcher *root, const char *strhash, uint32_t size, const char *virusname) {
     enum CLI_HASH_TYPE type;
-    uint8_t binhash[32];
-    int hashlen, i;
+    char binhash[CLI_HASHLEN_MAX];
+    int hlen;
 
-    if(!root || !hash) {
-	cli_errmsg("hm_addhash: NULL root or hash\n");
+    if(!root || !strhash) {
+	cli_errmsg("hm_addhash_str: NULL root or hash\n");
 	return CL_ENULLARG;
     }
 
-    if(!size || size == (uint32_t)-1) {
-	cli_errmsg("hm_addhash: null or invalid size (%u)\n", size);
+    /* size 0 here is now a wildcard size match */
+    if(size == (uint32_t)-1) {
+	cli_errmsg("hm_addhash_str: null or invalid size (%u)\n", size);
 	return CL_EARG;
     }
 
-    hashlen = strlen(hash);
-    switch(hashlen) {
+    hlen = strlen(strhash);
+    switch(hlen) {
     case 32:
 	type = CLI_HASH_MD5;
 	break;
@@ -56,46 +54,67 @@ int hm_addhash(struct cli_matcher *root, const char *hash, uint32_t size, const
 	type = CLI_HASH_SHA256;
 	break;
     default:
-	cli_errmsg("hm_addhash: invalid hash %s -- FIXME!\n", hash);
+	cli_errmsg("hm_addhash_str: invalid hash %s -- FIXME!\n", strhash);
 	return CL_EARG;
     }
-    if(cli_hex2str_to(hash, (char *)binhash, hashlen)) {
-	cli_errmsg("hm_addhash: invalid hash %s\n", hash);
+    if(cli_hex2str_to(strhash, (char *)binhash, hlen)) {
+	cli_errmsg("hm_addhash_str: invalid hash %s\n", strhash);
 	return CL_EARG;
     }
 
-    hashlen /= 2;
-    ht = &root->hm.sizehashes[type];
-    if(!root->hm.sizehashes[type].capacity) {
-	i = cli_htu32_init(ht, 64, root->mempool);
-	if(i) return i;
+    return hm_addhash_bin(root, binhash, type, size, virusname);
+}
+
+static const unsigned int hashlen[] = {
+    CLI_HASHLEN_MD5,
+    CLI_HASHLEN_SHA1,
+    CLI_HASHLEN_SHA256
+};
+
+int hm_addhash_bin(struct cli_matcher *root, const void *binhash, enum CLI_HASH_TYPE type, uint32_t size, const char *virusname) {
+    const unsigned int hlen = hashlen[type];
+    const struct cli_htu32_element *item;
+    struct cli_sz_hash *szh;
+    struct cli_htu32 *ht;
+    int i;
+
+    if (size) {
+        /* size non-zero, find sz_hash element in size-driven hashtable  */
+        ht = &root->hm.sizehashes[type];
+        if(!root->hm.sizehashes[type].capacity) {
+            i = cli_htu32_init(ht, 64, root->mempool);
+            if(i) return i;
+        }
+
+        item = cli_htu32_find(ht, size);
+        if(!item) {
+	    struct cli_htu32_element htitem;
+	    szh = mpool_calloc(root->mempool, 1, sizeof(*szh));
+	    if(!szh) {
+	        cli_errmsg("hm_addhash_bin: failed to allocate size hash\n");
+	        return CL_EMEM;
+	    }
+
+	    htitem.key = size;
+	    htitem.data.as_ptr = szh;
+	    i = cli_htu32_insert(ht, &htitem, root->mempool);
+	    if(i) {
+	        cli_errmsg("hm_addhash_bin: failed to add item to hashtab");
+	        mpool_free(root->mempool, szh);
+	        return i;
+	    }
+        } else
+	    szh = (struct cli_sz_hash *)item->data.as_ptr;
+    }
+    else {
+        /* size 0 = wildcard */
+        szh = &root->hwild.hashes[type];
     }
-
-    item = cli_htu32_find(ht, size);
-    if(!item) {
-	struct cli_htu32_element htitem;
-	szh = mpool_calloc(root->mempool, 1, sizeof(*szh));
-	if(!szh) {
-	    cli_errmsg("hm_addhash: failed to allocate size hash\n");
-	    return CL_EMEM;
-	}
-
-	htitem.key = size;
-	htitem.data.as_ptr = szh;
-	i = cli_htu32_insert(ht, &htitem, root->mempool);
-	if(i) {
-	    cli_errmsg("ht_addhash: failed to add item to hashtab");
-	    mpool_free(root->mempool, szh);
-	    return i;
-	}
-    } else
-	szh = (struct cli_sz_hash *)item->data.as_ptr;
-
     szh->items++;
 
-    szh->hash_array = mpool_realloc2(root->mempool, szh->hash_array, hashlen * szh->items);
+    szh->hash_array = mpool_realloc2(root->mempool, szh->hash_array, hlen * szh->items);
     if(!szh->hash_array) {
-	cli_errmsg("ht_add: failed to grow hash array to %u entries\n", szh->items);
+	cli_errmsg("hm_addhash_bin: failed to grow hash array to %u entries\n", szh->items);
 	szh->items=0;
 	mpool_free(root->mempool, szh->virusnames);
 	szh->virusnames = NULL;
@@ -104,28 +123,19 @@ int hm_addhash(struct cli_matcher *root, const char *hash, uint32_t size, const
 
     szh->virusnames = mpool_realloc2(root->mempool, szh->virusnames, sizeof(*szh->virusnames) * szh->items);
     if(!szh->virusnames) {
-	cli_errmsg("ht_add: failed to grow virusname array to %u entries\n", szh->items);
+	cli_errmsg("hm_addhash_bin: failed to grow virusname array to %u entries\n", szh->items);
 	szh->items=0;
 	mpool_free(root->mempool, szh->hash_array);
 	szh->hash_array = NULL;
 	return CL_EMEM;
     }
 
-    memcpy(&szh->hash_array[(szh->items-1) * hashlen], binhash, hashlen);
+    memcpy(&szh->hash_array[(szh->items-1) * hlen], binhash, hlen);
     szh->virusnames[(szh->items-1)] = virusname;
     
     return 0;
 }
 
-
-
-static const unsigned int hashlen[] = {
-    16, /* CLI_HASH_MD5 */
-    20, /* CLI_HASH_SHA1 */
-    32, /* CLI_HASH_SHA256 */
-};
-
-
 static inline int hm_cmp(const uint8_t *itm, const uint8_t *ref, unsigned int keylen) {
 #if WORDS_BIGENDIAN == 0
     uint32_t i = *(uint32_t *)itm, r = *(uint32_t *)ref;
@@ -138,7 +148,7 @@ static inline int hm_cmp(const uint8_t *itm, const uint8_t *ref, unsigned int ke
 }
 
 static void hm_sort(struct cli_sz_hash *szh, size_t l, size_t r, unsigned int keylen) {
-    uint8_t piv[32], tmph[32];
+    uint8_t piv[CLI_HASHLEN_MAX], tmph[CLI_HASHLEN_MAX];
     size_t l1, r1;
 
     const char *tmpv;
@@ -177,9 +187,11 @@ static void hm_sort(struct cli_sz_hash *szh, size_t l, size_t r, unsigned int ke
     hm_sort(szh, r1, r, keylen);
 }
 
-
+/* flush both size-specific and agnostic hash sets */
 void hm_flush(struct cli_matcher *root) {
     enum CLI_HASH_TYPE type;
+    unsigned int keylen;
+    struct cli_sz_hash *szh;
 
     if(!root)
 	return;
@@ -187,18 +199,27 @@ void hm_flush(struct cli_matcher *root) {
     for(type = CLI_HASH_MD5; type < CLI_HASH_AVAIL_TYPES; type++) {
 	struct cli_htu32 *ht = &root->hm.sizehashes[type];
 	const struct cli_htu32_element *item = NULL;
+	szh = NULL;
 
 	if(!root->hm.sizehashes[type].capacity)
 	    continue;
 
 	while((item = cli_htu32_next(ht, item))) {
-	    struct cli_sz_hash *szh = (struct cli_sz_hash *)item->data.as_ptr;
-	    unsigned int keylen = hashlen[type];
+	    szh = (struct cli_sz_hash *)item->data.as_ptr;
+	    keylen = hashlen[type];
 
 	    if(szh->items > 1)
 		hm_sort(szh, 0, szh->items, keylen);
 	}
     }
+
+    for(type = CLI_HASH_MD5; type < CLI_HASH_AVAIL_TYPES; type++) {
+	szh = &root->hwild.hashes[type];
+	keylen = hashlen[type];
+
+	if(szh->items > 1)
+	    hm_sort(szh, 0, szh->items, keylen);
+    }
 }
 
 
@@ -206,20 +227,18 @@ int cli_hm_have_size(const struct cli_matcher *root, enum CLI_HASH_TYPE type, ui
     return (size && size != 0xffffffff && root && root->hm.sizehashes[type].capacity && cli_htu32_find(&root->hm.sizehashes[type], size));
 }
 
-int cli_hm_scan(const unsigned char *digest, uint32_t size, const char **virname, const struct cli_matcher *root, enum CLI_HASH_TYPE type) {
-    const struct cli_htu32_element *item;
+int cli_hm_have_wild(const struct cli_matcher *root, enum CLI_HASH_TYPE type) {
+    return (root && root->hwild.hashes[type].items);
+}
+
+/* cli_hm_scan will scan only size-specific hashes, if any */
+static int hm_scan(const unsigned char *digest, const char **virname, const struct cli_sz_hash *szh, enum CLI_HASH_TYPE type) {
     unsigned int keylen;
-    struct cli_sz_hash *szh;
     size_t l, r;
 
-    if(!digest || !size || size == 0xffffffff || !root || !root->hm.sizehashes[type].capacity)
+    if(!digest || !szh || !szh->items)
 	return CL_CLEAN;
 
-    item = cli_htu32_find(&root->hm.sizehashes[type], size);
-    if(!item)
-	return CL_CLEAN;
-
-    szh = (struct cli_sz_hash *)item->data.as_ptr;
     keylen = hashlen[type];
 
     l = 0;
@@ -243,6 +262,32 @@ int cli_hm_scan(const unsigned char *digest, uint32_t size, const char **virname
     return CL_CLEAN;
 }
 
+/* cli_hm_scan will scan only size-specific hashes, if any */
+int cli_hm_scan(const unsigned char *digest, uint32_t size, const char **virname, const struct cli_matcher *root, enum CLI_HASH_TYPE type) {
+    const struct cli_htu32_element *item;
+    struct cli_sz_hash *szh;
+
+    if(!digest || !size || size == 0xffffffff || !root || !root->hm.sizehashes[type].capacity)
+	return CL_CLEAN;
+
+    item = cli_htu32_find(&root->hm.sizehashes[type], size);
+    if(!item)
+	return CL_CLEAN;
+
+    szh = (struct cli_sz_hash *)item->data.as_ptr;
+
+    return hm_scan(digest, virname, szh, type);
+}
+
+/* cli_hm_scan_wild will scan only size-agnostic hashes, if any */
+int cli_hm_scan_wild(const unsigned char *digest, const char **virname, const struct cli_matcher *root, enum CLI_HASH_TYPE type) {
+    if(!digest || !root || !root->hwild.hashes[type].items)
+	return CL_CLEAN;
+
+    return hm_scan(digest, virname, &root->hwild.hashes[type], type);
+}
+
+/* free both size-specific and agnostic hash sets */
 void hm_free(struct cli_matcher *root) {
     enum CLI_HASH_TYPE type;
 
@@ -258,7 +303,6 @@ void hm_free(struct cli_matcher *root) {
 
 	while((item = cli_htu32_next(ht, item))) {
 	    struct cli_sz_hash *szh = (struct cli_sz_hash *)item->data.as_ptr;
-	    unsigned int keylen = hashlen[type];
 
 	    mpool_free(root->mempool, szh->hash_array);
 	    while(szh->items)
@@ -268,5 +312,17 @@ void hm_free(struct cli_matcher *root) {
 	}
 	cli_htu32_free(ht, root->mempool);
     }
+
+    for(type = CLI_HASH_MD5; type < CLI_HASH_AVAIL_TYPES; type++) {
+	struct cli_sz_hash *szh = &root->hwild.hashes[type];
+
+	if(!szh->items)
+	    continue;
+
+	mpool_free(root->mempool, szh->hash_array);
+	while(szh->items)
+	    mpool_free(root->mempool, (void *)szh->virusnames[--szh->items]);
+	mpool_free(root->mempool, szh->virusnames);
+    }
 }
 
