@@ -141,7 +141,7 @@ typedef	enum {
 #include <netdb.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
-#ifndef	C_BEOS
+#if !defined(C_BEOS) && !defined(C_INTERIX)
 #include <net/if.h>
 #include <arpa/inet.h>
 #endif
@@ -186,7 +186,7 @@ typedef	struct	mbox_ctx {
 #define UNLOCKFILE(fp)
 #endif
 
-static	int	cli_parse_mbox(const char *dir, int desc, cli_ctx *ctx);
+static	int	cli_parse_mbox(const char *dir, cli_ctx *ctx);
 static	message	*parseEmailFile(fmap_t *map, size_t *at, const table_t *rfc821Table, const char *firstLine, const char *dir);
 static	message	*parseEmailHeaders(message *m, const table_t *rfc821Table);
 static	int	parseEmailHeader(message *m, const char *line, const table_t *rfc821Table);
@@ -300,15 +300,17 @@ static	const	struct tableinit {
 #ifdef	CL_THREAD_SAFE
 static	pthread_mutex_t	tables_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif
+static	table_t *rfc821 = NULL;
+static	table_t *subtype = NULL;
 
 int
-cli_mbox(const char *dir, int desc, cli_ctx *ctx)
+cli_mbox(const char *dir, cli_ctx *ctx)
 {
 	if(dir == NULL) {
 		cli_dbgmsg("cli_mbox called with NULL dir\n");
 		return CL_ENULLARG;
 	}
-	return cli_parse_mbox(dir, desc, ctx);
+	return cli_parse_mbox(dir, ctx);
 }
 
 /*
@@ -327,13 +329,12 @@ cli_mbox(const char *dir, int desc, cli_ctx *ctx)
  *	e.g. \0Content-Type: application/binary;
  */
 static int
-cli_parse_mbox(const char *dir, int desc, cli_ctx *ctx)
+cli_parse_mbox(const char *dir, cli_ctx *ctx)
 {
 	int retcode;
 	message *body;
 	char buffer[RFC2821LENGTH + 1];
 	mbox_ctx mctx;
-	static table_t *rfc821, *subtype;
 	size_t at = 0;
 	fmap_t *map = *ctx->fmap;
 
@@ -434,7 +435,7 @@ cli_parse_mbox(const char *dir, int desc, cli_ctx *ctx)
 						continue;
 					} else if(rc == VIRUS) {
 						cli_dbgmsg("Message number %d is infected\n",
-							messagenumber);
+							messagenumber-1);
 						retcode = CL_VIRUS;
 						m = NULL;
 						break;
@@ -543,7 +544,7 @@ cli_parse_mbox(const char *dir, int desc, cli_ctx *ctx)
 		 */
 		messageDestroy(body);
 	}
-
+	
 	if((retcode == CL_CLEAN) && ctx->found_possibly_unwanted &&
 	   (*ctx->virname == NULL || SCAN_ALL)) {
 	    cli_append_virus(ctx, "Heuristics.Phishing.Email");
@@ -581,7 +582,7 @@ parseEmailFile(fmap_t *map, size_t *at, const table_t *rfc821, const char *first
 	if(ret == NULL)
 		return NULL;
 
-	strcpy(buffer, firstLine);
+	strncpy(buffer, firstLine, sizeof(buffer)-1);
 	do {
 		const char *line;
 
@@ -657,7 +658,7 @@ parseEmailFile(fmap_t *map, size_t *at, const table_t *rfc821, const char *first
 				bodyIsEmpty = TRUE;
 			} else {
 				char *ptr;
-				char *lookahead;
+				const char *lookahead;
 
 				if(fullline == NULL) {
 					char cmd[RFC2821LENGTH + 1], out[RFC2821LENGTH + 1];
@@ -2436,6 +2437,7 @@ parseMimeHeader(message *m, const char *cmd, const table_t *rfc821Table, const c
 
 				buf = cli_malloc(strlen(ptr) + 1);
 				if(buf == NULL) {
+                    cli_errmsg("parseMimeHeader: Unable to allocate memory for buf %u\n", strlen(ptr) + 1);
 					if(copy)
 						free(copy);
 					return -1;
@@ -2544,6 +2546,7 @@ parseMimeHeader(message *m, const char *cmd, const table_t *rfc821Table, const c
 		case CONTENT_DISPOSITION:
 			buf = cli_malloc(strlen(ptr) + 1);
 			if(buf == NULL) {
+                cli_errmsg("parseMimeHeader: Unable to allocate memory for buf %u\n", strlen(ptr) + 1);
 				if(copy)
 					free(copy);
 				return -1;
@@ -2620,8 +2623,10 @@ rfc822comments(const char *in, char *out)
 
 	if(out == NULL) {
 		out = cli_malloc(strlen(in) + 1);
-		if(out == NULL)
+		if(out == NULL) {
+            cli_errmsg("rfc822comments: Unable to allocate memory for out %u\n", strlen(in) + 1);
 			return NULL;
+        }
 	}
 
 	backslash = commentlevel = inquote = 0;
@@ -2686,8 +2691,10 @@ rfc2047(const char *in)
 	cli_dbgmsg("rfc2047 '%s'\n", in);
 	out = cli_malloc(strlen(in) + 1);
 
-	if(out == NULL)
+	if(out == NULL) {
+        cli_errmsg("rfc2047: Unable to allocate memory for out %u\n", strlen(in) + 1);
 		return NULL;
+    }
 
 	pout = out;
 
@@ -2804,9 +2811,9 @@ rfc1341(message *m, const char *dir)
 		free(id);
 		return -1;
 	} else if(errno == EEXIST) {
-		struct stat statb;
+		STATBUF statb;
 
-		if(stat(pdir, &statb) < 0) {
+		if(STAT(pdir, &statb) < 0) {
 			char err[128];
 			cli_errmsg("Partial directory %s: %s\n", pdir,
 				cli_strerror(errno, err, sizeof(err)));
@@ -2918,8 +2925,9 @@ rfc1341(message *m, const char *dir)
 					FILE *fin;
 					char buffer[BUFSIZ], fullname[NAME_MAX + 1];
 					int nblanks;
-					struct stat statb;
+					STATBUF statb;
 					const char *dentry_idpart;
+                    int test_fd;
 
 					if(dent->d_ino == 0)
 						continue;
@@ -2935,8 +2943,15 @@ rfc1341(message *m, const char *dir)
 							strcmp(filename, dentry_idpart) != 0) {
 						if(!m->ctx->engine->keeptmp)
 							continue;
-						if(stat(fullname, &statb) < 0)
+
+                        if ((test_fd = open(fullname, O_RDONLY)) < 0)
+                            continue;
+
+						if(FSTAT(test_fd, &statb) < 0) {
+                            close(test_fd);
 							continue;
+                        }
+
 						if(now - statb.st_mtime > (time_t)(7 * 24 * 3600)) {
 							if (cli_unlink(fullname)) {
 								cli_unlink(outname);
@@ -2945,9 +2960,12 @@ rfc1341(message *m, const char *dir)
 								free(id);
 								free(number);
 								closedir(dd);
+                                close(test_fd);
 								return -1;
 							}
 						}
+
+                        close(test_fd);
 						continue;
 					}
 
@@ -2956,6 +2974,7 @@ rfc1341(message *m, const char *dir)
 						cli_errmsg("Can't open '%s' for reading", fullname);
 						fclose(fout);
 						cli_unlink(outname);
+						free(md5_hex);
 						free(id);
 						free(number);
 						closedir(dd);
@@ -3213,7 +3232,8 @@ usefulHeader(int commandNumber, const char *cmd)
 static char *
 getline_from_mbox(char *buffer, size_t buffer_len, fmap_t *map, size_t *at)
 {
-    char *src, *cursrc, *curbuf;
+    const char *src, *cursrc;
+    char *curbuf;
     size_t i;
     size_t input_len = MIN(map->len - *at, buffer_len + 1);
     src = cursrc = fmap_need_off_once(map, *at, input_len);
