@@ -212,6 +212,8 @@ const char *cl_strerror(int clerror)
 	    return "Can't verify database integrity";
 	case CL_EUNPACK:
 	    return "Can't unpack some data";
+	case CL_EPARSE: /* like CL_EFORMAT but reported outside magicscan() */
+	    return "Can't parse data";
 
 	/* I/O and memory errors */
 	case CL_EOPEN:
@@ -240,6 +242,8 @@ const char *cl_strerror(int clerror)
 	    return "Can't map file into memory";
 	case CL_EMEM:
 	    return "Can't allocate memory";
+	case CL_ETIMEOUT:
+	    return "Time limit reached";
 	/* internal (needed for debug messages) */
 	case CL_EMAXREC:
 	    return "CL_EMAXREC";
@@ -308,6 +312,12 @@ struct cl_engine *cl_engine_new(void)
     new->maxfiles = CLI_DEFAULT_MAXFILES;
     new->min_cc_count = CLI_DEFAULT_MIN_CC_COUNT;
     new->min_ssn_count = CLI_DEFAULT_MIN_SSN_COUNT;
+    /* Engine Max sizes */
+    new->maxembeddedpe = CLI_DEFAULT_MAXEMBEDDEDPE;
+    new->maxhtmlnormalize = CLI_DEFAULT_MAXHTMLNORMALIZE;
+    new->maxhtmlnotags = CLI_DEFAULT_MAXHTMLNOTAGS;
+    new->maxscriptnormalize = CLI_DEFAULT_MAXSCRIPTNORMALIZE;
+    new->maxziptypercg = CLI_DEFAULT_MAXZIPTYPERCG;
 
     new->bytecode_security = CL_BYTECODE_TRUST_SIGNED;
     /* 5 seconds timeout */
@@ -347,6 +357,18 @@ struct cl_engine *cl_engine_new(void)
 	return NULL;
     }
 
+    crtmgr_init(&(new->cmgr));
+    if(crtmgr_add_roots(new, &(new->cmgr)))  {
+	cli_errmsg("cl_engine_new: Can't initialize root certificates\n");
+	mpool_free(new->mempool, new->dconf);
+	mpool_free(new->mempool, new->root);
+#ifdef USE_MPOOL
+	mpool_destroy(new->mempool);
+#endif
+	free(new);
+	return NULL;
+    }
+
     cli_dbgmsg("Initialized %s engine\n", cl_retver());
     return new;
 }
@@ -376,6 +398,41 @@ int cl_engine_set_num(struct cl_engine *engine, enum cl_engine_field field, long
 	case CL_ENGINE_MAX_FILES:
 	    engine->maxfiles = num;
 	    break;
+	case CL_ENGINE_MAX_EMBEDDEDPE:
+	    if(num < 0) {
+		cli_warnmsg("MaxEmbeddedPE: negative values are not allowed, using default: %u\n", CLI_DEFAULT_MAXEMBEDDEDPE);
+		engine->maxembeddedpe = CLI_DEFAULT_MAXEMBEDDEDPE;
+	    } else
+		engine->maxembeddedpe = num;
+	    break;
+	case CL_ENGINE_MAX_HTMLNORMALIZE:
+	    if(num < 0) {
+		cli_warnmsg("MaxHTMLNormalize: negative values are not allowed, using default: %u\n", CLI_DEFAULT_MAXHTMLNORMALIZE);
+		engine->maxhtmlnormalize = CLI_DEFAULT_MAXHTMLNORMALIZE;
+	    } else
+		engine->maxhtmlnormalize = num;
+	    break;
+	case CL_ENGINE_MAX_HTMLNOTAGS:
+	    if(num < 0) {
+		cli_warnmsg("MaxHTMLNoTags: negative values are not allowed, using default: %u\n", CLI_DEFAULT_MAXHTMLNOTAGS);
+		engine->maxhtmlnotags = CLI_DEFAULT_MAXHTMLNOTAGS;
+	    } else
+		engine->maxhtmlnotags = num;
+	    break;
+	case CL_ENGINE_MAX_SCRIPTNORMALIZE:
+	    if(num < 0) {
+		cli_warnmsg("MaxScriptNormalize: negative values are not allowed, using default: %u\n", CLI_DEFAULT_MAXSCRIPTNORMALIZE);
+		engine->maxscriptnormalize = CLI_DEFAULT_MAXSCRIPTNORMALIZE;
+	    } else
+		engine->maxscriptnormalize = num;
+	    break;
+	case CL_ENGINE_MAX_ZIPTYPERCG:
+	    if(num < 0) {
+		cli_warnmsg("MaxZipTypeRcg: negative values are not allowed, using default: %u\n", CLI_DEFAULT_MAXZIPTYPERCG);
+		engine->maxziptypercg = CLI_DEFAULT_MAXZIPTYPERCG;
+	    } else
+		engine->maxziptypercg = num;
+	    break;
 	case CL_ENGINE_MIN_CC_COUNT:
 	    engine->min_cc_count = num;
 	    break;
@@ -453,6 +510,16 @@ long long cl_engine_get_num(const struct cl_engine *engine, enum cl_engine_field
 	    return engine->maxreclevel;
 	case CL_ENGINE_MAX_FILES:
 	    return engine->maxfiles;
+	case CL_ENGINE_MAX_EMBEDDEDPE:
+	    return engine->maxembeddedpe;
+	case CL_ENGINE_MAX_HTMLNORMALIZE:
+	    return engine->maxhtmlnormalize;
+	case CL_ENGINE_MAX_HTMLNOTAGS:
+	    return engine->maxhtmlnotags;
+	case CL_ENGINE_MAX_SCRIPTNORMALIZE:
+	    return engine->maxscriptnormalize;
+	case CL_ENGINE_MAX_ZIPTYPERCG:
+	    return engine->maxziptypercg;
 	case CL_ENGINE_MIN_CC_COUNT:
 	    return engine->min_cc_count;
 	case CL_ENGINE_MIN_SSN_COUNT:
@@ -537,8 +604,10 @@ struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engine)
 	struct cl_settings *settings;
 
     settings = (struct cl_settings *) malloc(sizeof(struct cl_settings));
-    if(!settings)
-	return NULL;
+    if(!settings) {
+        cli_errmsg("cl_engine_settings_copy: Unable to allocate memory for settings %u\n", sizeof(struct cl_settings));
+        return NULL;
+    }
 
     settings->ac_only = engine->ac_only;
     settings->ac_mindepth = engine->ac_mindepth;
@@ -549,6 +618,11 @@ struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engine)
     settings->maxfilesize = engine->maxfilesize;
     settings->maxreclevel = engine->maxreclevel;
     settings->maxfiles = engine->maxfiles;
+    settings->maxembeddedpe = engine->maxembeddedpe;
+    settings->maxhtmlnormalize = engine->maxhtmlnormalize;
+    settings->maxhtmlnotags = engine->maxhtmlnotags;
+    settings->maxscriptnormalize = engine->maxscriptnormalize;
+    settings->maxziptypercg = engine->maxziptypercg;
     settings->min_cc_count = engine->min_cc_count;
     settings->min_ssn_count = engine->min_ssn_count;
     settings->bytecode_security = engine->bytecode_security;
@@ -556,11 +630,13 @@ struct cl_settings *cl_engine_settings_copy(const struct cl_engine *engine)
     settings->bytecode_mode = engine->bytecode_mode;
     settings->pua_cats = engine->pua_cats ? strdup(engine->pua_cats) : NULL;
 
+    settings->cb_pre_cache = engine->cb_pre_cache;
     settings->cb_pre_scan = engine->cb_pre_scan;
     settings->cb_post_scan = engine->cb_post_scan;
     settings->cb_sigload = engine->cb_sigload;
     settings->cb_sigload_ctx = engine->cb_sigload_ctx;
     settings->cb_hash = engine->cb_hash;
+    settings->cb_meta = engine->cb_meta;
 
     return settings;
 }
@@ -575,6 +651,11 @@ int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings
     engine->maxfilesize = settings->maxfilesize;
     engine->maxreclevel = settings->maxreclevel;
     engine->maxfiles = settings->maxfiles;
+    engine->maxembeddedpe = settings->maxembeddedpe;
+    engine->maxhtmlnormalize = settings->maxhtmlnormalize;
+    engine->maxhtmlnotags = settings->maxhtmlnotags;
+    engine->maxscriptnormalize = settings->maxscriptnormalize;
+    engine->maxziptypercg = settings->maxziptypercg;
     engine->min_cc_count = settings->min_cc_count;
     engine->min_ssn_count = settings->min_ssn_count;
     engine->bytecode_security = settings->bytecode_security;
@@ -601,11 +682,13 @@ int cl_engine_settings_apply(struct cl_engine *engine, const struct cl_settings
 	engine->pua_cats = NULL;
     }
 
+    engine->cb_pre_cache = settings->cb_pre_cache;
     engine->cb_pre_scan = settings->cb_pre_scan;
     engine->cb_post_scan = settings->cb_post_scan;
     engine->cb_sigload = settings->cb_sigload;
     engine->cb_sigload_ctx = settings->cb_sigload_ctx;
     engine->cb_hash = settings->cb_hash;
+    engine->cb_meta = settings->cb_meta;
 
     return CL_SUCCESS;
 }
@@ -636,7 +719,7 @@ int cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned
         /* if the remaining scansize is too small... */
         if(ctx->engine->maxscansize-ctx->scansize<needed) {
 	    /* ... we tell the caller to skip this file */
-	    cli_dbgmsg("%s: scansize exceeded (initial: %lu, remaining: %lu, needed: %lu)\n", who, (unsigned long int) ctx->engine->maxscansize, (unsigned long int) ctx->scansize, needed);
+	    cli_dbgmsg("%s: scansize exceeded (initial: %lu, consumed: %lu, needed: %lu)\n", who, (unsigned long int) ctx->engine->maxscansize, (unsigned long int) ctx->scansize, needed);
 	    ret = CL_EMAXSIZE;
 	}
     }
@@ -758,17 +841,19 @@ void cli_append_virus(cli_ctx * ctx, const char * virname)
 	return;
     if (SCAN_ALL) {
 	if (ctx->size_viruses == 0) {
-	    ctx->size_viruses = 2;
-	    if (!(ctx->virname = malloc(ctx->size_viruses * sizeof(char *)))) {
+	    if (!(ctx->virname = malloc(2 * sizeof(char *)))) {
 		cli_errmsg("cli_append_virus: fails on malloc() - virus %s virname not appended.\n", virname);
 		return;
 	    }
+	    ctx->size_viruses = 2;
 	} else if (ctx->num_viruses+1 == ctx->size_viruses) {
-	    ctx->size_viruses *= 2;
-	    if ((ctx->virname = realloc((void *)ctx->virname, ctx->size_viruses * sizeof (char *))) == NULL) {
+	    void * newptr = NULL;
+	    if ((newptr = realloc((void *)ctx->virname, 2 * ctx->size_viruses * sizeof (char *))) == NULL) {
 		cli_errmsg("cli_append_virus: fails on realloc() - virus %s virname not appended.\n", virname);
 		return;
 	    }
+	    ctx->virname = newptr;
+	    ctx->size_viruses *= 2;
 	}
 	ctx->virname[ctx->num_viruses++] = virname;
 	ctx->virname[ctx->num_viruses] = NULL;
@@ -797,6 +882,7 @@ const char * cli_get_last_virus_str(const cli_ctx * ctx)
 }
 
 
+
 #ifdef	C_WINDOWS
 /*
  * Windows doesn't allow you to delete a directory while it is still open
@@ -805,7 +891,7 @@ int
 cli_rmdirs(const char *name)
 {
 	int rc;
-	struct stat statb;	
+	STATBUF statb;	
 	DIR *dd;
 	struct dirent *dent;
 #if defined(HAVE_READDIR_R_3) || defined(HAVE_READDIR_R_2)
@@ -817,7 +903,7 @@ cli_rmdirs(const char *name)
 	char err[128];
 
 
-    if(stat(name, &statb) < 0) {
+    if(STAT(name, &statb) < 0) {
 	cli_warnmsg("cli_rmdirs: Can't locate %s: %s\n", name, cli_strerror(errno, err, sizeof(err)));
 	return -1;
     }
@@ -849,6 +935,7 @@ cli_rmdirs(const char *name)
 	path = cli_malloc(strlen(name) + strlen(dent->d_name) + 2);
 
 	if(path == NULL) {
+        cli_errmsg("cli_rmdirs: Unable to allocate memory for path %u\n", strlen(name) + strlen(dent->d_name) + 2);
 	    closedir(dd);
 	    return -1;
 	}
@@ -880,14 +967,14 @@ int cli_rmdirs(const char *dirname)
 	    char b[offsetof(struct dirent, d_name) + NAME_MAX + 1];
 	} result;
 #endif
-	struct stat maind, statbuf;
+	STATBUF maind, statbuf;
 	char *path;
 	char err[128];
 
 
     chmod(dirname, 0700);
     if((dd = opendir(dirname)) != NULL) {
-	while(stat(dirname, &maind) != -1) {
+	while(STAT(dirname, &maind) != -1) {
 	    if(!rmdir(dirname)) break;
 	    if(errno != ENOTEMPTY && errno != EEXIST && errno != EBADF) {
 		cli_errmsg("cli_rmdirs: Can't remove temporary directory %s: %s\n", dirname, cli_strerror(errno, err, sizeof(err)));
@@ -907,6 +994,7 @@ int cli_rmdirs(const char *dirname)
 		    if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..")) {
 			path = cli_malloc(strlen(dirname) + strlen(dent->d_name) + 2);
 			if(!path) {
+                cli_errmsg("cli_rmdirs: Unable to allocate memory for path %u\n", strlen(dirname) + strlen(dent->d_name) + 2);
 			    closedir(dd);
 			    return -1;
 			}
@@ -914,7 +1002,7 @@ int cli_rmdirs(const char *dirname)
 			sprintf(path, "%s"PATHSEP"%s", dirname, dent->d_name);
 
 			/* stat the file */
-			if(lstat(path, &statbuf) != -1) {
+			if(LSTAT(path, &statbuf) != -1) {
 			    if(S_ISDIR(statbuf.st_mode) && !S_ISLNK(statbuf.st_mode)) {
 				if(rmdir(path) == -1) { /* can't be deleted */
 				    if(errno == EACCES) {
@@ -954,65 +1042,6 @@ int cli_rmdirs(const char *dirname)
 }
 #endif
 
-int cli_dumpscan(int fd, off_t offset, size_t size, cli_ctx *ctx)
-{
-	int newfd, bread, sum = 0, ret;
-	char buff[FILEBUFF];
-	char *name;
-
-    if(offset) {
-	if(lseek(fd, offset, SEEK_SET) == -1) {
-	    cli_dbgmsg("cli_dumpscan: Can't lseek to %u\n", (unsigned int) offset);
-	    return CL_EFORMAT; /* most likely because of corrupted file */
-	}
-    }
-
-    if(!(name = cli_gentemp(ctx->engine->tmpdir)))
-	return CL_EMEM;
-
-    if((newfd = open(name, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, S_IRWXU)) < 0) {
-	cli_errmsg("cli_dumpscan: Can't create file %s\n", name);
-	free(name);
-	return CL_ECREAT;
-    }
-
-    while((bread = cli_readn(fd, buff, FILEBUFF)) > 0) {
-	if((uint32_t) (sum + bread) >= size) {
-	    if(write(newfd, buff, size - sum) == -1) {
-		cli_errmsg("cli_dumpscan: Can't write to %s\n", name);
-		close(newfd);
-		cli_unlink(name);
-		free(name);
-		return CL_EWRITE;
-	    }
-	    break;
-	} else {
-	    if(write(newfd, buff, bread) == -1) {
-		cli_errmsg("cli_dumpscan: Can't write to %s\n", name);
-		close(newfd);
-		cli_unlink(name);
-		free(name);
-		return CL_EWRITE;
-	    }
-	}
-	sum += bread;
-    }
-    cli_dbgmsg("DUMP&SCAN: File extracted to %s\n", name);
-    lseek(newfd, 0, SEEK_SET);
-    if((ret = cli_magic_scandesc(newfd, ctx)) == CL_VIRUS)
-	cli_dbgmsg("cli_dumpscan: Infected with %s\n", cli_get_last_virus_str(ctx));
-
-    close(newfd);
-    if(!ctx->engine->keeptmp) {
-	if(cli_unlink(name)) {
-	    free(name);
-	    return CL_EUNLINK;
-	}
-    }
-    free(name);
-    return ret;
-}
-
 /* Implement a generic bitset, trog@clamav.net */
 
 #define BITS_PER_CHAR (8)
@@ -1037,11 +1066,13 @@ bitset_t *cli_bitset_init(void)
 	
 	bs = cli_malloc(sizeof(bitset_t));
 	if (!bs) {
+        cli_errmsg("cli_bitset_init: Unable to allocate memory for bs %u\n", sizeof(bitset_t));
 		return NULL;
 	}
 	bs->length = BITSET_DEFAULT_SIZE;
 	bs->bitset = cli_calloc(BITSET_DEFAULT_SIZE, 1);
 	if (!bs->bitset) {
+        cli_errmsg("cli_bitset_init: Unable to allocate memory for bs->bitset %u\n", BITSET_DEFAULT_SIZE);
 	    free(bs);
 	    return NULL;
 	}
@@ -1105,6 +1136,10 @@ int cli_bitset_test(bitset_t *bs, unsigned long bit_offset)
 	return (bs->bitset[char_offset] & ((unsigned char)1 << bit_offset));
 }
 
+void cl_engine_set_clcb_pre_cache(struct cl_engine *engine, clcb_pre_cache callback) {
+    engine->cb_pre_cache = callback;
+}
+
 void cl_engine_set_clcb_pre_scan(struct cl_engine *engine, clcb_pre_scan callback) {
     engine->cb_pre_scan = callback;
 }
@@ -1122,3 +1157,8 @@ void cl_engine_set_clcb_hash(struct cl_engine *engine, clcb_hash callback)
 {
     engine->cb_hash = callback;
 }
+
+void cl_engine_set_clcb_meta(struct cl_engine *engine, clcb_meta callback)
+{
+    engine->cb_meta = callback;
+}
