@@ -43,6 +43,8 @@
 #include "regex/regex.h"
 #include "bytecode.h"
 #include "bytecode_api.h"
+#include "events.h"
+#include "crtmgr.h"
 
 /*
  * CL_FLEVEL is the signature f-level specific to the current code and
@@ -53,7 +55,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 69
+#define CL_FLEVEL 74
 #define CL_FLEVEL_DCONF	CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -120,6 +122,7 @@ typedef struct cli_ctx_tag {
     unsigned int scannedfiles;
     unsigned int found_possibly_unwanted;
     unsigned int corrupted_input;
+    unsigned int img_validate;
     cli_file_t container_type; /* FIXME: to be made into a stack or array - see bb#1579 & bb#1293 */
     size_t container_size;
     unsigned char handlertype_hash[16];
@@ -127,6 +130,7 @@ typedef struct cli_ctx_tag {
     fmap_t **fmap;
     bitset_t* hook_lsig_matches;
     void *cb_ctx;
+    cli_events_t* perf;
 #ifdef HAVE__INTERNAL__SHA_COLLECT
     char entry_filename[2048];
     int sha_collect;
@@ -250,12 +254,17 @@ struct cl_engine {
     /* Used for memory pools */
     mpool_t *mempool;
 
+    /* crtmgr stuff */
+    crtmgr cmgr;
+
     /* Callback(s) */
+    clcb_pre_cache cb_pre_cache;
     clcb_pre_scan cb_pre_scan;
     clcb_post_scan cb_post_scan;
     clcb_sigload cb_sigload;
     void *cb_sigload_ctx;
     clcb_hash cb_hash;
+    clcb_meta cb_meta;
 
     /* Used for bytecode */
     struct cli_all_bc bcs;
@@ -265,6 +274,13 @@ struct cl_engine {
     enum bytecode_security bytecode_security;
     uint32_t bytecode_timeout;
     enum bytecode_mode bytecode_mode;
+
+    /* Engine max settings */
+    uint64_t maxembeddedpe;  /* max size to scan MSEXE for PE */
+    uint64_t maxhtmlnormalize; /* max size to normalize HTML */
+    uint64_t maxhtmlnotags; /* max size for scanning normalized HTML */
+    uint64_t maxscriptnormalize; /* max size to normalize scripts */
+    uint64_t maxziptypercg; /* max size to re-do zip filetype */
 };
 
 struct cl_settings {
@@ -289,12 +305,21 @@ struct cl_settings {
     char *pua_cats;
 
     /* callbacks */
+    clcb_pre_cache cb_pre_cache;
     clcb_pre_scan cb_pre_scan;
     clcb_post_scan cb_post_scan;
     clcb_sigload cb_sigload;
     void *cb_sigload_ctx;
     clcb_msg cb_msg;
     clcb_hash cb_hash;
+    clcb_meta cb_meta;
+
+    /* Engine max settings */
+    uint64_t maxembeddedpe;  /* max size to scan MSEXE for PE */
+    uint64_t maxhtmlnormalize; /* max size to normalize HTML */
+    uint64_t maxhtmlnotags; /* max size for scanning normalized HTML */
+    uint64_t maxscriptnormalize; /* max size to normalize scripts */
+    uint64_t maxziptypercg; /* max size to re-do zip filetype */
 };
 
 extern int (*cli_unrar_open)(int fd, const char *dirname, unrar_state_t *state);
@@ -317,6 +342,7 @@ extern int have_rar;
 #define BLOCK_MACROS	    (ctx->options & CL_SCAN_BLOCKMACROS)
 #define SCAN_STRUCTURED	    (ctx->options & CL_SCAN_STRUCTURED)
 #define SCAN_ALL            (ctx->options & CL_SCAN_ALLMATCHES)
+#define SCAN_SWF            (ctx->options & CL_SCAN_SWF)
 
 /* based on macros from A. Melnikoff */
 #define cbswap16(v) (((v & 0xff) << 8) | (((v) >> 8) & 0xff))
@@ -524,7 +550,7 @@ char *cli_gentemp(const char *dir);
 int cli_gentempfd(const char *dir, char **name, int *fd);
 unsigned int cli_rndnum(unsigned int max);
 int cli_filecopy(const char *src, const char *dest);
-int cli_dumpscan(int fd, off_t offset, size_t size, cli_ctx *ctx);
+int cli_mapscan(fmap_t *map, off_t offset, size_t size, cli_ctx *ctx);
 bitset_t *cli_bitset_init(void);
 void cli_bitset_free(bitset_t *bs);
 int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
@@ -571,7 +597,7 @@ struct cli_ftw_cbdata {
  * after an error, we call the callback with reason == error,
  * and if it returns CL_BREAK we break.
  */
-typedef int (*cli_ftw_cb)(struct stat *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
+typedef int (*cli_ftw_cb)(STATBUF *stat_buf, char *filename, const char *path, enum cli_ftw_reason reason, struct cli_ftw_cbdata *data);
 
 /*
  * returns 1 if the path should be skipped and 0 otherwise
