@@ -134,7 +134,7 @@ void cl_set_clcb_msg(clcb_msg callback)
 
 #define MSGCODE(buff, len, x)				    \
 	va_list args;					    \
-	int len = sizeof(x) - 1;			    \
+	size_t len = sizeof(x) - 1;			    \
 	char buff[BUFSIZ];				    \
     strncpy(buff, x, len);				    \
     va_start(args, str);				    \
@@ -436,20 +436,25 @@ int cli_filecopy(const char *src, const char *dest)
 
 const char *cli_gettmpdir(void) {
 	const char *tmpdir;
-    if(
+    unsigned int i;
+
 #ifdef _WIN32
-	!(tmpdir = getenv("TEMP")) && !(tmpdir = getenv("TMP"))
+    char *envs[] = { "TEMP", "TMP", NULL };
 #else
-	!(tmpdir = getenv("TMPDIR"))
+    char *envs[] = { "TMPDIR", NULL };
 #endif
-    ) tmpdir = P_tmpdir;
-    return tmpdir;
+
+    for (i=0; envs[i] != NULL; i++)
+        if ((tmpdir = getenv(envs[i])))
+            return tmpdir;
+
+    return P_tmpdir;
 }
 
 struct dirent_data {
     char *filename;
     const char *dirname;
-    struct stat *statbuf;
+    STATBUF *statbuf;
     long  ino; /* -1: inode not available */
     int   is_dir;/* 0 - no, 1 - yes */
 };
@@ -482,7 +487,7 @@ static inline int ft_skipped(enum filetype ft)
 
 #define FOLLOW_SYMLINK_MASK (CLI_FTW_FOLLOW_FILE_SYMLINK | CLI_FTW_FOLLOW_DIR_SYMLINK)
 static int get_filetype(const char *fname, int flags, int need_stat,
-			 struct stat *statbuf, enum filetype *ft)
+			 STATBUF *statbuf, enum filetype *ft)
 {
     int stated = 0;
 
@@ -496,7 +501,7 @@ static int get_filetype(const char *fname, int flags, int need_stat,
 	     * to lstat(), we can just stat() directly.*/
 	    if (*ft != ft_link) {
 		/* need to lstat to determine if it is a symlink */
-		if (lstat(fname, statbuf) == -1)
+		if (LSTAT(fname, statbuf) == -1)
 		    return -1;
 		if (S_ISLNK(statbuf->st_mode)) {
 		    *ft = ft_link;
@@ -515,7 +520,7 @@ static int get_filetype(const char *fname, int flags, int need_stat,
     }
 
     if (need_stat) {
-	if (stat(fname, statbuf) == -1)
+	if (STAT(fname, statbuf) == -1)
 	    return -1;
 	stated = 1;
     }
@@ -540,7 +545,7 @@ static int get_filetype(const char *fname, int flags, int need_stat,
 }
 
 static int handle_filetype(const char *fname, int flags,
-			   struct stat *statbuf, int *stated, enum filetype *ft,
+			   STATBUF *statbuf, int *stated, enum filetype *ft,
 			   cli_ftw_cb callback, struct cli_ftw_cbdata *data)
 {
     int ret;
@@ -576,7 +581,7 @@ static int handle_entry(struct dirent_data *entry, int flags, int maxdepth, cli_
 
 int cli_ftw(char *path, int flags, int maxdepth, cli_ftw_cb callback, struct cli_ftw_cbdata *data, cli_ftw_pathchk pathchk)
 {
-    struct stat statbuf;
+    STATBUF statbuf;
     enum filetype ft = ft_unknown;
     struct dirent_data entry;
     int stated = 0;
@@ -646,8 +651,8 @@ static int cli_ftw_dir(const char *dirname, int flags, int maxdepth, cli_ftw_cb
 	    int stated = 0;
 	    enum filetype ft;
 	    char *fname;
-	    struct stat statbuf;
-	    struct stat *statbufp;
+	    STATBUF statbuf;
+	    STATBUF *statbufp;
 
 	    if(!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, ".."))
 		continue;
@@ -859,7 +864,7 @@ char *cli_gentemp(const char *dir)
 
     mdir = dir ? dir : cli_gettmpdir();
 
-    len = strlen(mdir) + 42;
+    len = strlen(mdir) + 42 + 4;
     name = (char *) cli_calloc(len, sizeof(char));
     if(!name) {
 	cli_dbgmsg("cli_gentemp('%s'): out of memory\n", mdir);
@@ -887,7 +892,7 @@ char *cli_gentemp(const char *dir)
 	return NULL;
     }
 
-	snprintf(name, len, "%s"PATHSEP"clamav-%s", mdir, tmp);
+	snprintf(name, len, "%s"PATHSEP"clamav-%s.tmp", mdir, tmp);
     free(tmp);
 
     return(name);
