@@ -58,6 +58,8 @@
 #include "disasm.h"
 #include "special.h"
 #include "ishield.h"
+#include "asn1.h"
+#include "sha1.h"
 
 #define DCONF ctx->dconf->pe
 
@@ -76,8 +78,8 @@
 #define UPX_LZMA1 "\x56\x83\xc3\x04\x53\x50\xc7\x03\x03\x00\x02\x00\x90\x90\x90\x55\x57\x56\x53\x83"
 #define UPX_LZMA2 "\x56\x83\xc3\x04\x53\x50\xc7\x03\x03\x00\x02\x00\x90\x90\x90\x90\x90\x55\x57\x56"
 
-#define EC32(x) le32_to_host(x) /* Convert little endian to host */
-#define EC16(x) le16_to_host(x)
+#define EC32(x) ((uint32_t)cli_readint32(&(x))) /* Convert little endian to host */
+#define EC16(x) ((uint16_t)cli_readint16(&(x)))
 /* lower and upper bondary alignment (size vs offset) */
 #define PEALIGN(o,a) (((a))?(((o)/(a))*(a)):(o))
 #define PESALIGN(o,a) (((a))?(((o)/(a)+((o)%(a)!=0))*(a)):(o))
@@ -325,7 +327,7 @@ static int cli_ddump(int desc, int offset, int size, const char *file) {
 void findres(uint32_t by_type, uint32_t by_name, uint32_t res_rva, fmap_t *map, struct cli_exe_section *exe_sections, uint16_t nsections, uint32_t hdr_size, int (*cb)(void *, uint32_t, uint32_t, uint32_t, uint32_t), void *opaque) {
     unsigned int err = 0;
     uint32_t type, type_offs, name, name_offs, lang, lang_offs;
-    uint8_t *resdir, *type_entry, *name_entry, *lang_entry ;
+    const uint8_t *resdir, *type_entry, *name_entry, *lang_entry ;
     uint16_t type_cnt, name_cnt, lang_cnt;
 
     if (!(resdir = fmap_need_off_once(map, cli_rawaddr(res_rva, exe_sections, nsections, &err, map->len, hdr_size), 16)) || err)
@@ -388,35 +390,10 @@ void findres(uint32_t by_type, uint32_t by_name, uint32_t res_rva, fmap_t *map,
     }
 }
 
-static unsigned int cli_md5sect(fmap_t *map, struct cli_exe_section *s, unsigned char *digest) {
-    void *hashme;
-    cli_md5_ctx md5;
-
-    if (s->rsz > CLI_MAX_ALLOCATION) {
-	cli_dbgmsg("cli_md5sect: skipping md5 calculation for too big section\n");
-	return 0;
-    }
-
-    if(!s->rsz) return 0;
-    if(!(hashme=fmap_need_off_once(map, s->raw, s->rsz))) {
-	cli_dbgmsg("cli_md5sect: unable to read section data\n");
-	return 0;
-    }
-
-    cli_md5_init(&md5);
-    cli_md5_update(&md5, hashme, s->rsz);
-    cli_md5_final(digest, &md5);
-    cli_dbgmsg("MDB: %u:%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x\n",
-        s->rsz, digest[0], digest[1], digest[2], digest[3], digest[4], digest[5], digest[6],
-        digest[7], digest[8], digest[9], digest[10], digest[11], digest[12], digest[13], digest[14],
-        digest[15]);
-    return 1;
-}
-
 static void cli_parseres_special(uint32_t base, uint32_t rva, fmap_t *map, struct cli_exe_section *exe_sections, uint16_t nsections, size_t fsize, uint32_t hdr_size, unsigned int level, uint32_t type, unsigned int *maxres, struct swizz_stats *stats) {
     unsigned int err = 0, i;
-    uint8_t *resdir;
-    uint8_t *entry, *oentry;
+    const uint8_t *resdir;
+    const uint8_t *entry, *oentry;
     uint16_t named, unnamed;
     uint32_t rawaddr = cli_rawaddr(rva, exe_sections, nsections, &err, fsize, hdr_size);
     uint32_t entries;
@@ -485,7 +462,7 @@ static void cli_parseres_special(uint32_t base, uint32_t rva, fmap_t *map, struc
 			rawaddr = cli_rawaddr(base + offs, exe_sections, nsections, &err, fsize, hdr_size);
 			if (!err && (resdir = fmap_need_off_once(map, rawaddr, 16))) {
 				uint32_t isz = cli_readint32(resdir+4);
-				uint8_t *str;
+				const uint8_t *str;
 				rawaddr = cli_rawaddr(cli_readint32(resdir), exe_sections, nsections, &err, fsize, hdr_size);
 				if (err || !isz || isz >= fsize || rawaddr+isz >= fsize) {
 					cli_dbgmsg("cli_parseres_special: invalid resource table entry: %lu + %lu\n", 
@@ -504,6 +481,98 @@ static void cli_parseres_special(uint32_t base, uint32_t rva, fmap_t *map, struc
     fmap_unneed_ptr(map, oentry, entries*8);
 }
 
+static unsigned int cli_hashsect(fmap_t *map, struct cli_exe_section *s, unsigned char **digest, int * foundhash, int * foundwild)
+{
+    const void *hashme;
+    cli_md5_ctx md5;
+    SHA1Context sha1ctx;
+    SHA256_CTX sha256ctx;
+
+    if (s->rsz > CLI_MAX_ALLOCATION) {
+        cli_dbgmsg("cli_hashsect: skipping hash calculation for too big section\n");
+        return 0;
+    }
+
+    if(!s->rsz) return 0;
+    if(!(hashme=fmap_need_off_once(map, s->raw, s->rsz))) {
+        cli_dbgmsg("cli_hashsect: unable to read section data\n");
+        return 0;
+    }
+
+    if(foundhash[CLI_HASH_MD5] || foundwild[CLI_HASH_MD5]) {
+        cli_md5_init(&md5);
+        cli_md5_update(&md5, hashme, s->rsz);
+        cli_md5_final(digest[CLI_HASH_MD5], &md5);
+    }
+    if(foundhash[CLI_HASH_SHA1] || foundwild[CLI_HASH_SHA1]) {
+        SHA1Init(&sha1ctx);
+        SHA1Update(&sha1ctx, hashme, s->rsz);
+        SHA1Final(&sha1ctx, digest[CLI_HASH_SHA1]);
+    }
+    if(foundhash[CLI_HASH_SHA256] || foundwild[CLI_HASH_SHA256]) {
+        sha256_init(&sha256ctx);
+        sha256_update(&sha256ctx, hashme, s->rsz);
+        sha256_final(&sha256ctx, digest[CLI_HASH_SHA256]);
+    }
+
+    return 1;
+}
+
+/* check hash section sigs */
+static int scan_pe_mdb (cli_ctx * ctx, struct cli_exe_section *exe_section)
+{
+    struct cli_matcher * mdb_sect = ctx->engine->hm_mdb;
+    unsigned char * hashset[CLI_HASH_AVAIL_TYPES];
+    const char * virname = NULL;
+    int foundsize[CLI_HASH_AVAIL_TYPES];
+    int foundwild[CLI_HASH_AVAIL_TYPES];
+    enum CLI_HASH_TYPE type;
+    int ret = CL_CLEAN;
+ 
+    /* pick hashtypes to generate */
+    for(type = CLI_HASH_MD5; type < CLI_HASH_AVAIL_TYPES; type++) {
+        foundsize[type] = cli_hm_have_size(mdb_sect, type, exe_section->rsz);
+        foundwild[type] = cli_hm_have_wild(mdb_sect, type);
+        if(foundsize[type] || foundwild[type]) {
+            hashset[type] = cli_malloc(cli_hashlength(type));
+            if(!hashset[type]) {
+                cli_errmsg("scan_pe: cli_malloc failed!\n");
+                for(; type > 0;)
+                    free(hashset[--type]);
+                return CL_EMEM;
+            }
+        }
+        else {
+            hashset[type] = NULL;
+        }
+    }
+
+    /* Generate hashes */
+    cli_hashsect(*ctx->fmap, exe_section, hashset, foundsize, foundwild);
+
+    /* Do scans */
+    for(type = CLI_HASH_MD5; type < CLI_HASH_AVAIL_TYPES; type++) {
+       if(foundsize[type] && cli_hm_scan(hashset[type], exe_section->rsz, &virname, mdb_sect, type) == CL_VIRUS) {
+            cli_append_virus(ctx, virname);
+            ret = CL_VIRUS;
+            if (!SCAN_ALL) {
+                break;
+            }
+       }
+       if(foundwild[type] && cli_hm_scan_wild(hashset[type], &virname, mdb_sect, type) == CL_VIRUS) {
+            cli_append_virus(ctx, virname);
+            ret = CL_VIRUS;
+            if (!SCAN_ALL) {
+                break;
+            }
+       }
+    }
+
+    for(type = CLI_HASH_AVAIL_TYPES; type > 0;)
+        free(hashset[--type]);
+    return ret;
+}
+
 int cli_scanpe(cli_ctx *ctx)
 {
 	uint16_t e_magic; /* DOS signature ("MZ") */
@@ -523,13 +592,14 @@ int cli_scanpe(cli_ctx *ctx)
 	ssize_t bytes, at;
 	unsigned int i, found, upx_success = 0, min = 0, max = 0, err, overlays = 0;
 	unsigned int ssize = 0, dsize = 0, dll = 0, pe_plus = 0, corrupted_cur;
-	int (*upxfn)(char *, uint32_t, char *, uint32_t *, uint32_t, uint32_t, uint32_t) = NULL;
-	char *src = NULL, *dest = NULL;
+	int (*upxfn)(const char *, uint32_t, char *, uint32_t *, uint32_t, uint32_t, uint32_t) = NULL;
+	const char *src = NULL;
+	char *dest = NULL;
 	int ndesc, ret = CL_CLEAN, upack = 0, native=0;
 	size_t fsize;
 	uint32_t valign, falign, hdr_size, j;
 	struct cli_exe_section *exe_sections;
-	struct cli_matcher *md5_sect;
+	struct cli_matcher *mdb_sect;
 	char timestr[32];
 	struct pe_image_data_dir *dirs;
 	struct cli_bc_ctx *bc_ctx;
@@ -854,13 +924,13 @@ int cli_scanpe(cli_ctx *ctx)
     cli_dbgmsg("------------------------------------\n");
 
     if (DETECT_BROKEN_PE && !native && (!(pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment)) || (pe_plus?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment))%0x1000)) {
-        cli_dbgmsg("Bad virtual alignment\n");
+        cli_dbgmsg("Bad virtual alignemnt\n");
 	cli_append_virus(ctx,"Heuristics.Broken.Executable");
 	return CL_VIRUS;
     }
 
     if (DETECT_BROKEN_PE && !native && (!(pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment)) || (pe_plus?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment))%0x200)) {
-        cli_dbgmsg("Bad file alignment\n");
+        cli_dbgmsg("Bad file alignemnt\n");
 	cli_append_virus(ctx, "Heuristics.Broken.Executable");
 	return CL_VIRUS;
     }
@@ -954,9 +1024,10 @@ int cli_scanpe(cli_ctx *ctx)
 	if(exe_sections[i].chr & 0x80000000)
 	    cli_dbgmsg("Section's memory is writeable\n");
 
+	cli_dbgmsg("------------------------------------\n");
+
 	if (DETECT_BROKEN_PE && (!valign || (exe_sections[i].urva % valign))) { /* Bad virtual alignment */
 	    cli_dbgmsg("VirtualAddress is misaligned\n");
-	    cli_dbgmsg("------------------------------------\n");
 	    cli_append_virus(ctx, "Heuristics.Broken.Executable");
 	    free(section_hdr);
 	    free(exe_sections);
@@ -966,7 +1037,6 @@ int cli_scanpe(cli_ctx *ctx)
 	if (exe_sections[i].rsz) { /* Don't bother with virtual only sections */
 	    if (exe_sections[i].raw >= fsize) { /* really broken */
 	      cli_dbgmsg("Broken PE file - Section %d starts beyond the end of file (Offset@ %lu, Total filesize %lu)\n", i, (unsigned long)exe_sections[i].raw, (unsigned long)fsize);
-		cli_dbgmsg("------------------------------------\n");
 		free(section_hdr);
 		free(exe_sections);
 		if(DETECT_BROKEN_PE) {
@@ -978,30 +1048,19 @@ int cli_scanpe(cli_ctx *ctx)
 
 	    if(SCAN_ALGO && (DCONF & PE_CONF_POLIPOS) && !*sname && exe_sections[i].vsz > 40000 && exe_sections[i].vsz < 70000 && exe_sections[i].chr == 0xe0000060) polipos = i;
 
-	    /* check MD5 section sigs */
-	    md5_sect = ctx->engine->hm_mdb;
-	    if((DCONF & PE_CONF_MD5SECT) && md5_sect) {
-		unsigned char md5_dig[16];
-		if(cli_hm_have_size(md5_sect, CLI_HASH_MD5, exe_sections[i].rsz) && 
-		   cli_md5sect(map, &exe_sections[i], md5_dig) &&
-		   cli_hm_scan(md5_dig, exe_sections[i].rsz, &virname, md5_sect, CLI_HASH_MD5) == CL_VIRUS) {
-		    cli_append_virus(ctx, virname);
-		    if(cli_hm_scan(md5_dig, fsize, NULL, ctx->engine->hm_fp, CLI_HASH_MD5) != CL_VIRUS) {
-			if (!SCAN_ALL) {
-			    cli_dbgmsg("------------------------------------\n");
-			    free(section_hdr);
-			    free(exe_sections);
-			    return CL_VIRUS;
-			}
-		    }
-		    viruses_found++;
-		}
+	    /* check hash section sigs */
+	    if((DCONF & PE_CONF_MD5SECT) && ctx->engine->hm_mdb) {
+	        ret = scan_pe_mdb(ctx, &exe_sections[i]);
+	        if (ret != CL_CLEAN) {
+	            if (ret != CL_VIRUS)
+	                cli_errmsg("scan_pe: scan_pe_mdb failed: %s!\n", cl_strerror(ret));
+	            free(section_hdr);
+	            free(exe_sections);
+	            return ret;
+	        }
 	    }
-
 	}
 
-	cli_dbgmsg("------------------------------------\n");
-
 	if (exe_sections[i].urva>>31 || exe_sections[i].uvsz>>31 || (exe_sections[i].rsz && exe_sections[i].uraw>>31) || exe_sections[i].ursz>>31) {
 	    cli_dbgmsg("Found PE values with sign bit set\n");
 	    free(section_hdr);
@@ -1103,18 +1162,23 @@ int cli_scanpe(cli_ctx *ctx)
     bc_ctx = cli_bytecode_context_alloc();
     if (!bc_ctx) {
 	cli_errmsg("cli_scanpe: can't allocate memory for bc_ctx\n");
+	free(exe_sections);
 	return CL_EMEM;
     }
     cli_bytecode_context_setpe(bc_ctx, &pedata, exe_sections);
     cli_bytecode_context_setctx(bc_ctx, ctx);
     ret = cli_bytecode_runhook(ctx, ctx->engine, bc_ctx, BC_PE_ALL, map);
-    if (ret == CL_VIRUS || ret == CL_BREAK) {
-	free(exe_sections);
-	cli_bytecode_context_destroy(bc_ctx);
-	return ret == CL_VIRUS ? CL_VIRUS : CL_CLEAN;
+    switch (ret) {
+        case CL_ENULLARG:
+            cli_warnmsg("cli_scanpe: NULL argument supplied\n");
+            break;
+        case CL_VIRUS:
+        case CL_BREAK:
+            free(exe_sections);
+            cli_bytecode_context_destroy(bc_ctx);
+            return ret == CL_VIRUS ? CL_VIRUS : CL_CLEAN;
     }
     cli_bytecode_context_destroy(bc_ctx);
-
     /* Attempt to detect some popular polymorphic viruses */
 
     /* W32.Parite.B */
@@ -1210,9 +1274,10 @@ int cli_scanpe(cli_ctx *ctx)
 	    case KZSLOOP:
 		if (op==kzdsize+0x48 && *kzcode==0x75 && kzlen-(int8_t)kzcode[1]-3<=kzinitlen && kzlen-(int8_t)kzcode[1]>=kzxorlen) {
 		    cli_append_virus(ctx,"Heuristics.W32.Kriz");
-		    free(exe_sections);
-		    if (!SCAN_ALL)
+		    if (!SCAN_ALL) {
+		        free(exe_sections);
 			return CL_VIRUS;
+		    }
 		    viruses_found++;
 		}
 		cli_dbgmsg("kriz: loop out of bounds, corrupted sample?\n");
@@ -1234,28 +1299,30 @@ int cli_scanpe(cli_ctx *ctx)
 
 	if(vsize >= 0x612c && rsize >= 0x612c && ((vsize & 0xff) == 0xec)) {
 		int bw = rsize < 0x7000 ? rsize : 0x7000;
-		char *tbuff;
+		const char *tbuff;
 
 	    if((tbuff = fmap_need_off_once(map, exe_sections[nsections - 1].raw + rsize - bw, 4096))) {
 		if(cli_memstr(tbuff, 4091, "\xe8\x2c\x61\x00\x00", 5)) {
 		    cli_append_virus(ctx, dam ? "Heuristics.W32.Magistr.A.dam" : "Heuristics.W32.Magistr.A");
-		    free(exe_sections);
-		    if (!SCAN_ALL)
+		    if (!SCAN_ALL) {
+		        free(exe_sections);
 			return CL_VIRUS;
+		    }
 		    viruses_found++;
 		}
 	    }
 
 	} else if(rsize >= 0x7000 && vsize >= 0x7000 && ((vsize & 0xff) == 0xed)) {
 		int bw = rsize < 0x8000 ? rsize : 0x8000;
-		char *tbuff;
+		const char *tbuff;
 
 	    if((tbuff = fmap_need_off_once(map, exe_sections[nsections - 1].raw + rsize - bw, 4096))) {
 		if(cli_memstr(tbuff, 4091, "\xe8\x04\x72\x00\x00", 5)) {
 		    cli_append_virus(ctx,dam ? "Heuristics.W32.Magistr.B.dam" : "Heuristics.W32.Magistr.B");
-		    free(exe_sections);
-		    if (!SCAN_ALL)
+		    if (!SCAN_ALL) {
+		        free(exe_sections);
 			return CL_VIRUS;
+		    }
 		    viruses_found++;
 		} 
 	    }
@@ -1265,7 +1332,7 @@ int cli_scanpe(cli_ctx *ctx)
     /* W32.Polipos.A */
     while(polipos && !dll && nsections > 2 && nsections < 13 && e_lfanew <= 0x800 && (EC16(optional_hdr32.Subsystem) == 2 || EC16(optional_hdr32.Subsystem) == 3) && EC16(file_hdr.Machine) == 0x14c && optional_hdr32.SizeOfStackReserve >= 0x80000) {
 	uint32_t jump, jold, *jumps = NULL;
-	uint8_t *code;
+	const uint8_t *code;
 	unsigned int xsjs = 0;
 
 	if(exe_sections[0].rsz > CLI_MAX_ALLOCATION) break;
@@ -1303,10 +1370,11 @@ int cli_scanpe(cli_ctx *ctx)
 	    if(!(code = fmap_need_off_once(map, jumps[i], 9))) continue;
 	    if((jump=cli_readint32(code))==0x60ec8b55 || (code[4]==0x0ec && ((jump==0x83ec8b55 && code[6]==0x60) || (jump==0x81ec8b55 && !code[7] && !code[8])))) {
 		cli_append_virus(ctx,"Heuristics.W32.Polipos.A");
-		free(jumps);
-		free(exe_sections);
-		if (!SCAN_ALL)
+		if (!SCAN_ALL) {
+		    free(jumps);
+		    free(exe_sections);
 		    return CL_VIRUS;
+		}
 		viruses_found++;
 	    }
 	}
@@ -1363,10 +1431,11 @@ int cli_scanpe(cli_ctx *ctx)
     /* MEW support */
     if (found && (DCONF & PE_CONF_MEW) && epsize>=16 && epbuff[0]=='\xe9') {
 	uint32_t fileoffset;
-	char *tbuff;
+	const char *tbuff;
 
 	fileoffset = (vep + cli_readint32(epbuff + 1) + 5);
 	while (fileoffset == 0x154 || fileoffset == 0x158) {
+	    char *src;
 	    uint32_t offdiff, uselzma;
 
 	    cli_dbgmsg ("MEW: found MEW characteristics %08X + %08X + 5 = %08X\n", 
@@ -1539,7 +1608,8 @@ int cli_scanpe(cli_ctx *ctx)
     }
 
     
-    while(found && (DCONF & PE_CONF_FSG) && epbuff[0] == '\x87' && epbuff[1] == '\x25') {
+    while(found  && (DCONF & PE_CONF_FSG) && epbuff[0] == '\x87' && epbuff[1] == '\x25') {
+	const char *dst;
 
 	/* FSG v2.0 support - thanks to aCaB ! */
 
@@ -1568,28 +1638,28 @@ int cli_scanpe(cli_ctx *ctx)
 	    return CL_ESEEK;
 	}
 
-	dest = src + newedx - exe_sections[i + 1].rva;
-	if(newedx < exe_sections[i + 1].rva || !CLI_ISCONTAINED(src, ssize, dest, 4)) {
+	dst = src + newedx - exe_sections[i + 1].rva;
+	if(newedx < exe_sections[i + 1].rva || !CLI_ISCONTAINED(src, ssize, dst, 4)) {
 	    cli_dbgmsg("FSG: New ESP out of bounds\n");
 	    break;
 	}
 
-	newedx = cli_readint32(dest) - EC32(optional_hdr32.ImageBase);
+	newedx = cli_readint32(dst) - EC32(optional_hdr32.ImageBase);
 	if(!CLI_ISCONTAINED(exe_sections[i + 1].rva, exe_sections[i + 1].rsz, newedx, 4)) {
 	    cli_dbgmsg("FSG: New ESP (%x) is wrong\n", newedx);
 	    break;
 	}
  
-	dest = src + newedx - exe_sections[i + 1].rva;
-	if(!CLI_ISCONTAINED(src, ssize, dest, 32)) {
+	dst = src + newedx - exe_sections[i + 1].rva;
+	if(!CLI_ISCONTAINED(src, ssize, dst, 32)) {
 	    cli_dbgmsg("FSG: New stack out of bounds\n");
 	    break;
 	}
 
-	newedi = cli_readint32(dest) - EC32(optional_hdr32.ImageBase);
-	newesi = cli_readint32(dest + 4) - EC32(optional_hdr32.ImageBase);
-	newebx = cli_readint32(dest + 16) - EC32(optional_hdr32.ImageBase);
-	newedx = cli_readint32(dest + 20);
+	newedi = cli_readint32(dst) - EC32(optional_hdr32.ImageBase);
+	newesi = cli_readint32(dst + 4) - EC32(optional_hdr32.ImageBase);
+	newebx = cli_readint32(dst + 16) - EC32(optional_hdr32.ImageBase);
+	newedx = cli_readint32(dst + 20);
 
 	if(newedi != exe_sections[i].rva) {
 	    cli_dbgmsg("FSG: Bad destination buffer (edi is %x should be %x)\n", newedi, exe_sections[i].rva);
@@ -1611,7 +1681,6 @@ int cli_scanpe(cli_ctx *ctx)
 
 	if((dest = (char *) cli_calloc(dsize, sizeof(char))) == NULL) {
 	    free(exe_sections);
-	    free(src);
 	    return CL_EMEM;
 	}
 
@@ -1626,7 +1695,7 @@ int cli_scanpe(cli_ctx *ctx)
 	/* FSG support - v. 1.33 (thx trog for the many samples) */
 
 	int sectcnt = 0;
-	char *support;
+	const char *support;
 	uint32_t newesi, newedi, oldep, gp, t;
 	struct cli_exe_section *sections;
 
@@ -1693,6 +1762,7 @@ int cli_scanpe(cli_ctx *ctx)
 	}
 
 	if((sections = (struct cli_exe_section *) cli_malloc((sectcnt + 1) * sizeof(struct cli_exe_section))) == NULL) {
+        cli_errmsg("FSG: Unable to allocate memory for sections %u\n", (sectcnt + 1) * sizeof(struct cli_exe_section));
 	    free(exe_sections);
 	    return CL_EMEM;
 	}
@@ -1729,7 +1799,7 @@ int cli_scanpe(cli_ctx *ctx)
 
 	int sectcnt = 0;
 	uint32_t gp, t = cli_rawaddr(cli_readint32(epbuff+1) - EC32(optional_hdr32.ImageBase), NULL, 0 , &err, fsize, hdr_size);
-	char *support;
+	const char *support;
 	uint32_t newesi = cli_readint32(epbuff+11) - EC32(optional_hdr32.ImageBase);
 	uint32_t newedi = cli_readint32(epbuff+6) - EC32(optional_hdr32.ImageBase);
 	uint32_t oldep = vep - exe_sections[i + 1].rva;
@@ -1792,6 +1862,7 @@ int cli_scanpe(cli_ctx *ctx)
 	}
 
 	if((sections = (struct cli_exe_section *) cli_malloc((sectcnt + 1) * sizeof(struct cli_exe_section))) == NULL) {
+        cli_errmsg("FSG: Unable to allocate memory for sections %u\n", (sectcnt + 1) * sizeof(struct cli_exe_section));
 	    free(exe_sections);
 	    return CL_EMEM;
 	}
@@ -1832,6 +1903,8 @@ int cli_scanpe(cli_ctx *ctx)
 	ssize = exe_sections[i + 1].rsz;
 	dsize = exe_sections[i].vsz + exe_sections[i + 1].vsz;
 
+        /* cli_dbgmsg("UPX: ssize %u dsize %u\n", ssize, dsize); */
+
 	CLI_UNPSIZELIMITS("UPX", MAX(dsize, ssize));
 
 	if(ssize <= 0x19 || dsize <= ssize || dsize > CLI_MAX_ALLOCATION ) {
@@ -1868,19 +1941,18 @@ int cli_scanpe(cli_ctx *ctx)
 
 	    if(epbuff[1] != '\xbe' || skew <= 0 || skew > 0xfff) { /* FIXME: legit skews?? */
 		skew = 0; 
-	    }
-	    else if(skew > ssize) {
+	    } else if (skew > ssize) {
 		/* Ignore suggested skew larger than section size */
-		cli_dbgmsg("UPX: Ignoring bad skew of %d bytes\n", skew);
 		skew = 0;
-	    }
-	    else {
+	    } else {
 		cli_dbgmsg("UPX: UPX1 seems skewed by %d bytes\n", skew);
 	    }
 
-	    if(upxfn(src + skew, ssize - skew, dest, &dsize, exe_sections[i].rva, exe_sections[i + 1].rva, vep-skew) >= 0 || upxfn(src, ssize, dest, &dsize, exe_sections[i].rva, exe_sections[i + 1].rva, vep) >= 0) {
+	    /* Try skewed first (skew may be zero) */
+	    if(upxfn(src + skew, ssize - skew, dest, &dsize, exe_sections[i].rva, exe_sections[i + 1].rva, vep-skew) >= 0) {
 		upx_success = 1;
 	    }
+	    /* If skew not successful and non-zero, try no skew */
 	    else if(skew && (upxfn(src, ssize, dest, &dsize, exe_sections[i].rva, exe_sections[i + 1].rva, vep) >= 0)) {
 		upx_success = 1;
 	    }
@@ -1958,7 +2030,14 @@ int cli_scanpe(cli_ctx *ctx)
 	}
 
 	free(dest);
-	lseek(ndesc, 0, SEEK_SET);
+	if (lseek(ndesc, 0, SEEK_SET) == -1) {
+        cli_dbgmsg("UPX/FSG: lseek() failed\n");
+        close(ndesc);
+        CLI_TMPUNLK();
+        free(tempfile);
+        SHA_RESET;
+        return CL_ESEEK;
+    }
 
 	if(ctx->engine->keeptmp)
 	    cli_dbgmsg("UPX/FSG: Decompressed data saved in %s\n", tempfile);
@@ -2040,6 +2119,7 @@ int cli_scanpe(cli_ctx *ctx)
 	CLI_UNPSIZELIMITS("PEspin", fsize);
 
 	if((spinned = (char *) cli_malloc(fsize)) == NULL) {
+        cli_errmsg("PESping: Unable to allocate memory for spinned %lu\n", (unsigned long)fsize);
 	    free(exe_sections);
 	    return CL_EMEM;
 	}
@@ -2103,6 +2183,7 @@ int cli_scanpe(cli_ctx *ctx)
 	    char *spinned;
 
 	    if((spinned = (char *) cli_malloc(fsize)) == NULL) {
+            cli_errmsg("yC: Unable to allocate memory for spinned %lu\n", (unsigned long)fsize);
 	      free(exe_sections);
 	      return CL_EMEM;
 	    }
@@ -2128,6 +2209,7 @@ int cli_scanpe(cli_ctx *ctx)
        memcmp(epbuff+0x68, "\xe8\x00\x00\x00\x00\x58\x2d\x6d\x00\x00\x00\x50\x60\x33\xc9\x50\x58\x50\x50", 19) == 0)  {
 	uint32_t head = exe_sections[nsections - 1].raw;
         uint8_t *packer;
+	char *src;
 
 	ssize = 0;
 	for(i=0 ; ; i++) {
@@ -2182,6 +2264,7 @@ int cli_scanpe(cli_ctx *ctx)
 
     /* ASPACK support */
     while((DCONF & PE_CONF_ASPACK) && ep+58+0x70e < fsize && !memcmp(epbuff,"\x60\xe8\x03\x00\x00\x00\xe9\xeb",8)) {
+	char *src;
 
         if(epsize<0x3bf || memcmp(epbuff+0x3b9, "\x68\x00\x00\x00\x00\xc3",6)) break;
 	ssize = 0;
@@ -2218,7 +2301,7 @@ int cli_scanpe(cli_ctx *ctx)
 	uint32_t eprva = vep;
 	uint32_t start_of_stuff, rep = ep;
 	unsigned int nowinldr;
-	char *nbuff;
+	const char *nbuff;
 
 	src=epbuff;
 	if (*epbuff=='\xe9') { /* bitched headers */
@@ -2248,7 +2331,10 @@ int cli_scanpe(cli_ctx *ctx)
 	CLI_UNPSIZELIMITS("NsPack", MAX(ssize,dsize));
 
 	if (!ssize || !dsize || dsize != exe_sections[0].vsz) break;
-	if (!(dest=cli_malloc(dsize))) break;
+	if (!(dest=cli_malloc(dsize))) {
+        cli_errmsg("NsPack: Unable to allocate memory for dest %u\n", dsize);
+        break;
+    }
 	/* memset(dest, 0xfc, dsize); */
 
 	if(!(src = fmap_need_off(map, start_of_stuff, ssize))) {
@@ -2477,7 +2563,7 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 
     while(dirs[2].Size) {
 	struct vinfo_list vlist;
-	uint8_t *vptr, *baseptr;
+	const uint8_t *vptr, *baseptr;
     	uint32_t rva, res_sz;
 
 	memset(&vlist, 0, sizeof(vlist));
@@ -2561,7 +2647,7 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
 
 		    while(sfi_sz > 6) { /* enum all stringtables - RESUMABLE */
 			uint32_t st_sz = cli_readint32(vptr) & 0xffff;
-			uint8_t *next_vptr = vptr + st_sz;
+			const uint8_t *next_vptr = vptr + st_sz;
 			uint32_t next_sfi_sz = sfi_sz - st_sz;
 
 			if(st_sz > sfi_sz || st_sz <= 24) {
@@ -2659,3 +2745,209 @@ int cli_peheader(fmap_t *map, struct cli_exe_info *peinfo)
     free(section_hdr);
     return 0;
 }
+
+
+static int sort_sects(const void *first, const void *second) {
+    const struct cli_exe_section *a = first, *b = second;
+    return (a->raw - b->raw);
+}
+
+int cli_checkfp_pe(cli_ctx *ctx, uint8_t *authsha1) {
+    uint16_t e_magic; /* DOS signature ("MZ") */
+    uint16_t nsections;
+    uint32_t e_lfanew; /* address of new exe header */
+    struct pe_image_file_hdr file_hdr;
+    union {
+	struct pe_image_optional_hdr64 opt64;
+	struct pe_image_optional_hdr32 opt32;
+    } pe_opt;
+    const struct pe_image_section_hdr *section_hdr;
+    ssize_t at;
+    unsigned int i, pe_plus = 0, hlen;
+    size_t fsize;
+    uint32_t valign, falign, hdr_size;
+    struct cli_exe_section *exe_sections;
+    struct pe_image_data_dir *dirs;
+    fmap_t *map = *ctx->fmap;
+    SHA1Context sha1;
+
+    if(!(DCONF & PE_CONF_CATALOG))
+	return CL_EFORMAT;
+
+    if(fmap_readn(map, &e_magic, 0, sizeof(e_magic)) != sizeof(e_magic))
+	return CL_EFORMAT;
+
+    if(EC16(e_magic) != PE_IMAGE_DOS_SIGNATURE && EC16(e_magic) != PE_IMAGE_DOS_SIGNATURE_OLD)
+	return CL_EFORMAT;
+
+    if(fmap_readn(map, &e_lfanew, 58 + sizeof(e_magic), sizeof(e_lfanew)) != sizeof(e_lfanew))
+	return CL_EFORMAT;
+
+    e_lfanew = EC32(e_lfanew);
+    if(!e_lfanew)
+	return CL_EFORMAT;
+
+    if(fmap_readn(map, &file_hdr, e_lfanew, sizeof(struct pe_image_file_hdr)) != sizeof(struct pe_image_file_hdr))
+	return CL_EFORMAT;
+
+    if(EC32(file_hdr.Magic) != PE_IMAGE_NT_SIGNATURE)
+	return CL_EFORMAT;
+
+    nsections = EC16(file_hdr.NumberOfSections);
+    if(nsections < 1 || nsections > 96)
+	return CL_EFORMAT;
+
+    if(EC16(file_hdr.SizeOfOptionalHeader) < sizeof(struct pe_image_optional_hdr32))
+	return CL_EFORMAT;
+
+    at = e_lfanew + sizeof(struct pe_image_file_hdr);
+    if(fmap_readn(map, &optional_hdr32, at, sizeof(struct pe_image_optional_hdr32)) != sizeof(struct pe_image_optional_hdr32))
+	return CL_EFORMAT;
+    at += sizeof(struct pe_image_optional_hdr32);
+
+    /* This will be a chicken and egg problem until we drop 9x */
+    if(EC16(optional_hdr64.Magic)==PE32P_SIGNATURE) {
+        if(EC16(file_hdr.SizeOfOptionalHeader)!=sizeof(struct pe_image_optional_hdr64))
+	    return CL_EFORMAT;
+	pe_plus = 1;
+    }
+
+    if(!pe_plus) { /* PE */
+	if (EC16(file_hdr.SizeOfOptionalHeader)!=sizeof(struct pe_image_optional_hdr32)) {
+	    /* Seek to the end of the long header */
+	    at += EC16(file_hdr.SizeOfOptionalHeader)-sizeof(struct pe_image_optional_hdr32);
+	}
+
+	hdr_size = EC32(optional_hdr32.SizeOfHeaders);
+	dirs = optional_hdr32.DataDirectory;
+    } else { /* PE+ */
+        /* read the remaining part of the header */
+        if(fmap_readn(map, &optional_hdr32 + 1, at, sizeof(struct pe_image_optional_hdr64) - sizeof(struct pe_image_optional_hdr32)) != sizeof(struct pe_image_optional_hdr64) - sizeof(struct pe_image_optional_hdr32))
+	    return CL_EFORMAT;
+	at += sizeof(struct pe_image_optional_hdr64) - sizeof(struct pe_image_optional_hdr32);
+	hdr_size = EC32(optional_hdr64.SizeOfHeaders);
+	dirs = optional_hdr64.DataDirectory;
+    }
+
+    if(!cli_hm_have_size(ctx->engine->hm_fp, CLI_HASH_SHA1, 2) && dirs[4].Size < 8)
+	return CL_BREAK;
+
+    fsize = map->len;
+
+    valign = (pe_plus)?EC32(optional_hdr64.SectionAlignment):EC32(optional_hdr32.SectionAlignment);
+    falign = (pe_plus)?EC32(optional_hdr64.FileAlignment):EC32(optional_hdr32.FileAlignment);
+
+    section_hdr = fmap_need_off_once(map, at, sizeof(*section_hdr) * nsections);
+    if(!section_hdr)
+	return CL_EFORMAT;
+    at += sizeof(*section_hdr) * nsections;
+
+    exe_sections = (struct cli_exe_section *) cli_calloc(nsections, sizeof(struct cli_exe_section));
+    if(!exe_sections)
+	return CL_EMEM;
+
+    for(i = 0; falign!=0x200 && i<nsections; i++) {
+	/* file alignment fallback mode - blah */
+	if (falign && section_hdr[i].SizeOfRawData && EC32(section_hdr[i].PointerToRawData)%falign && !(EC32(section_hdr[i].PointerToRawData)%0x200))
+	    falign = 0x200;
+    }
+
+    hdr_size = PESALIGN(hdr_size, falign); /* Aligned headers virtual size */
+
+    for(i = 0; i < nsections; i++) {
+	exe_sections[i].rva = PEALIGN(EC32(section_hdr[i].VirtualAddress), valign);
+	exe_sections[i].vsz = PESALIGN(EC32(section_hdr[i].VirtualSize), valign);
+	exe_sections[i].raw = PEALIGN(EC32(section_hdr[i].PointerToRawData), falign);
+	exe_sections[i].rsz = PESALIGN(EC32(section_hdr[i].SizeOfRawData), falign);
+
+	if (!exe_sections[i].vsz && exe_sections[i].rsz)
+	    exe_sections[i].vsz=PESALIGN(exe_sections[i].ursz, valign);
+
+	if (exe_sections[i].rsz && fsize>exe_sections[i].raw && !CLI_ISCONTAINED(0, (uint32_t) fsize, exe_sections[i].raw, exe_sections[i].rsz))
+	    exe_sections[i].rsz = fsize - exe_sections[i].raw;
+
+	if (exe_sections[i].rsz && exe_sections[i].raw >= fsize) {
+		free(exe_sections);
+		return CL_EFORMAT;
+	}
+
+	if (exe_sections[i].urva>>31 || exe_sections[i].uvsz>>31 || (exe_sections[i].rsz && exe_sections[i].uraw>>31) || exe_sections[i].ursz>>31) {
+	    free(exe_sections);
+	    return CL_EFORMAT;
+	}
+    }
+
+    cli_qsort(exe_sections, nsections, sizeof(*exe_sections), sort_sects);
+
+    SHA1Init(&sha1);
+
+#define hash_chunk(where, size)					\
+    do {							\
+	const uint8_t *hptr;					\
+	if(!(size)) break;					\
+	if(!(hptr = fmap_need_off_once(map, where, size))){	\
+	    free(exe_sections);					\
+	    return CL_EFORMAT;					\
+	}							\
+	SHA1Update(&sha1, hptr, size);				\
+    } while(0)
+
+    /* MZ to checksum */
+    at = 0;
+    hlen = e_lfanew + sizeof(struct pe_image_file_hdr) + (pe_plus ? offsetof(struct pe_image_optional_hdr64, CheckSum) : offsetof(struct pe_image_optional_hdr32, CheckSum));
+    hash_chunk(0, hlen);
+    at = hlen + 4;
+
+    /* Checksum to security */
+    if(pe_plus)
+	hlen = offsetof(struct pe_image_optional_hdr64, DataDirectory[4]) - offsetof(struct pe_image_optional_hdr64, CheckSum) - 4;
+    else
+	hlen = offsetof(struct pe_image_optional_hdr32, DataDirectory[4]) - offsetof(struct pe_image_optional_hdr32, CheckSum) - 4;
+    hash_chunk(at, hlen);
+    at += hlen + 8;
+
+    if(at > hdr_size) {
+	free(exe_sections);
+	return CL_EFORMAT;
+    }
+
+    /* Security to End of header */
+    hlen = hdr_size - at;
+    hash_chunk(at, hlen);
+
+    /* Sections */
+    at = hdr_size;
+    for(i = 0; i < nsections; i++) {
+	if(!exe_sections[i].rsz)
+	    continue;
+	hash_chunk(exe_sections[i].raw, exe_sections[i].rsz);
+	at += exe_sections[i].rsz;
+    }
+
+    if(at < fsize) {
+	hlen = fsize - at;
+	if(dirs[4].Size > hlen) {
+	    free(exe_sections);
+	    return CL_EFORMAT;
+	}
+	hlen -= dirs[4].Size;
+	hash_chunk(at, hlen);
+	at += hlen;
+    }
+    free(exe_sections);
+
+    SHA1Final(&sha1, authsha1);
+
+    if(cli_debug_flag) {
+	char shatxt[SHA1_HASH_SIZE*2+1];
+	for(i=0; i<SHA1_HASH_SIZE; i++)
+	    sprintf(&shatxt[i*2], "%02x", authsha1[i]);
+	cli_dbgmsg("Authenticode: %s\n", shatxt);
+    }
+
+    hlen = dirs[4].Size;
+    if(hlen < 8)
+	return CL_VIRUS;
+    hlen -= 8;
+    return asn1_check_mscat((struct cl_engine *)(ctx->engine), map, at + 8, hlen, authsha1);
+}
