@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007-2010 Sourcefire, Inc.
+ *  Copyright (C) 2007-2012 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -59,6 +59,7 @@
 #include "md5.h"
 #include "sha256.h"
 #include "dsig.h"
+#include "asn1.h"
 
 #include "phishcheck.h"
 #include "phish_whitelist.h"
@@ -112,8 +113,9 @@ char *cli_virname(char *virname, unsigned int official)
 int cli_parse_add(struct cli_matcher *root, const char *virname, const char *hexsig, uint16_t rtype, uint16_t type, const char *offset, uint8_t target, const uint32_t *lsigid, unsigned int options)
 {
 	struct cli_bm_patt *bm_new;
-	char *pt, *hexcpy, *start, *n;
-	int ret, asterisk = 0;
+	char *pt, *hexcpy, *start, *n, l, r;
+	const char *wild;
+	int ret, asterisk = 0, range;
 	unsigned int i, j, hexlen, parts = 0;
 	int mindist = 0, maxdist = 0, error = 0;
 
@@ -151,7 +153,7 @@ int cli_parse_add(struct cli_matcher *root, const char *virname, const char *hex
 	patt->length = root->ac_mindepth;
 	/* dummy */
 	patt->pattern = mpool_calloc(root->mempool, patt->length, sizeof(*patt->pattern));
-	if (patt->pattern) {
+	if (!patt->pattern) {
 	    free(patt);
 	    return CL_EMEM;
 	}
@@ -162,7 +164,24 @@ int cli_parse_add(struct cli_matcher *root, const char *virname, const char *hex
 	}
 	return CL_SUCCESS;
     }
-    if(strchr(hexsig, '{')) {
+    if((wild = strchr(hexsig, '{'))) {
+	if(sscanf(wild, "%c%u%c", &l, &range, &r) == 3 && l == '{' && r == '}' && range > 0 && range < 128) {
+	    hexcpy = cli_calloc(hexlen + 2 * range, sizeof(char));
+	    if(!hexcpy)
+		return CL_EMEM;
+	    strncpy(hexcpy, hexsig, wild - hexsig);
+	    for(i = 0; i < (unsigned int) range; i++)
+		strcat(hexcpy, "??");
+	    if(!(wild = strchr(wild, '}'))) {
+		cli_errmsg("cli_parse_add(): Problem adding signature: missing bracket\n");
+		free(hexcpy);
+		return CL_EMALFDB;
+	    }
+	    strcat(hexcpy, ++wild);
+	    ret = cli_parse_add(root, virname, hexcpy, rtype, type, offset, target, lsigid, options);
+	    free(hexcpy);
+	    return ret;
+	}
 
 	root->ac_partsigs++;
 
@@ -255,8 +274,10 @@ int cli_parse_add(struct cli_matcher *root, const char *virname, const char *hex
 	}
 
 	free(hexcpy);
-	if(error)
+	if(error) {
+	    cli_errmsg("cli_parseadd(): Problem adding signature (1b).\n");
 	    return CL_EMALFDB;
+	}
 
     } else if(strchr(hexsig, '*')) {
 	root->ac_partsigs++;
@@ -387,7 +408,7 @@ char *cli_dbgets(char *buff, unsigned int size, FILE *fs, struct cli_dbio *dbio)
 		} else {
 		    bread = fread(dbio->readpt, 1, dbio->readsize, dbio->fs);
 		    if(!bread && ferror(dbio->fs)) {
-			cli_errmsg("cli_dbgets: gzread() failed\n");
+			cli_errmsg("cli_dbgets: fread() failed\n");
 			return NULL;
 		    }
 		}
@@ -533,8 +554,10 @@ static int cli_loaddb(FILE *fs, struct cl_engine *engine, unsigned int *signo, u
     root = engine->root[0];
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loaddb: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -557,7 +580,7 @@ static int cli_loaddb(FILE *fs, struct cl_engine *engine, unsigned int *signo, u
 	if(engine->ignored && cli_chkign(engine->ignored, start, buffer_cpy))
 	    continue;
 
-	if(engine->cb_sigload && engine->cb_sigload("db", start, engine->cb_sigload_ctx)) {
+	if(engine->cb_sigload && engine->cb_sigload("db", start, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	    cli_dbgmsg("cli_loaddb: skipping %s due to callback\n", start);
 	    continue;
 	}
@@ -565,6 +588,7 @@ static int cli_loaddb(FILE *fs, struct cl_engine *engine, unsigned int *signo, u
 	if(*pt == '=') continue;
 
 	if((ret = cli_parse_add(root, start, pt, 0, 0, "*", 0, NULL, options))) {
+	    cli_dbgmsg("cli_loaddb: cli_parse_add failed on line %d\n", line);
 	    ret = CL_EMALFDB;
 	    break;
 	}
@@ -607,6 +631,7 @@ static int cli_loadidb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     
     if(engine->ignored)
 	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loadidb: Can't allocate memory for buffer_cpy\n");
 	    mpool_free(engine->mempool, matcher);
 	    return CL_EMEM;
 	}
@@ -636,7 +661,7 @@ static int cli_loadidb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	if(engine->ignored && cli_chkign(engine->ignored, tokens[0], buffer_cpy))
 	    continue;
 
-	if(engine->cb_sigload && engine->cb_sigload("idb", tokens[0], engine->cb_sigload_ctx)) {
+	if(engine->cb_sigload && engine->cb_sigload("idb", tokens[0], ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	    cli_dbgmsg("cli_loadidb: skipping %s due to callback\n", tokens[0]);
 	    continue;
 	}
@@ -881,8 +906,10 @@ static int cli_loadndb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	return ret;
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loadndb: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -912,7 +939,7 @@ static int cli_loadndb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	if(engine->ignored && cli_chkign(engine->ignored, virname, buffer_cpy))
 	    continue;
 
-	if(!sdb && engine->cb_sigload && engine->cb_sigload("ndb", virname, engine->cb_sigload_ctx)) {
+	if(!sdb && engine->cb_sigload && engine->cb_sigload("ndb", virname, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	    cli_dbgmsg("cli_loadndb: skipping %s due to callback\n", virname);
 	    continue;
 	}
@@ -1143,6 +1170,11 @@ static int lsigattribs(char *attribs, struct cli_lsig_tdb *tdb)
 		memcpy(&tdb->str[cnt], pt, strlen(pt));
 		tdb->str[tdb->cnt[CLI_TDB_STR] - 1] = 0;
 		break;
+
+	    default:
+		/* All known TDB types handled above, skip unknown */
+		cli_dbgmsg("lsigattribs: Unknown attribute type '%u'\n", apt->type);
+		return 1; /* +1 = skip */
 	}
     }
 
@@ -1224,7 +1256,7 @@ static int load_oneldb(char *buffer, int chkpua, struct cl_engine *engine, unsig
 	return CL_SUCCESS;
     }
 
-    if(engine->cb_sigload && engine->cb_sigload("ldb", virname, engine->cb_sigload_ctx)) {
+    if(engine->cb_sigload && engine->cb_sigload("ldb", virname, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	cli_dbgmsg("cli_loadldb: skipping %s due to callback\n", virname);
 	(*sigs)--;
 	return CL_SUCCESS;
@@ -1378,8 +1410,10 @@ static int cli_loadldb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	return ret;
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(sizeof(buffer))))
+	if(!(buffer_cpy = cli_malloc(sizeof(buffer)))) {
+        cli_errmsg("cli_loadldb: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
     while(cli_dbgets(buffer, sizeof(buffer), fs, dbio)) {
 	line++;
 	if(buffer[0] == '#')
@@ -1433,7 +1467,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	return CL_SUCCESS;
     }
 
-    if(engine->cb_sigload && engine->cb_sigload("cbc", dbname, engine->cb_sigload_ctx)) {
+    if(engine->cb_sigload && engine->cb_sigload("cbc", dbname, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	cli_dbgmsg("cli_loadcbc: skipping %s due to callback\n", dbname);
 	return CL_SUCCESS;
     }
@@ -1460,7 +1494,7 @@ static int cli_loadcbc(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    security_trust = 0;
     }
 
-    rc = cli_bytecode_load(bc, fs, dbio, security_trust);
+    rc = cli_bytecode_load(bc, fs, dbio, security_trust, options&CL_DB_BYTECODE_STATS);
     /* read remainder of DB, needed because cvd.c checks that we read the entire
      * file */
     while (cli_dbgets(buf, sizeof(buf), fs, dbio)) {}
@@ -1693,7 +1727,7 @@ static int cli_loadinfo(FILE *fs, struct cl_engine *engine, unsigned int options
     sha256_init(&ctx);
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
-	if(!strncmp(buffer, "DSIG:", 5)) {
+	if(!(options & CL_DB_UNSIGNED) && !strncmp(buffer, "DSIG:", 5)) {
 	    dsig = 1;
 	    sha256_final(&ctx, hash);
 	    if(cli_versig2(hash, buffer + 5, INFO_NSTR, INFO_ESTR) != CL_SUCCESS) {
@@ -1703,11 +1737,16 @@ static int cli_loadinfo(FILE *fs, struct cl_engine *engine, unsigned int options
 	    break;
 	}
 	len = strlen(buffer);
-	if(dbio->usebuf && buffer[len - 1] != '\n' && len + 1 < FILEBUFF) {
-	    /* cli_dbgets in buffered mode strips \n */
-	    buffer[len] = '\n';
-	    buffer[len + 1] = 0;
-	}
+    if (!len) {
+        buffer[len] = '\n';
+        buffer[len+1] = 0;
+    } else {
+        if(dbio->usebuf && buffer[len - 1] != '\n' && len + 1 < FILEBUFF) {
+            /* cli_dbgets in buffered mode strips \n */
+            buffer[len] = '\n';
+            buffer[len + 1] = 0;
+        }
+    }
 	sha256_update(&ctx, buffer, strlen(buffer));
 	cli_chomp(buffer);
 	if(!strncmp("ClamAV-VDB:", buffer, 11)) {
@@ -1772,7 +1811,7 @@ static int cli_loadinfo(FILE *fs, struct cl_engine *engine, unsigned int options
 	last = new;
     }
 
-    if(!dsig) {
+    if(!(options & CL_DB_UNSIGNED) && !dsig) {
 	cli_errmsg("cli_loadinfo: Digital signature not found\n");
 	return CL_EMALFDB;
     }
@@ -1880,23 +1919,44 @@ static int cli_loadign(FILE *fs, struct cl_engine *engine, unsigned int options,
 #define MD5_TOKENS 5
 static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo, unsigned int mode, unsigned int options, struct cli_dbio *dbio, const char *dbname)
 {
-	const char *tokens[MD5_TOKENS + 1];
-	char buffer[FILEBUFF], *buffer_cpy = NULL;
-	const char *pt, *virname;
-	int ret = CL_SUCCESS;
-	unsigned int size_field = 1, md5_field = 0, line = 0, sigs = 0, tokens_count;
-	struct cli_matcher *db = NULL;
-	unsigned long size;
+    const char *tokens[MD5_TOKENS + 1];
+    char buffer[FILEBUFF], *buffer_cpy = NULL;
+    const char *pt, *virname;
+    int ret = CL_SUCCESS;
+    unsigned int size_field = 1, md5_field = 0, line = 0, sigs = 0, tokens_count;
+    unsigned int req_fl = 0; 
+    struct cli_matcher *db;
+    unsigned long size;
 
 
     if(mode == MD5_MDB) {
 	size_field = 0;
 	md5_field = 1;
+	db = engine->hm_mdb;
+    } else if(mode == MD5_HDB)
+	db = engine->hm_hdb;
+    else
+	db = engine->hm_fp;
+
+    if(!db) {
+	if(!(db = mpool_calloc(engine->mempool, 1, sizeof(*db))))
+	    return CL_EMEM;
+#ifdef USE_MPOOL
+	db->mempool = engine->mempool;
+#endif
+	if(mode == MD5_HDB)
+	    engine->hm_hdb = db;
+	else if(mode == MD5_MDB)
+	    engine->hm_mdb = db;
+	else
+	    engine->hm_fp = db;
     }
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loadhash: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -1912,7 +1972,7 @@ static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    break;
 	}
 	if(tokens_count > MD5_TOKENS - 2) {
-	    unsigned int req_fl = atoi(tokens[MD5_TOKENS - 2]);
+	    req_fl = atoi(tokens[MD5_TOKENS - 2]);
 
 	    if(tokens_count > MD5_TOKENS) {
 		ret = CL_EMALFDB;
@@ -1922,17 +1982,28 @@ static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    if(cl_retflevel() < req_fl)
 		continue;
 	    if(tokens_count == MD5_TOKENS) {
-		req_fl = atoi(tokens[MD5_TOKENS - 1]);
-		if(cl_retflevel() > req_fl)
+		int max_fl = atoi(tokens[MD5_TOKENS - 1]);
+		if(cl_retflevel() > max_fl)
 		    continue;
 	    }
 	}
 
-	size = strtol(tokens[size_field], (char **)&pt, 10);
-	if(*pt || !size || size >= 0xffffffff) {
-	    cli_errmsg("cli_loadhash: Invalid value for the size field\n");
-	    ret = CL_EMALFDB;
-	    break;
+	if((mode == MD5_MDB) || strcmp(tokens[size_field],"*")) {
+	    size = strtoul(tokens[size_field], (char **)&pt, 10);
+	    if(*pt || !size || size >= 0xffffffff) {
+		cli_errmsg("cli_loadhash: Invalid value for the size field\n");
+		ret = CL_EMALFDB;
+		break;
+	    }
+	}
+	else {
+	    size = 0;
+	    if((tokens_count < MD5_TOKENS - 1) || (req_fl < 73)) {
+		cli_errmsg("cli_loadhash: Minimum FLEVEL field must be at least 73 for wildcard size hash signatures."
+			" For reference, running FLEVEL is %d\n", cl_retflevel());
+		ret = CL_EMALFDB;
+		break;
+	    }
 	}
 
 	pt = tokens[2]; /* virname */
@@ -1949,7 +2020,7 @@ static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 		dot = dbname;
 	    else
 		dot++;
-	    if(engine->cb_sigload(dot, pt, engine->cb_sigload_ctx)) {
+	    if(engine->cb_sigload(dot, pt, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 		cli_dbgmsg("cli_loadhash: skipping %s (%s) due to callback\n", pt, dot);
 	        continue;
 	    }
@@ -1961,31 +2032,8 @@ static int cli_loadhash(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    break;
 	}
 
-	if(mode == MD5_HDB)
-	    db = engine->hm_hdb;
-	else if(mode == MD5_MDB)
-	    db = engine->hm_mdb;
-	else
-	    db = engine->hm_fp;
-
-	if(!db) {
-	    if(!(db = mpool_calloc(engine->mempool, 1, sizeof(*db)))) {
-		ret = CL_EMEM;
-		break;
-	    }
-#ifdef USE_MPOOL
-	    db->mempool = engine->mempool;
-#endif
-	    if(mode == MD5_HDB)
-		engine->hm_hdb = db;
-	    else if(mode == MD5_MDB)
-		engine->hm_mdb = db;
-	    else
-		engine->hm_fp = db;
-	}
-
-	if((ret = hm_addhash(db, tokens[md5_field], size, virname))) {
-	    cli_errmsg("cli_loadhash: Malformed MD5 string at line %u\n", line);
+	if((ret = hm_addhash_str(db, tokens[md5_field], size, virname))) {
+	    cli_errmsg("cli_loadhash: Malformed hash string at line %u\n", line);
 	    mpool_free(engine->mempool, (void *)virname);
 	    break;
 	}
@@ -2022,8 +2070,10 @@ static int cli_loadmd(FILE *fs, struct cl_engine *engine, unsigned int *signo, i
 
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loadmd: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -2091,7 +2141,7 @@ static int cli_loadmd(FILE *fs, struct cl_engine *engine, unsigned int *signo, i
 	    continue;
 	}
 
-	if(engine->cb_sigload && engine->cb_sigload("md", new->virname, engine->cb_sigload_ctx)) {
+	if(engine->cb_sigload && engine->cb_sigload("md", new->virname, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	    cli_dbgmsg("cli_loadmd: skipping %s due to callback\n", new->virname);
 	    mpool_free(engine->mempool, new->virname);
 	    mpool_free(engine->mempool, new);
@@ -2175,8 +2225,10 @@ static int cli_loadcdb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 
 
     if(engine->ignored)
-	if(!(buffer_cpy = cli_malloc(FILEBUFF)))
+	if(!(buffer_cpy = cli_malloc(FILEBUFF))) {
+        cli_errmsg("cli_loadcdb: Can't allocate memory for buffer_cpy\n");
 	    return CL_EMEM;
+    }
 
     while(cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
 	line++;
@@ -2231,7 +2283,7 @@ static int cli_loadcdb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
 	    continue;
 	}
 
-	if(engine->cb_sigload && engine->cb_sigload("cdb", new->virname, engine->cb_sigload_ctx)) {
+	if(engine->cb_sigload && engine->cb_sigload("cdb", new->virname, ~options & CL_DB_OFFICIAL, engine->cb_sigload_ctx)) {
 	    cli_dbgmsg("cli_loadcdb: skipping %s due to callback\n", new->virname);
 	    mpool_free(engine->mempool, new->virname);
 	    mpool_free(engine->mempool, new);
@@ -2340,6 +2392,195 @@ static int cli_loadcdb(FILE *fs, struct cl_engine *engine, unsigned int *signo,
     return CL_SUCCESS;
 }
 
+/* 
+ * name;trusted;subject;serial;pubkey;exp;codesign;timesign;certsign;notbefore;comment[;minFL[;maxFL]]
+ * Name and comment are ignored. They're just for the end user.
+ * Exponent is ignored for now and hardcoded to \x01\x00\x01.
+ */
+#define CRT_TOKENS 13
+static int cli_loadcrt(FILE *fs, struct cl_engine *engine, struct cli_dbio *dbio) {
+    char buffer[FILEBUFF];
+    char *tokens[CRT_TOKENS+1];
+    size_t line=0, tokens_count, i, j;
+    cli_crt ca;
+    int ret=CL_SUCCESS;
+    char *subject=NULL, *pubkey=NULL, *exponent=NULL, *serial=NULL;
+    const uint8_t exp[] = "\x01\x00\x01";
+    char c;
+
+    cli_crt_init(&ca);
+    memset(ca.issuer, 0xca, sizeof(ca.issuer));
+
+    while (cli_dbgets(buffer, FILEBUFF, fs, dbio)) {
+        line++;
+
+        if (buffer[0] == '#')
+            continue;
+
+        cli_chomp(buffer);
+        if (!strlen(buffer))
+            continue;
+
+        tokens_count = cli_strtokenize(buffer, ';', CRT_TOKENS + 1, (const char **)tokens);
+        if (tokens_count > CRT_TOKENS || tokens_count < CRT_TOKENS - 2) {
+            cli_errmsg("cli_loadcrt: line %u: Invalid number of tokens: %u\n", (unsigned int)line, (unsigned int)tokens_count);
+            ret = CL_EMALFDB;
+            goto end;
+        }
+
+        if (tokens_count > CRT_TOKENS - 2) {
+            if (!cli_isnumber(tokens[CRT_TOKENS-1])) {
+                cli_errmsg("cli_loadcrt: line %u: Invalid minimum feature level\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+            }
+            if ((unsigned int)atoi(tokens[CRT_TOKENS-1]) > cl_retflevel()) {
+                cli_dbgmsg("cli_loadcrt: Cert %s not loaded (required f-level: %u)\n", tokens[0], cl_retflevel());
+                continue;
+            }
+
+            if (tokens_count == CRT_TOKENS) {
+                if (!cli_isnumber(tokens[CRT_TOKENS])) {
+                    cli_errmsg("cli_loadcrt: line %u: Invalid maximum feature level\n", (unsigned int)line);
+                    ret = CL_EMALFDB;
+                    goto end;
+                }
+
+                if ((unsigned int)atoi(tokens[CRT_TOKENS]) < cl_retflevel()) {
+                    cli_dbgmsg("cli_ladcrt: Cert %s not loaded (maximum f-level: %s)\n", tokens[0], tokens[CRT_TOKENS]);
+                    continue;
+                }
+            }
+        }
+
+        switch (tokens[1][0]) {
+            case '1':
+                ca.isBlacklisted = 0;
+                break;
+            case '0':
+                ca.isBlacklisted = 1;
+                break;
+            default:
+                cli_errmsg("cli_loadcrt: line %u: Invalid trust specification. Expected 0 or 1\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+        }
+
+        subject = cli_hex2str(tokens[2]);
+        if (strlen(tokens[3])) {
+            serial = cli_hex2str(tokens[3]);
+            if (!serial) {
+                cli_errmsg("cli_loadcrt: line %u: Cannot convert serial to binary string\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+            }
+            memcpy(ca.serial, serial, sizeof(ca.serial));
+            free(serial);
+        } else {
+            memset(ca.serial, 0xca, sizeof(ca.serial));
+        }
+        pubkey = cli_hex2str(tokens[4]);
+        cli_dbgmsg("cli_loadcrt: subject: %s\n", tokens[2]);
+        cli_dbgmsg("cli_loadcrt: public key: %s\n", tokens[4]);
+
+        if (!subject) {
+            cli_errmsg("cli_loadcrt: line %u: Cannot convert subject to binary string\n", (unsigned int)line);
+            ret = CL_EMALFDB;
+            goto end;
+        }
+        if (!pubkey) {
+            cli_errmsg("cli_loadcrt: line %u: Cannot convert public key to binary string\n", (unsigned int)line);
+            ret = CL_EMALFDB;
+            goto end;
+        }
+
+        memcpy(ca.subject, subject, sizeof(ca.subject));
+        if (mp_read_unsigned_bin(&(ca.n), pubkey, strlen(tokens[4])/2) || mp_read_unsigned_bin(&(ca.e), exp, sizeof(exp)-1)) {
+            cli_errmsg("cli_loadcrt: line %u: Cannot convert exponent to binary data\n", (unsigned int)line);
+            ret = CL_EMALFDB;
+            goto end;
+        }
+
+        switch (tokens[6][0]) {
+            case '1':
+                ca.codeSign = 1;
+                break;
+            case '0':
+                ca.codeSign = 0;
+                break;
+            default:
+                cli_errmsg("cli_loadcrt: line %u: Invalid code sign specification. Expected 0 or 1\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+        }
+
+        switch (tokens[7][0]) {
+            case '1':
+                ca.timeSign = 1;
+                break;
+            case '0':
+                ca.timeSign = 0;
+                break;
+            default:
+                cli_errmsg("cli_loadcrt: line %u: Invalid time sign specification. Expected 0 or 1\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+        }
+
+        switch (tokens[8][0]) {
+            case '1':
+                ca.certSign = 1;
+                break;
+            case '0':
+                ca.certSign = 0;
+                break;
+            default:
+                cli_errmsg("cli_loadcrt: line %u: Invalid cert sign specification. Expected 0 or 1\n", (unsigned int)line);
+                ret = CL_EMALFDB;
+                goto end;
+        }
+
+        if (strlen(tokens[0]))
+            ca.name = tokens[0];
+        else
+            ca.name = NULL;
+
+        if (strlen(tokens[9]))
+            ca.not_before = atoi(tokens[8]);
+        ca.not_after = (-1U)>>1;
+
+        ca.hashtype = CLI_SHA1RSA;
+        crtmgr_add(&(engine->cmgr), &ca);
+        free(subject);
+        free(pubkey);
+        subject = pubkey = NULL;
+    }
+
+end:
+    if (subject)
+        free(subject);
+    if (pubkey)
+        free(pubkey);
+
+    cli_dbgmsg("Number of certs: %d\n", engine->cmgr.items);
+    cli_crt_clear(&ca);
+    return ret;
+}
+
+static int cli_loadmscat(FILE *fs, const char *dbname, struct cl_engine *engine, unsigned int options, struct cli_dbio *dbio) {
+    fmap_t *map;
+
+    if(!(map = fmap(fileno(fs), 0, 0))) {
+	cli_dbgmsg("Can't map cat: %s\n", dbname);
+	return 0;
+    }
+
+    if(asn1_load_mscat(map, engine))
+	cli_dbgmsg("Failed to load certificates from cat: %s\n", dbname);
+    funmap(map);
+    return 0;
+}
+
 static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned int *signo, unsigned int options);
 
 int cli_load(const char *filename, struct cl_engine *engine, unsigned int *signo, unsigned int options, struct cli_dbio *dbio)
@@ -2383,6 +2624,12 @@ int cli_load(const char *filename, struct cl_engine *engine, unsigned int *signo
     } else if(cli_strbcasestr(dbname, ".cld")) {
 	ret = cli_cvdload(fs, engine, signo, options, 1, filename, 0);
 
+    } else if(cli_strbcasestr(dbname, ".cud")) {
+	ret = cli_cvdload(fs, engine, signo, options, 2, filename, 0);
+
+    } else if (cli_strbcasestr(dbname, ".crb")) {
+        ret = cli_loadcrt(fs, engine, dbio);
+
     } else if(cli_strbcasestr(dbname, ".hdb") || cli_strbcasestr(dbname, ".hsb")) {
 	ret = cli_loadhash(fs, engine, signo, MD5_HDB, options, dbio, dbname);
     } else if(cli_strbcasestr(dbname, ".hdu") || cli_strbcasestr(dbname, ".hsu")) {
@@ -2460,10 +2707,12 @@ int cli_load(const char *filename, struct cl_engine *engine, unsigned int *signo
 
     } else if(cli_strbcasestr(dbname, ".cdb")) {
     	ret = cli_loadcdb(fs, engine, signo, options, dbio);
+    } else if(cli_strbcasestr(dbname, ".cat")) {
+	ret = cli_loadmscat(fs, dbname, engine, options, dbio);
     } else {
 	cli_dbgmsg("cli_load: unknown extension - assuming old database format\n");
 	ret = cli_loaddb(fs, engine, signo, options, dbio, dbname);
-    }
+    } 
 
     if(ret) {
 	cli_errmsg("Can't load %s: %s\n", filename, cl_strerror(ret));
@@ -2515,14 +2764,14 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 	    if(cli_strbcasestr(dent->d_name, ".ign") || cli_strbcasestr(dent->d_name, ".ign2")) {
 		dbfile = (char *) cli_malloc(strlen(dent->d_name) + strlen(dirname) + 2);
 		if(!dbfile) {
-		    cli_dbgmsg("cli_loaddbdir(): dbfile == NULL\n");
+		    cli_errmsg("cli_loaddbdir(): dbfile == NULL\n");
 		    closedir(dd);
 		    return CL_EMEM;
 		}
 		sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
 		ret = cli_load(dbfile, engine, signo, options, NULL);
 		if(ret) {
-		    cli_dbgmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
+		    cli_errmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
 		    free(dbfile);
 		    closedir(dd);
 		    return ret;
@@ -2536,6 +2785,7 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
     dbfile = (char *) cli_malloc(strlen(dirname) + 20);
     if(!dbfile) {
 	closedir(dd);
+    cli_errmsg("cli_loaddbdir: Can't allocate memory for dbfile\n");
 	return CL_EMEM;
     }
 
@@ -2613,14 +2863,14 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 
 		dbfile = (char *) cli_malloc(strlen(dent->d_name) + strlen(dirname) + 2);
 		if(!dbfile) {
-		    cli_dbgmsg("cli_loaddbdir(): dbfile == NULL\n");
+		    cli_errmsg("cli_loaddbdir(): dbfile == NULL\n");
 		    closedir(dd);
 		    return CL_EMEM;
 		}
 		sprintf(dbfile, "%s"PATHSEP"%s", dirname, dent->d_name);
 		ret = cli_load(dbfile, engine, signo, options, NULL);
 		if(ret) {
-		    cli_dbgmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
+		    cli_errmsg("cli_loaddbdir(): error loading database %s\n", dbfile);
 		    free(dbfile);
 		    closedir(dd);
 		    return ret;
@@ -2638,7 +2888,7 @@ static int cli_loaddbdir(const char *dirname, struct cl_engine *engine, unsigned
 
 int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, unsigned int dboptions)
 {
-	struct stat sb;
+	STATBUF sb;
 	int ret;
 
     if(!engine) {
@@ -2651,7 +2901,7 @@ int cl_load(const char *path, struct cl_engine *engine, unsigned int *signo, uns
 	return CL_EARG;
     }
 
-    if(stat(path, &sb) == -1) {
+    if(STAT(path, &sb) == -1) {
         cli_errmsg("cl_load(): Can't get status of %s\n", path);
         return CL_ESTAT;
     }
@@ -2735,7 +2985,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 	{
 	    if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..") && CLI_DBEXT(dent->d_name)) {
 		dbstat->entries++;
-		dbstat->stattab = (struct stat *) cli_realloc2(dbstat->stattab, dbstat->entries * sizeof(struct stat));
+		dbstat->stattab = (STATBUF *) cli_realloc2(dbstat->stattab, dbstat->entries * sizeof(STATBUF));
 		if(!dbstat->stattab) {
 		    cl_statfree(dbstat);
 		    closedir(dd);
@@ -2745,6 +2995,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 #ifdef _WIN32
 		dbstat->statdname = (char **) cli_realloc2(dbstat->statdname, dbstat->entries * sizeof(char *));
 		if(!dbstat->statdname) {
+            cli_errmsg("cl_statinidir: Can't allocate memory for dbstat->statdname\n");
 		    cl_statfree(dbstat);
 		    closedir(dd);
 		    return CL_EMEM;
@@ -2753,6 +3004,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 
                 fname = cli_malloc(strlen(dirname) + strlen(dent->d_name) + 32);
 		if(!fname) {
+            cli_errmsg("cl_statinidir: Cant' allocate memory for fname\n");
 		    cl_statfree(dbstat);
 		    closedir(dd);
 		    return CL_EMEM;
@@ -2761,6 +3013,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 #ifdef _WIN32
 		dbstat->statdname[dbstat->entries - 1] = (char *) cli_malloc(strlen(dent->d_name) + 1);
 		if(!dbstat->statdname[dbstat->entries - 1]) {
+            cli_errmsg("cli_statinidir: Can't allocate memory for dbstat->statdname\n");
 		    cl_statfree(dbstat);
 		    closedir(dd);
 		    return CL_EMEM;
@@ -2768,7 +3021,7 @@ int cl_statinidir(const char *dirname, struct cl_stat *dbstat)
 
 		strcpy(dbstat->statdname[dbstat->entries - 1], dent->d_name);
 #endif
-		stat(fname, &dbstat->stattab[dbstat->entries - 1]);
+		STAT(fname, &dbstat->stattab[dbstat->entries - 1]);
 		free(fname);
 	    }
 	}
@@ -2788,7 +3041,7 @@ int cl_statchkdir(const struct cl_stat *dbstat)
 	    char b[offsetof(struct dirent, d_name) + NAME_MAX + 1];
 	} result;
 #endif
-	struct stat sb;
+	STATBUF sb;
 	unsigned int i, found;
 	char *fname;
 
@@ -2817,12 +3070,13 @@ int cl_statchkdir(const struct cl_stat *dbstat)
 	    if(strcmp(dent->d_name, ".") && strcmp(dent->d_name, "..") && CLI_DBEXT(dent->d_name)) {
                 fname = cli_malloc(strlen(dbstat->dir) + strlen(dent->d_name) + 32);
 		if(!fname) {
+            cli_errmsg("cl_statchkdir: can't allocate memory for fname\n");
 		    closedir(dd);
 		    return CL_EMEM;
 		}
 
 		sprintf(fname, "%s"PATHSEP"%s", dbstat->dir, dent->d_name);
-		stat(fname, &sb);
+		STAT(fname, &sb);
 		free(fname);
 
 		found = 0;
@@ -2951,6 +3205,8 @@ int cl_engine_free(struct cl_engine *engine)
 	mpool_free(engine->mempool, root);
     }
 
+    crtmgr_free(&engine->cmgr);
+
     while(engine->cdb) {
 	struct cli_cdb *pt = engine->cdb;
 	engine->cdb = pt->next;
@@ -3143,7 +3399,7 @@ static int countsigs(const char *dbname, unsigned int options, unsigned int *sig
 	if(options & CL_COUNTSIGS_UNOFFICIAL)
 	    (*sigs)++;
 
-    } else if(cli_strbcasestr(dbname, ".wdb") || cli_strbcasestr(dbname, ".fp") || cli_strbcasestr(dbname, ".ftm") || cli_strbcasestr(dbname, ".cfg")) {
+    } else if(cli_strbcasestr(dbname, ".wdb") || cli_strbcasestr(dbname, ".fp") || cli_strbcasestr(dbname, ".ftm") || cli_strbcasestr(dbname, ".cfg") || cli_strbcasestr(dbname, ".cat")) {
 	/* ignore */
 
     } else if((options & CL_COUNTSIGS_UNOFFICIAL) && CLI_DBEXT(dbname)) {
@@ -3155,7 +3411,7 @@ static int countsigs(const char *dbname, unsigned int options, unsigned int *sig
 
 int cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs)
 {
-	struct stat sb;
+	STATBUF sb;
 	char fname[1024];
 	struct dirent *dent;
 #if defined(HAVE_READDIR_R_3) || defined(HAVE_READDIR_R_2)
@@ -3170,7 +3426,7 @@ int cl_countsigs(const char *path, unsigned int countoptions, unsigned int *sigs
     if(!sigs)
 	return CL_ENULLARG;
 
-    if(stat(path, &sb) == -1) {
+    if(STAT(path, &sb) == -1) {
 	cli_errmsg("cl_countsigs: Can't stat %s\n", path);
 	return CL_ESTAT;
     }
