@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2007 - 2008 Sourcefire, Inc.
+ *  Copyright (C) 2007 - 2013 Sourcefire, Inc.
  *  Copyright (C) 2002 - 2007 Tomasz Kojm <tkojm@clamav.net>
  *  CDIFF code (C) 2006 Sensory Networks, Inc.
  *  Author: Tomasz Kojm <tkojm@clamav.net>
@@ -24,6 +24,8 @@
 #include "clamav-config.h"
 #endif
 
+#define _GNU_SOURCE
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -57,6 +59,7 @@
 #include "shared/optparser.h"
 #include "shared/misc.h"
 #include "shared/cdiff.h"
+#include "libclamav/sha1.h"
 #include "libclamav/sha256.h"
 #include "shared/tar.h"
 
@@ -70,9 +73,14 @@
 #include "libclamav/fmap.h"
 #include "libclamav/readdb.h"
 #include "libclamav/others.h"
+#include "libclamav/pe.h"
 
 #define MAX_DEL_LOOKAHEAD   5000
 
+//struct s_info info;
+short recursion = 0, bell = 0;
+short printinfected = 0, printclean = 1;
+
 static const struct dblist_s {
     const char *ext;
     unsigned int count;
@@ -117,7 +125,7 @@ static char *getdbname(const char *str, char *dst, int dstlen)
 {
 	int len = strlen(str);
 
-    if(cli_strbcasestr(str, ".cvd") || cli_strbcasestr(str, ".cld"))
+    if(cli_strbcasestr(str, ".cvd") || cli_strbcasestr(str, ".cld") || cli_strbcasestr(str, ".cud"))
 	len -= 4;
 
     if(dst) {
@@ -159,12 +167,12 @@ static int hashsig(const struct optstruct *opts, unsigned int mdb, int type)
 {
 	char *hash;
 	unsigned int i;
-	struct stat sb;
+	STATBUF sb;
 
 
     if(opts->filename) {
 	for(i = 0; opts->filename[i]; i++) {
-	    if(stat(opts->filename[i], &sb) == -1) {
+	    if(STAT(opts->filename[i], &sb) == -1) {
 		mprintf("!hashsig: Can't access file %s\n", opts->filename[i]);
 		perror("hashsig");
 		return -1;
@@ -278,6 +286,7 @@ static char *getdsig(const char *host, const char *user, const unsigned char *da
 	struct termios old, new;
 #endif
 
+    memset(&server, 0x00, sizeof(struct sockaddr_in));
 
     if((pt = getenv("SIGNDPASS"))) {
 	strncpy(pass, pt, sizeof(pass));
@@ -362,7 +371,8 @@ static char *getdsig(const char *host, const char *user, const unsigned char *da
     memset(pass, 0, sizeof(pass));
     memset(buff, 0, sizeof(buff));
 
-    if((bread = recv(sockd, buff, sizeof(buff), 0)) > 0) {
+    if((bread = recv(sockd, buff, sizeof(buff)-1, 0)) > 0) {
+	buff[bread] = '\0';
 	if(!strstr(buff, "Signature:")) {
 	    mprintf("!getdsig: Error generating digital signature\n");
 	    mprintf("!getdsig: Answer from remote server: %s\n", buff);
@@ -408,9 +418,14 @@ static char *sha256file(const char *file, unsigned int *size)
     sha256_final(&ctx, digest);
     sha = (char *) malloc(65);
     if(!sha)
+    {
+        fclose(fh);
 	return NULL;
+    }	
     for(i = 0; i < 32; i++)
 	sprintf(sha + i * 2, "%02x", digest[i]);
+
+    fclose(fh);	
     return sha;
 }
 
@@ -475,18 +490,20 @@ static int writeinfo(const char *dbname, const char *builder, const char *header
 	    }
 	}
     }
-    rewind(fh);
-    sha256_init(&ctx);
-    while((bytes = fread(buffer, 1, sizeof(buffer), fh)))
-	sha256_update(&ctx, buffer, bytes);
-    sha256_final(&ctx, digest);
-    if(!(pt = getdsig(optget(opts, "server")->strarg, builder, digest, 32, 3))) {
-	mprintf("!writeinfo: Can't get digital signature from remote server\n");
-	fclose(fh);
-	return -1;
+    if(!optget(opts, "unsigned")->enabled) {
+	rewind(fh);
+	sha256_init(&ctx);
+	while((bytes = fread(buffer, 1, sizeof(buffer), fh)))
+	    sha256_update(&ctx, buffer, bytes);
+	sha256_final(&ctx, digest);
+	if(!(pt = getdsig(optget(opts, "server")->strarg, builder, digest, 32, 3))) {
+	    mprintf("!writeinfo: Can't get digital signature from remote server\n");
+	    fclose(fh);
+	    return -1;
+	}
+	fprintf(fh, "DSIG:%s\n", pt);
+	free(pt);
     }
-    fprintf(fh, "DSIG:%s\n", pt);
-    free(pt);
     fclose(fh);
     return 0;
 }
@@ -499,14 +516,14 @@ static int script2cdiff(const char *script, const char *builder, const struct op
 	char *cdiff, *pt, buffer[FILEBUFF];
 	unsigned char digest[32];
 	SHA256_CTX ctx;
-	struct stat sb;
+	STATBUF sb;
 	FILE *scripth, *cdiffh;
 	gzFile gzh;
 	unsigned int ver, osize;
 	int bytes;
 
 
-    if(stat(script, &sb) == -1) {
+    if(STAT(script, &sb) == -1) {
 	mprintf("!script2diff: Can't stat file %s\n", script);
 	return -1;
     }
@@ -623,7 +640,7 @@ static int build(const struct optstruct *opts)
 	int ret, bc = 0;
 	size_t bytes;
 	unsigned int i, sigs = 0, oldsigs = 0, entries = 0, version, real_header, fl;
-	struct stat foo;
+	STATBUF foo;
 	unsigned char buffer[FILEBUFF];
 	char *tarfile, header[513], smbuff[32], builder[32], *pt, olddb[512];
 	char patch[32], broken[32], dbname[32], dbfile[32];
@@ -644,7 +661,7 @@ static int build(const struct optstruct *opts)
 	free(x[i]);		    \
     free(x);
 
-    if(!optget(opts, "server")->enabled) {
+    if(!optget(opts, "server")->enabled && !optget(opts, "unsigned")->enabled) {
 	mprintf("!build: --server is required for --build\n");
 	return -1;
     }
@@ -652,7 +669,7 @@ static int build(const struct optstruct *opts)
     if(optget(opts, "datadir")->active)
 	localdbdir = optget(opts, "datadir")->strarg;
 
-    if(stat("COPYING", &foo) == -1) {
+    if(STAT("COPYING", &foo) == -1) {
 	mprintf("!build: COPYING file not found in current working directory.\n");
 	return -1;
     }
@@ -701,7 +718,9 @@ static int build(const struct optstruct *opts)
 	    }
 	    closedir(dd);
 	    entries += dblist2cnt;
-	    qsort(dblist2, dblist2cnt, sizeof(char *), qcompare);
+	    if(dblist2 != NULL) {
+		qsort(dblist2, dblist2cnt, sizeof(char *), qcompare);
+	    }
 
 	    if(!access("last.hdb", R_OK)) {
 		if(!dblist2cnt) {
@@ -742,11 +761,11 @@ static int build(const struct optstruct *opts)
 
     /* try to read cvd header of current database */
     if(opts->filename) {
-	if(cli_strbcasestr(opts->filename[0], ".cvd") || cli_strbcasestr(opts->filename[0], ".cld")) {
+	if(cli_strbcasestr(opts->filename[0], ".cvd") || cli_strbcasestr(opts->filename[0], ".cld") || cli_strbcasestr(opts->filename[0], ".cud")) {
 	    strncpy(olddb, opts->filename[0], sizeof(olddb));
 	    olddb[sizeof(olddb)-1]='\0';
 	} else {
-	    mprintf("!build: Not a CVD/CLD file\n");
+	    mprintf("!build: Not a CVD/CLD/CUD file\n");
 	    FREE_LS(dblist2);
 	    return -1;
 	}
@@ -756,10 +775,12 @@ static int build(const struct optstruct *opts)
 	snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cvd", localdbdir ? localdbdir : pt, dbname);
 	if(access(olddb, R_OK))
 	    snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cld", localdbdir ? localdbdir : pt, dbname);
+	if(access(olddb, R_OK))
+	    snprintf(olddb, sizeof(olddb), "%s"PATHSEP"%s.cud", localdbdir ? localdbdir : pt, dbname);
 	free(pt);
     }
 
-    if(!(oldcvd = cl_cvdhead(olddb))) {
+    if(!(oldcvd = cl_cvdhead(olddb)) && !optget(opts, "unsigned")->enabled) {
 	mprintf("^build: CAN'T READ CVD HEADER OF CURRENT DATABASE %s (wait 3 s)\n", olddb);
 	sleep(3);
     }
@@ -917,15 +938,19 @@ static int build(const struct optstruct *opts)
     sprintf(header + strlen(header), "%s:", pt);
     free(pt);
 
-    if(!(pt = getdsig(optget(opts, "server")->strarg, builder, buffer, 16, 1))) {
-	mprintf("!build: Can't get digital signature from remote server\n");
-	fclose(fh);
-	unlink(tarfile);
-	free(tarfile);
-	return -1;
+    if(!optget(opts, "unsigned")->enabled) {
+	if(!(pt = getdsig(optget(opts, "server")->strarg, builder, buffer, 16, 1))) {
+	    mprintf("!build: Can't get digital signature from remote server\n");
+	    fclose(fh);
+	    unlink(tarfile);
+	    free(tarfile);
+	    return -1;
+	}
+	sprintf(header + strlen(header), "%s:", pt);
+	free(pt);
+    } else {
+	sprintf(header + strlen(header), "X:");
     }
-    sprintf(header + strlen(header), "%s:", pt);
-    free(pt);
 
     /* add builder */
     strcat(header, builder);
@@ -983,6 +1008,9 @@ static int build(const struct optstruct *opts)
 
     mprintf("Created %s\n", newcvd);
 
+    if(optget(opts, "unsigned")->enabled)
+	return 0;
+
     if(!oldcvd || optget(opts, "no-cdiff")->enabled) {
 	mprintf("Skipping .cdiff creation\n");
 	return 0;
@@ -1116,6 +1144,10 @@ static int cvdinfo(const struct optstruct *opts)
     mprintf("File: %s\n", pt);
 
     pt = strchr(cvd->time, '-');
+    if(!pt){
+        cl_cvdfree(cvd);
+        return -1;
+    }
     *pt = ':';
     mprintf("Build time: %s\n", cvd->time);
     mprintf("Version: %u\n", cvd->version);
@@ -1129,11 +1161,14 @@ static int cvdinfo(const struct optstruct *opts)
 	mprintf("Digital signature: %s\n", cvd->dsig);
     }
     cl_cvdfree(cvd);
-    if((ret = cl_cvdverify(pt))) {
+    if(cli_strbcasestr(pt, ".cud"))
+	mprintf("Verification: Unsigned container\n");
+    else if((ret = cl_cvdverify(pt))) {
 	mprintf("!cvdinfo: Verification: %s\n", cl_strerror(ret));
 	return -1;
-    }
-    mprintf("Verification OK.\n");
+    } else
+	mprintf("Verification OK.\n");
+
     return 0;
 }
 
@@ -1173,7 +1208,8 @@ static int listdir(const char *dirname, const regex_t *regex)
 	     cli_strbcasestr(dent->d_name, ".cdb") ||
 	     cli_strbcasestr(dent->d_name, ".cbc") ||
 	     cli_strbcasestr(dent->d_name, ".cld") ||
-	     cli_strbcasestr(dent->d_name, ".cvd"))) {
+	     cli_strbcasestr(dent->d_name, ".cvd") ||  
+	     cli_strbcasestr(dent->d_name, ".crb"))) {
 
 		dbfile = (char *) malloc(strlen(dent->d_name) + strlen(dirname) + 2);
 		if(!dbfile) {
@@ -1297,7 +1333,23 @@ static int listdb(const char *filename, const regex_t *regex)
 	    mprintf("%s\n", start);
 	}
 
-    } else if(cli_strbcasestr(filename, ".hdb") || cli_strbcasestr(filename, ".hdu") || cli_strbcasestr(filename, ".mdb") || cli_strbcasestr(filename, ".mdu")) { /* hash database */
+    } else if (cli_strbcasestr(filename, ".crb")) {
+        while (fgets(buffer, FILEBUFF, fh)) {
+            cli_chomp(buffer);
+            
+            if (buffer[0] == '#')
+                continue;
+
+            if (regex) {
+                if (!cli_regexec(regex, buffer, 0 , NULL, 0))
+                    mprintf("[%s] %s\n", dbname, buffer);
+
+                continue;
+            }
+            line++;
+            mprintf("%s\n", buffer);
+        }
+    } else if(cli_strbcasestr(filename, ".hdb") || cli_strbcasestr(filename, ".hdu") || cli_strbcasestr(filename, ".mdb") || cli_strbcasestr(filename, ".mdu") || cli_strbcasestr(filename, ".hsb") || cli_strbcasestr(filename, ".hsu") || cli_strbcasestr(filename, ".msb") || cli_strbcasestr(filename, ".msu")) { /* hash database */
 
 	while(fgets(buffer, FILEBUFF, fh)) {
 	    cli_chomp(buffer);
@@ -1381,7 +1433,7 @@ static int listsigs(const struct optstruct *opts, int mode)
 	int ret;
 	const char *name;
 	char *dbdir;
-	struct stat sb;
+	STATBUF sb;
 	regex_t reg;
 	const char *localdbdir = NULL;
 
@@ -1392,7 +1444,7 @@ static int listsigs(const struct optstruct *opts, int mode)
 	name = optget(opts, "list-sigs")->strarg;
 	if(access(name, R_OK) && localdbdir)
 	    name = localdbdir;
-	if(stat(name, &sb) == -1) {
+	if(STAT(name, &sb) == -1) {
 	    mprintf("--list-sigs: Can't get status of %s\n", name);
 	    return -1;
 	}
@@ -1596,9 +1648,11 @@ static int maxlinelen(const char *file)
 
     if(bytes == -1) {
 	mprintf("!maxlinelen: Can't read file %s\n", file);
+	close(fd);
 	return -1;
     }
-
+    
+    close(fd);
     return nmax + 1;
 }
 
@@ -1766,15 +1820,24 @@ static int compare(const char *oldpath, const char *newpath, FILE *diff)
     return 0;
 }
 
+static int compareone(const struct optstruct *opts)
+{
+    if(!opts->filename) {
+	mprintf("!makediff: --compare requires two arguments\n");
+	return -1;
+    }
+    return compare(optget(opts,"compare")->strarg, opts->filename[0], stdout);
+}
+
 static int dircopy(const char *src, const char *dest)
 {
 	DIR *dd;
 	struct dirent *dent;
-	struct stat sb;
+	STATBUF sb;
 	char spath[512], dpath[512];
 
 
-    if(stat(dest, &sb) == -1) {
+    if(STAT(dest, &sb) == -1) {
 	if(mkdir(dest, 0755)) {
 	    /* mprintf("!dircopy: Can't create temporary directory %s\n", dest); */
 	    return -1;
@@ -1906,7 +1969,7 @@ static void matchsig(const char *sig, const char *offset, int fd)
 {
 	struct cl_engine *engine;
         struct cli_ac_result *acres = NULL, *res;
-	struct stat sb;
+	STATBUF sb;
 	unsigned int matches = 0;
 	cli_ctx ctx;
 	int ret;
@@ -1945,7 +2008,7 @@ static void matchsig(const char *sig, const char *offset, int fd)
 	return;
     }
     lseek(fd, 0, SEEK_SET);
-    fstat(fd, &sb);
+    FSTAT(fd, &sb);
     if(!(*ctx.fmap = fmap(fd, 0, sb.st_size))) {
 	free(ctx.fmap);
 	cl_engine_free(engine);
@@ -1993,6 +2056,11 @@ static char *decodehexstr(const char *hex, unsigned int *dlen)
 	    wildcard++;
 
     decoded = calloc(len + 1 + wildcard * 32, sizeof(char));
+    if(!decoded) {
+        free(str16);
+	mprintf("!decodehexstr: Can't allocate memory for decoded\n");
+	return NULL;
+    }
 
     for(i = 0; i < len; i++) {
 	if(str16[i] & CLI_MATCH_WILDCARD) {
@@ -2012,6 +2080,7 @@ static char *decodehexstr(const char *hex, unsigned int *dlen)
 		default:
 		    mprintf("!decodehexstr: Unknown wildcard (0x%x@%u)\n", str16[i] & CLI_MATCH_WILDCARD, i);
 		    free(decoded);
+		    free(str16);
 		    return NULL;
 	    }
 	} else {
@@ -2022,7 +2091,7 @@ static char *decodehexstr(const char *hex, unsigned int *dlen)
 
     if(dlen)
 	*dlen = p;
-
+    free(str16);
     return decoded;
 }
 
@@ -2032,6 +2101,9 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	unsigned int i, len = 0, hlen, negative, altnum, alttype;
 	char *buff;
 
+    
+    hexcpy = NULL;
+    buff = NULL;
 
     hexcpy = strdup(hex);
     if(!hexcpy) {
@@ -2046,6 +2118,7 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	buff = calloc(strlen(hex) + 512, sizeof(char));
 	if(!buff) {
 	    mprintf("!decodehexspecial: Can't allocate memory for buff\n");
+	    free(hexcpy);
 	    return NULL;
 	}
 	start = hexcpy;
@@ -2054,6 +2127,8 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	    *pt++ = 0;
 	    if(!start) {
 		mprintf("!decodehexspecial: Unexpected EOL\n");
+		free(hexcpy);
+		free(buff);
 		return NULL;
 	    }
 	    if(pt >= hexcpy + 2) {
@@ -2065,6 +2140,7 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	    if(!(decoded = decodehexstr(start, &hlen))) {
 		mprintf("!Decoding failed (1): %s\n", pt);
 		free(hexcpy);
+		free(buff);
 		return NULL;
 	    }
 	    memcpy(&buff[len], decoded, hlen);
@@ -2074,6 +2150,7 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	    if(!(start = strchr(pt, ')'))) {
 		mprintf("!decodehexspecial: Missing closing parethesis\n");
 		free(hexcpy);
+		free(buff);
 		return NULL;
 	    }
 
@@ -2081,6 +2158,7 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	    if(!strlen(pt)) {
 		mprintf("!decodehexspecial: Empty block\n");
 		free(hexcpy);
+		free(buff);
 		return NULL;
 	    }
 
@@ -2121,6 +2199,7 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 		if(!altnum) {
 		    mprintf("!decodehexspecial: Empty block\n");
 		    free(hexcpy);
+		    free(buff);
 		    return NULL;
 		}
 		altnum++;
@@ -2142,12 +2221,14 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 		for(i = 0; i < altnum; i++) {
 		    if(!(h = cli_strtok(pt, i, "|"))) {
 			free(hexcpy);
+			free(buff);
 			return NULL;
 		    }
 
 		    if(!(c = cli_hex2str(h))) {
 			free(h);
 			free(hexcpy);
+			free(buff);
 			return NULL;
 		    }
 
@@ -2159,6 +2240,9 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 		    }
 		    if(i + 1 != altnum)
 			buff[len++] = '|';
+
+		    free(h);
+		    free(c);	
 		}
 		buff[len++] = '}';
 	    }
@@ -2167,11 +2251,13 @@ static char *decodehexspecial(const char *hex, unsigned int *dlen)
 	if(start) {
 	    if(!(decoded = decodehexstr(start, &hlen))) {
 		mprintf("!Decoding failed (2)\n");
+		free(buff);
 		free(hexcpy);
 		return NULL;
 	    }
 	    memcpy(&buff[len], decoded, hlen);
 	    len += hlen;
+	    free(decoded);
 	}
     }
     free(hexcpy);
@@ -2311,6 +2397,7 @@ static int decodehex(const char *hexsig)
 	    }
 	    if(!(decoded = decodehexspecial(pt, &dlen))) {
 		mprintf("!Decoding failed\n");
+        free(pt);
 		return -1;
 	    }
 	    bw = write(1, decoded, dlen);
@@ -2701,6 +2788,123 @@ static int makediff(const struct optstruct *opts)
     return 0;
 }
 
+static int dumpcerts(const struct optstruct *opts)
+{
+    char * filename = NULL;
+    STATBUF sb;
+    const char * fmptr;
+    SHA1Context sha1;
+    struct cl_engine *engine;
+    cli_ctx ctx;
+    int fd, ret;
+    uint8_t shash1[SHA1_HASH_SIZE];
+
+    logg_file = NULL;
+
+    filename = optget(opts, "print-certs")->strarg;
+    if(!filename) {
+	mprintf("!dumpcerts: No filename!\n");
+        return -1;
+    }
+
+    /* build engine */
+    if(!(engine = cl_engine_new())) {
+	mprintf("!dumpcerts: Can't create new engine\n");
+	return -1;
+    }
+    cl_engine_set_num(engine, CL_ENGINE_AC_ONLY, 1);
+
+    if(cli_initroots(engine, 0) != CL_SUCCESS) {
+	mprintf("!dumpcerts: cli_initroots() failed\n");
+	cl_engine_free(engine);
+	return -1;
+    }
+
+    if(cli_parse_add(engine->root[0], "test", "deadbeef", 0, 0, "*", 0, NULL, 0) != CL_SUCCESS) {
+	mprintf("!dumpcerts: Can't parse signature\n");
+	cl_engine_free(engine);
+	return -1;
+    }
+
+    if(cl_engine_compile(engine) != CL_SUCCESS) {
+	mprintf("!dumpcerts: Can't compile engine\n");
+	cl_engine_free(engine);
+	return -1;
+    }
+
+    engine->dconf->pe |= PE_CONF_DUMPCERT;
+    cl_debug();
+
+    /* prepare context */
+    memset(&ctx, '\0', sizeof(cli_ctx));
+    ctx.engine = engine;
+    ctx.options = CL_SCAN_STDOPT;
+    ctx.container_type = CL_TYPE_ANY;
+    ctx.dconf = (struct cli_dconf *) engine->dconf;
+    ctx.fmap = calloc(sizeof(fmap_t *), 1);
+    if(!ctx.fmap) {
+	cl_engine_free(engine);
+	return -1;
+    }
+
+    /* Prepare file */
+    fd = open(filename, O_RDONLY);
+    if(fd < 0) {
+	mprintf("!dumpcerts: Can't open file %s!\n", filename);
+        cl_engine_free(engine);
+        return -1;
+    }
+
+    lseek(fd, 0, SEEK_SET);
+    FSTAT(fd, &sb);
+    if(!(*ctx.fmap = fmap(fd, 0, sb.st_size))) {
+	free(ctx.fmap);
+	close(fd);
+	cl_engine_free(engine);
+	return -1;
+    }
+
+    fmptr = fmap_need_off_once(*ctx.fmap, 0, sb.st_size);
+    if(!fmptr) {
+        mprintf("!dumpcerts: fmap_need_off_once failed!\n");
+        free(ctx.fmap);
+        close(fd);
+        cl_engine_free(engine);
+	return -1;
+    }
+
+    /* Generate SHA1 */
+    SHA1Init(&sha1);
+    SHA1Update(&sha1, fmptr, sb.st_size);
+    SHA1Final(&sha1, shash1);
+
+    ret = cli_checkfp_pe(&ctx, shash1);
+    
+    switch(ret) {
+        case CL_CLEAN:
+            mprintf("*dumpcerts: CL_CLEAN after cli_checkfp_pe()!\n");
+            break;
+        case CL_VIRUS:
+            mprintf("*dumpcerts: CL_VIRUS after cli_checkfp_pe()!\n");
+            break;
+        case CL_BREAK:
+            mprintf("*dumpcerts: CL_BREAK after cli_checkfp_pe()!\n");
+            break;
+        case CL_EFORMAT:
+            mprintf("!dumpcerts: Not a valid PE file!\n");
+            break;
+        default:
+            mprintf("!dumpcerts: Other error %d inside cli_checkfp_pe.\n", ret);
+            break;
+    }
+
+    /* Cleanup */
+    free(ctx.fmap);
+    close(fd);
+    cl_engine_free(engine);
+    return 0;
+}
+
 static void help(void)
 {
     mprintf("\n");
@@ -2727,6 +2931,8 @@ static void help(void)
     mprintf("    --info=FILE            -i FILE         print database information\n");
     mprintf("    --build=NAME [cvd] -b NAME             build a CVD file\n");
     mprintf("    --no-cdiff                             Don't generate .cdiff file\n");
+    mprintf("    --unsigned                             Create unsigned database file (.cud)\n");
+    mprintf("    --print-certs=FILE                     Print Authenticode details from a PE\n");
     mprintf("    --server=ADDR                          ClamAV Signing Service address\n");
     mprintf("    --datadir=DIR				Use DIR as default database directory\n");
     mprintf("    --unpack=FILE          -u FILE         Unpack a CVD/CLD file\n");
@@ -2738,6 +2944,7 @@ static void help(void)
     mprintf("    --vba=FILE                             Extract VBA/Word6 macro code\n");
     mprintf("    --vba-hex=FILE                         Extract Word6 macro code with hex values\n");
     mprintf("    --diff=OLD NEW         -d OLD NEW      Create diff for OLD and NEW CVDs\n");
+    mprintf("    --compare=OLD NEW      -c OLD NEW      Show diff between OLD and NEW files in cdiff format\n");
     mprintf("    --run-cdiff=FILE       -r FILE         Execute update script FILE in cwd\n");
     mprintf("    --verify-cdiff=DIFF CVD/CLD            Verify DIFF against CVD/CLD\n");
     mprintf("\n");
@@ -2749,7 +2956,7 @@ int main(int argc, char **argv)
 {
 	int ret;
         struct optstruct *opts;
-	struct stat sb;
+	STATBUF sb;
 
     if(check_flevel())
 	exit(1);
@@ -2821,6 +3028,10 @@ int main(int argc, char **argv)
 	ret = vbadump(opts);
     else if(optget(opts, "diff")->enabled)
 	ret = makediff(opts);
+    else if(optget(opts, "compare")->enabled)
+	ret = compareone(opts);
+    else if(optget(opts, "print-certs")->enabled)
+	ret = dumpcerts(opts);
     else if(optget(opts, "run-cdiff")->enabled)
 	ret = rundiff(opts);
     else if(optget(opts, "verify-cdiff")->enabled) {
@@ -2828,7 +3039,7 @@ int main(int argc, char **argv)
 	    mprintf("!--verify-cdiff requires two arguments\n");
 	    ret = -1;
 	} else {
-	    if(stat(opts->filename[0], &sb) == -1) {
+	    if(STAT(opts->filename[0], &sb) == -1) {
 		mprintf("--verify-cdiff: Can't get status of %s\n", opts->filename[0]);
 		ret = -1;
 	    } else {
