@@ -75,6 +75,8 @@ my %CONF = (
     'HAVE_DL_H' => -1,
     'HAVE_DYLD' => -1,
     'HAVE_ERROR_T' => -1,
+    'HAVE_ENABLE_EXTENDED_FILE_STDIO' => -1,
+    'HAVE_SYS_TIMES_H' => -1,
     'HAVE_FD_PASSING' => -1,
     'HAVE_FSEEKO' => '1',
     'HAVE_GETADDRINFO' => '1',
@@ -181,6 +183,7 @@ my %CONF = (
     'USE_SYSLOG' => -1,
     'VERSION_SUFFIX' => '""',
     'WORDS_BIGENDIAN' => '0',
+    'LT_LIBPREFIX' => '-1',
     '_LARGEFILE_SOURCE' => -1,
     '_POSIX_PII_SOCKET' => -1,
     '_REENTRANT' => '1',
@@ -193,7 +196,8 @@ my %CONF = (
     'restrict' => -1,
     'socklen_t' => -1,
     'HAVE_UNAME_SYSCALL' => -1,
-    'HAVE__INTERNAL__SHA_COLLECT' => -1
+    'HAVE__INTERNAL__SHA_COLLECT' => -1,
+    'FANOTIFY' => -1
     );
 
 
